/*  solfege
    program to edit and play score
    written for wonderwitch environment  */

/* define screen shot enable */
/* #define SHOT_ENABLE */

#include <stdio.h>
#include <sys/bios.h>
#include <sys/libwwc.h>

/* intrrupt and sound definition */

intvector_t intVector;

int vcount_1 = 0; /* envelope volume counter */
int vcount_2 = 0;
int vcount_3 = 0;

unsigned char wave_form[16]={
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff
};

#define    CHANNEL1    0  /* arguments for sound_set_pitch/sound_set_volume */
#define    CHANNEL2    1
#define    CHANNEL3    2
#define    CHANNEL4    3

#define SSC_CH1   (1 << 0)    /* sound_set_channel:Ch1 enable */
#define SSC_CH2   (1 << 1)    /* sound_set_channel:Ch2 enable */
#define SSC_CH3   (1 << 2)    /* sound_set_channel:Ch3 enable */
#define SSC_CH4   (1 << 3)    /* sound_set_channel:Ch4 enable */
#define SSC_VOICE (1 << 5)    /* sound_set_channel:Ch2 voice mode */
#define SSC_SWEEP (1 << 6)    /* sound_set_channel:Ch3 sweep mode */
#define SSC_NOISE (1 << 7)    /* sound_set_channel:Ch4 noise mode */

#define SSO_SPK    (1 << 0)    /* sound_set_output: built-in speeker enable */
#define SSO_EXT    (1 << 3)    /* sound_set_output: external phone nable */
#define SSO_D07    (0 << 1)    /* sound_set_output: D0-D7 output */
#define SSO_D18    (1 << 1)    /* sound_set_output: D1-D8 output */
#define SSO_D29    (2 << 1)    /* sound_set_output: D2-D9 output */
#define SSO_D3A    (3 << 1)    /* sound_set_output: D3-D10 output */


/* font data */
char buffer[32*28*18]; /* 32bytes * screen_w * screen_h */

/* font data is diplayed or not */
int invalidate = 0;

/* diplay position offset */
int position_offset = 0;

/* display scope of x coordination on screen */
int sc_from_x = 0;
int sc_to_x = 224;

/* display scope of y coordination on screen */
int sc_from_y = 0;
int sc_to_y = 144;

int selection_from = 0;
int selection_to = 0;

int menu_selection = 0;

int tempo = 140;

/* edit treble or bass */

#define TREBLE  0
#define BASS    1

int t_or_b = TREBLE;


/* score element class */

#define SCORE_CREF   1
#define SCORE_KEY    2
#define SCORE_RITHM  3
#define SCORE_NOTE   4
#define SCORE_REST   5
#define SCORE_SIG    6    /* key signiture */
#define SCORE_BAR    7    /* bar section mark */


/* crefs */

#define G_CREF  1
#define F_CREF  2

int treble_cref = G_CREF;
int bass_cref = G_CREF;


/* rithm */

#define BINARIA      2    /*  2/2  */
#define TERNARIA     3	  /*  3/4  */
#define QUATERNARIA  4	  /*  4/4  */
#define CROME        6	  /*  6/8  */
#define SEMICROME    9	  /*  9/16 */
#define TWOFOURTH    24   /*  2/4  */
#define TWELVEEIGHTH 128  /*  12/8 */

int treble_rithm = QUATERNARIA;
int bass_rithm = QUATERNARIA;

int treble_rithm_play = QUATERNARIA;
int bass_rithm_play = QUATERNARIA;


/* keys */

#define C_MAJOR 1
#define D_MAJOR 2
#define E_MAJOR 3
#define F_MAJOR 4
#define G_MAJOR 5
#define A_MAJOR 6
#define B_MAJOR 7
#define A_MINOR 8
#define B_MINOR 9
#define C_MINOR 10
#define D_MINOR 11
#define E_MINOR 12
#define F_MINOR 13
#define G_MINOR 14
#define A_FLAT_MAJOR 15
#define B_FLAT_MAJOR 16
#define C_FLAT_MAJOR 17
#define D_FLAT_MAJOR 18
#define E_FLAT_MAJOR 19
#define B_FLAT_MINOR 20


/* key signituras */

#define SIG_SHARP        1
#define SIG_FLAT         2
#define SIG_NATURAL      3
#define SIG_DOUBLE_SHARP 4
#define SIG_DOUBLE_FLAT  5


/* note and rest
   duration tick resolution : 480 = 1/4 */

#define WHOLE             1920
#define DOTTED_WHOLE      2880
#define MINIM              960
#define DOTTED_MINIM      1440
#define CROTCHET           480
#define DOTTED_CROTCHET    720
#define QUAVER             240
#define DOTTED_QUAVER      360
#define SIXTEENTH          120
#define DOTTED_SIXTEENTH   180
#define THIRTY_SECOND       60
#define MINIM_TRIPLET      320
#define CROTCHET_TRIPLET   160


/* bar */

#define BAR_SINGLE 1
#define BAR_DOUBLE 2


/* pitch */

#define PITCH_NONE  0
#define PITCH_C3   36
#define PITCH_D3   38
#define PITCH_E3   40
#define PITCH_F3   41
#define PITCH_G3   43
#define PITCH_A3   45
#define PITCH_B3   47
#define PITCH_C4   48
#define PITCH_D4   50
#define PITCH_E4   52
#define PITCH_F4   53
#define PITCH_G4   55
#define PITCH_A4   57
#define PITCH_B4   59
#define PITCH_C5   60
#define PITCH_D5   62
#define PITCH_E5   64
#define PITCH_F5   65
#define PITCH_G5   67
#define PITCH_A5   69
#define PITCH_B5   71
#define PITCH_C6   72
#define PITCH_D6   74
#define PITCH_E6   76
#define PITCH_F6   77
#define PITCH_G6   79
#define PITCH_A6   81
#define PITCH_B6   83
#define PITCH_C7   84
#define PITCH_D7   86
#define PITCH_E7   88
#define PITCH_F7   89
#define PITCH_G7   91
#define PITCH_A7   93
#define PITCH_B7   95
#define PITCH_C8   96
#define PITCH_D8   98
#define PITCH_E8  100
#define PITCH_F8  101
#define PITCH_G8  103
#define PITCH_A8  105


/* pitch by octave */

#define PITCH_HIGH   1
#define PITCH_MIDDLE 2
#define PITCH_LOW    3

int treble_pitch = PITCH_HIGH;
int bass_pitch = PITCH_LOW;


/* form data */

struct Form {
    int width;
    int height;
    unsigned int *bits;
};


#include "forms.h"


struct Form *char_forms[128];


/* score model
   duration tick resolution : 480 = 1/4
   C=48, #C=49, D=50, ...                */

struct Score_element {
    int category;
    int type;    /* bit9-16 are used to hold last key type in key_c_major */
    int pitch;
    int attributes;  /* bit2: DIRECTION OF NOTE, bit3-4: TIE for SCORE_NOTE */
                     /* pitch for SCORE_CREF */
    int position;
};

#define SCORE_SIZE 810

struct Score_element score_t[SCORE_SIZE];
int no_of_elements_t;
int score_end_position_t;

struct Score_element score_b[SCORE_SIZE];
int no_of_elements_b;
int score_end_position_b;


struct Score_element elements_menu[60] = {
    {SCORE_CREF,  G_CREF,           PITCH_A6,  0x01}, /* attributes .. PITCH_HIGH */
    {SCORE_CREF,  F_CREF,           PITCH_A6,  0x03}, /* attributes .. PITCH_LOW */
    {SCORE_KEY,   G_MAJOR,          PITCH_A6,  0x00},
    {SCORE_KEY,   D_MAJOR,          PITCH_A6,  0x00},
    {SCORE_KEY,   A_MAJOR,          PITCH_A6,  0x00},
    {SCORE_KEY,   E_MAJOR,          PITCH_A6,  0x00}, /*  5 */
    {SCORE_KEY,   C_MAJOR,          PITCH_A6,  0x00},
    {SCORE_KEY,   F_MAJOR,          PITCH_A6,  0x00},
    {SCORE_KEY,   B_FLAT_MAJOR,     PITCH_A6,  0x00},
    {SCORE_KEY,   E_FLAT_MAJOR,     PITCH_A6,  0x00},
    {SCORE_KEY,   A_FLAT_MAJOR,     PITCH_A6,  0x00}, /* 10 */
    {SCORE_KEY,   D_FLAT_MAJOR,     PITCH_A6,  0x00},
    {SCORE_RITHM, BINARIA,          PITCH_A6,  0x00},
    {SCORE_RITHM, TERNARIA,         PITCH_A6,  0x00},
    {SCORE_RITHM, QUATERNARIA,      PITCH_A6,  0x00},
    {SCORE_RITHM, CROME,            PITCH_A6,  0x00}, /* 15 */
    {SCORE_RITHM, TWOFOURTH,        PITCH_A6,  0x00},
    {SCORE_RITHM, TWELVEEIGHTH,     PITCH_A6,  0x00},
    {SCORE_BAR,   BAR_SINGLE,       PITCH_A6,  0x00},
    {SCORE_BAR,   BAR_DOUBLE,       PITCH_A6,  0x00},
    {SCORE_NOTE,  WHOLE,            PITCH_B6,  0x00}, /* 20 */
    {SCORE_NOTE,  DOTTED_MINIM,     PITCH_D7,  0x02},
    {SCORE_NOTE,  DOTTED_MINIM,     PITCH_G6,  0x00},
    {SCORE_NOTE,  MINIM,            PITCH_D7,  0x02}, /* 0x02 : DIRECTION_REVERSED */
    {SCORE_NOTE,  MINIM,            PITCH_G6,  0x00}, /* 0x00 : ~DIRECTION_REVERSED */
    {SCORE_NOTE,  DOTTED_CROTCHET,  PITCH_D7,  0x02},
    {SCORE_NOTE,  DOTTED_CROTCHET,  PITCH_G6,  0x00},
    {SCORE_NOTE,  CROTCHET,         PITCH_D7,  0x02},
    {SCORE_NOTE,  CROTCHET,         PITCH_G6,  0x00},
    {SCORE_NOTE,  DOTTED_QUAVER,    PITCH_D7,  0x02},
    {SCORE_NOTE,  DOTTED_QUAVER,    PITCH_G6,  0x00}, /* 30 */
    {SCORE_NOTE,  QUAVER,           PITCH_D7,  0x02},
    {SCORE_NOTE,  QUAVER,           PITCH_G6,  0x00},
    {SCORE_NOTE,  DOTTED_SIXTEENTH, PITCH_D7,  0x02},
    {SCORE_NOTE,  DOTTED_SIXTEENTH, PITCH_G6,  0x00},
    {SCORE_NOTE,  SIXTEENTH,        PITCH_D7,  0x02}, /* 35 */
    {SCORE_NOTE,  SIXTEENTH,        PITCH_G6,  0x00},
    {SCORE_NOTE,  THIRTY_SECOND,    PITCH_D7,  0x02},
    {SCORE_NOTE,  THIRTY_SECOND,    PITCH_G6,  0x00},
    {SCORE_NOTE,  MINIM_TRIPLET,    PITCH_D7,  0x02},
    {SCORE_NOTE,  MINIM_TRIPLET,    PITCH_G6,  0x00}, /* 40 */
    {SCORE_NOTE,  CROTCHET_TRIPLET, PITCH_D7,  0x02},
    {SCORE_NOTE,  CROTCHET_TRIPLET, PITCH_G6,  0x00},
    {SCORE_REST,  WHOLE,            PITCH_A6,  0x00},
    {SCORE_REST,  DOTTED_MINIM,     PITCH_A6,  0x00},
    {SCORE_REST,  MINIM,            PITCH_A6,  0x00}, /* 45 */
    {SCORE_REST,  DOTTED_CROTCHET,  PITCH_A6,  0x00},
    {SCORE_REST,  CROTCHET,         PITCH_A6,  0x00},
    {SCORE_REST,  DOTTED_QUAVER,    PITCH_A6,  0x00},
    {SCORE_REST,  QUAVER,           PITCH_A6,  0x00},
    {SCORE_REST,  DOTTED_SIXTEENTH, PITCH_A6,  0x00}, /* 50 */
    {SCORE_REST,  SIXTEENTH,        PITCH_A6,  0x00},
    {SCORE_REST,  THIRTY_SECOND,    PITCH_A6,  0x00},
    {SCORE_REST,  MINIM_TRIPLET,    PITCH_A6,  0x00},
    {SCORE_REST,  CROTCHET_TRIPLET, PITCH_A6,  0x00},
    {SCORE_SIG,   SIG_SHARP,        PITCH_A6,  0x00}, /* 55 */
    {SCORE_SIG,   SIG_FLAT,         PITCH_A6,  0x00},
    {SCORE_SIG,   SIG_NATURAL,      PITCH_A6,  0x00},
    {SCORE_SIG,   SIG_DOUBLE_SHARP, PITCH_A6,  0x00},
    {SCORE_SIG,   SIG_DOUBLE_FLAT,  PITCH_A6,  0x00}
};

int elements_menu_selection = 30;

/* attributes */
/* tie & triplet */

#define TIE_SET     (1 << 2)  /* if bit2 in attributes is on, tie is set */
#define TIE_UPPER   (1 << 3)  /* if bit3 in attributes is on, tie is upper, else tie is lower */ 

int tie_display_index = 1;

int tie_play_t = 0;
int tie_play_b = 0;

int triplet_index = 0; /* no of triplet already drawn */
int triplet_is_reversed = 0; /* direction of triplet */
int triplet_indication_ypos;

struct Score_element *former_element;
struct Score_element *far_former_element;

/* direction */

#define DIRECTION_REVERSED (1 << 1)  /* if bit1 in attributes is on, direction of note is reversed */


/* play */

long total_tick = 0;
int tick_switch = 0;
long next_tick_t = 0;
long next_tick_b = 0;

int play_selection_t = 0;
int play_selection_b = 0;

int shown_play_selection_t = 0;
int shown_play_selection_b = 0;

int just_started_t;
int just_started_b;


struct Pitch_table {
    int name;
    int value;
};

const struct Pitch_table natural_pitchs[41] = {
    {PITCH_C3,  580}, /*  0 */
    {PITCH_D3,  740},
    {PITCH_E3,  883},
    {PITCH_F3,  948},
    {PITCH_G3, 1068},
    {PITCH_A3, 1175}, /*  5 */
    {PITCH_B3, 1270},
    {PITCH_C4, 1314},
    {PITCH_D4, 1394},
    {PITCH_E4, 1466},
    {PITCH_F4, 1498}, /* 10 */
    {PITCH_G4, 1558},
    {PITCH_A4, 1612},
    {PITCH_B4, 1659},
    {PITCH_C5, 1681},
    {PITCH_D5, 1721}, /* 15 */
    {PITCH_E5, 1757},
    {PITCH_F5, 1773},
    {PITCH_G5, 1803},
    {PITCH_A5, 1830},
    {PITCH_B5, 1854}, /* 20 */
    {PITCH_C6, 1865},
    {PITCH_D6, 1885},
    {PITCH_E6, 1902},
    {PITCH_F6, 1911},
    {PITCH_G6, 1926}, /* 25 */
    {PITCH_A6, 1939},
    {PITCH_B6, 1951},
    {PITCH_C7, 1956},
    {PITCH_D7, 1966},
    {PITCH_E7, 1975}, /* 30 */
    {PITCH_F7, 1979},
    {PITCH_G7, 1987},
    {PITCH_A7, 1993},
    {PITCH_B7, 1999},
    {PITCH_C8, 2002}, /* 35 */
    {PITCH_D8, 2007},
    {PITCH_E8, 2012},
    {PITCH_F8, 2014},
    {PITCH_G8, 2017},
    {PITCH_A8, 2021}
};

const struct Pitch_table sharp_pitchs[41] = {
    {PITCH_C3,  663}, /*  0 */
    {PITCH_D3,  814},
    {PITCH_E3,  948},
    {PITCH_F3, 1010},
    {PITCH_G3, 1123},
    {PITCH_A3, 1224}, /*  5 */
    {PITCH_B3, 1314},
    {PITCH_C4, 1355},
    {PITCH_D4, 1431},
    {PITCH_E4, 1498},
    {PITCH_F4, 1529}, /* 10 */
    {PITCH_G4, 1586},
    {PITCH_A4, 1636},
    {PITCH_B4, 1681},
    {PITCH_C5, 1702},
    {PITCH_D5, 1739}, /* 15 */
    {PITCH_E5, 1773},
    {PITCH_F5, 1789},
    {PITCH_G5, 1817},
    {PITCH_A5, 1842},
    {PITCH_B5, 1865}, /* 20 */
    {PITCH_C6, 1875},
    {PITCH_D6, 1894},
    {PITCH_E6, 1911},
    {PITCH_F6, 1918},
    {PITCH_G6, 1932}, /* 25 */
    {PITCH_A6, 1945},
    {PITCH_B6, 1956},
    {PITCH_C7, 1961},
    {PITCH_D7, 1971},
    {PITCH_E7, 1979}, /* 30 */
    {PITCH_F7, 1983},
    {PITCH_G7, 1990},
    {PITCH_A7, 1997},
    {PITCH_B7, 2002},
    {PITCH_C8, 2005}, /* 35 */
    {PITCH_D8, 2009},
    {PITCH_E8, 2014},
    {PITCH_F8, 2016},
    {PITCH_G8, 2019},
    {PITCH_A8, 2022}
};

const struct Pitch_table flat_pitchs[41] = {
    {PITCH_C3,  490}, /*  0 */
    {PITCH_D3,  663},
    {PITCH_E3,  814},
    {PITCH_F3,  883},
    {PITCH_G3, 1010},
    {PITCH_A3, 1123}, /*  5 */
    {PITCH_B3, 1224},
    {PITCH_C4, 1270},
    {PITCH_D4, 1355},
    {PITCH_E4, 1431},
    {PITCH_F4, 1465}, /* 10 */
    {PITCH_G4, 1529},
    {PITCH_A4, 1586},
    {PITCH_B4, 1636},
    {PITCH_C5, 1659},
    {PITCH_D5, 1702}, /* 15 */
    {PITCH_E5, 1739},
    {PITCH_F5, 1757},
    {PITCH_G5, 1789},
    {PITCH_A5, 1817},
    {PITCH_B5, 1842}, /* 20 */
    {PITCH_C6, 1854},
    {PITCH_D6, 1875},
    {PITCH_E6, 1894},
    {PITCH_F6, 1902},
    {PITCH_G6, 1918}, /* 25 */
    {PITCH_A6, 1932},
    {PITCH_B6, 1945},
    {PITCH_C7, 1951},
    {PITCH_D7, 1961},
    {PITCH_E7, 1971}, /* 30 */
    {PITCH_F7, 1975},
    {PITCH_G7, 1983},
    {PITCH_A7, 1990},
    {PITCH_B7, 1997},
    {PITCH_C8, 1999}, /* 35 */
    {PITCH_D8, 2005},
    {PITCH_E8, 2009},
    {PITCH_F8, 2012},
    {PITCH_G8, 2016},
    {PITCH_A8, 2019}
};

const struct Pitch_table double_sharp_pitchs[41] = {
    {PITCH_C3,  740}, /*  0 */
    {PITCH_D3,  883},
    {PITCH_E3, 1010},
    {PITCH_F3, 1068},
    {PITCH_G3, 1175},
    {PITCH_A3, 1270}, /*  5 */
    {PITCH_B3, 1355},
    {PITCH_C4, 1394},
    {PITCH_D4, 1466},
    {PITCH_E4, 1529},
    {PITCH_F4, 1558}, /* 10 */
    {PITCH_G4, 1612},
    {PITCH_A4, 1659},
    {PITCH_B4, 1702},
    {PITCH_C5, 1721},
    {PITCH_D5, 1757}, /* 15 */
    {PITCH_E5, 1789},
    {PITCH_F5, 1803},
    {PITCH_G5, 1830},
    {PITCH_A5, 1854},
    {PITCH_B5, 1875}, /* 20 */
    {PITCH_C6, 1885},
    {PITCH_D6, 1902},
    {PITCH_E6, 1918},
    {PITCH_F6, 1926},
    {PITCH_G6, 1939}, /* 25 */
    {PITCH_A6, 1951},
    {PITCH_B6, 1961},
    {PITCH_C7, 1966},
    {PITCH_D7, 1975},
    {PITCH_E7, 1983}, /* 30 */
    {PITCH_F7, 1987},
    {PITCH_G7, 1993},
    {PITCH_A7, 1999},
    {PITCH_B7, 2005},
    {PITCH_C8, 2007}, /* 35 */
    {PITCH_D8, 2012},
    {PITCH_E8, 2016},
    {PITCH_F8, 2017},
    {PITCH_G8, 2021},
    {PITCH_A8, 2024}
};

const struct Pitch_table double_flat_pitchs[41] = {
    {PITCH_C3,  400}, /*  0 */
    {PITCH_D3,  580},
    {PITCH_E3,  740},
    {PITCH_F3,  814},
    {PITCH_G3,  948},
    {PITCH_A3, 1068}, /*  5 */
    {PITCH_B3, 1175},
    {PITCH_C4, 1224},
    {PITCH_D4, 1314},
    {PITCH_E4, 1394},
    {PITCH_F4, 1431}, /* 10 */
    {PITCH_G4, 1498},
    {PITCH_A4, 1558},
    {PITCH_B4, 1612},
    {PITCH_C5, 1636},
    {PITCH_D5, 1681}, /* 15 */
    {PITCH_E5, 1721},
    {PITCH_F5, 1739},
    {PITCH_G5, 1773},
    {PITCH_A5, 1803},
    {PITCH_B5, 1830}, /* 20 */
    {PITCH_C6, 1842},
    {PITCH_D6, 1865},
    {PITCH_E6, 1885},
    {PITCH_F6, 1894},
    {PITCH_G6, 1911}, /* 25 */
    {PITCH_A6, 1926},
    {PITCH_B6, 1939},
    {PITCH_C7, 1945},
    {PITCH_D7, 1956},
    {PITCH_E7, 1966}, /* 30 */
    {PITCH_F7, 1971},
    {PITCH_G7, 1979},
    {PITCH_A7, 1987},
    {PITCH_B7, 1993},
    {PITCH_C8, 1997}, /* 35 */
    {PITCH_D8, 2002},
    {PITCH_E8, 2007},
    {PITCH_F8, 2009},
    {PITCH_G8, 2014},
    {PITCH_A8, 2017}
};

struct Pitch_table score_pitchs_t[41], bar_pitchs_t[41];
struct Pitch_table score_pitchs_b[41], bar_pitchs_b[41];


/* message position */
int message_pos = 224;
int message_pos_timer = 10;
int show_msg = 0;
char message[40] = "";
int message_pos_min = 100;

/* shot */
void do_shot() {

    FILE far *out;
    int i;

    sys_interrupt_reset_hook(SYS_INT_VBLANK, NULL);

    creat("/ram0/shot", O_RDWR, 130);
    out = fopen("/ram0/shot", "w");

    if (out == NULL) {
        while((key_press_check() & KEY_Y1) != 0)
            {}

        sys_interrupt_set_hook(SYS_INT_VBLANK, &intVector, NULL);

        return;
    }

    for (i=0; i<(32*28*18); i++)  fputc(buffer[i], out);

    fclose(out);

    while((key_press_check() & KEY_Y1) != 0)
        {}

    sys_interrupt_set_hook(SYS_INT_VBLANK, &intVector, NULL);
    return;
}


/* \ʒup̊e element ̕` */

int element_width_for_adj(struct Score_element *element) {

    int category = element->category;
    int type = element->type;

    switch (category) {
        case SCORE_CREF :
            switch (type) {
                case G_CREF : return 14;
                case F_CREF : return 14;
            }
        case SCORE_KEY :
            switch (type & 0x00ff) {  /* bit9-16 is used to hold last key in C_MAJOR */
                case C_MAJOR : case A_MINOR :
                    switch ((type>>8) & 0x007f) {  /* bit9-16 is used to hold last key in C_MAJOR */
                        case C_MAJOR : return  2;
                        case G_MAJOR : return 10;
                        case D_MAJOR : return 16;
                        case A_MAJOR : return 18;
                        case E_MAJOR : return 24;
                        case F_MAJOR : return 10;
                        case B_FLAT_MAJOR : return 14;
                        case E_FLAT_MAJOR : return 20;
                        case A_FLAT_MAJOR : return 24;
                        case D_FLAT_MAJOR : return 30;
                        case B_MINOR : return 14;
                        case D_MINOR : return 10;
                        case E_MINOR : return 10;
                        case G_MINOR : return 14;
                        case B_FLAT_MINOR : return 30;
                    }
                case G_MAJOR : return 10;
                case D_MAJOR : return 16;
                case A_MAJOR : return 20;
                case E_MAJOR : return 26;
                case F_MAJOR : return 10;
                case B_FLAT_MAJOR : return 14;
                case E_FLAT_MAJOR : return 20;
                case A_FLAT_MAJOR : return 24;
                case D_FLAT_MAJOR : return 30;
                case B_MINOR : return 14;
                case D_MINOR : return 10;
                case E_MINOR : return 10;
                case G_MINOR : return 14;
                case B_FLAT_MINOR : return 30;
            }
        case SCORE_RITHM :
            switch (type) {
                case BINARIA :      return 10;
                case TERNARIA :     return 10;
                case QUATERNARIA :  return 10;
                case CROME :        return 10;
                case SEMICROME :    return 10;
                case TWOFOURTH :    return 10;
                case TWELVEEIGHTH : return 10;
            }
        case SCORE_SIG :
            switch (type) {
                case SIG_SHARP :        return  6;
                case SIG_FLAT :         return  6;
                case SIG_NATURAL :      return  6;
                case SIG_DOUBLE_SHARP : return  6;
                case SIG_DOUBLE_FLAT :  return 10;
            }
        case SCORE_REST :
            switch (type) {
                case DOTTED_WHOLE :     return 75;
                case WHOLE :            return 50;
                case DOTTED_MINIM :     return 40;
                case MINIM :            return 27;
                case DOTTED_CROTCHET :  return 22;
                case CROTCHET :         return 14;
                case DOTTED_QUAVER :    return 12;
                case QUAVER :           return  8;
                case DOTTED_SIXTEENTH : return  6;
                case SIXTEENTH :        return  6;
                case THIRTY_SECOND :    return  6;
            }
        case SCORE_NOTE :
            switch (type) {
                case DOTTED_WHOLE :     return 75;
                case WHOLE :            return 50;
                case DOTTED_MINIM :     return 40;
                case MINIM :            return 27;
                case DOTTED_CROTCHET :  return 22;
                case CROTCHET :         return 14;
                case DOTTED_QUAVER :    return 12;
                case QUAVER :           return  8;
                case DOTTED_SIXTEENTH : return  6;
                case SIXTEENTH :        return  6;
                case THIRTY_SECOND :    return  6;
                case MINIM_TRIPLET :    return  8;
                case CROTCHET_TRIPLET : return  8;
            }
        case SCORE_BAR :
            return 6;
    }
    return 8;
}


/* e element ̑I\` */
int element_width_for_sel(struct Score_element *element) {

    int category = element->category;
    int type = element->type;

    switch (category) {
        case SCORE_CREF :
            switch (type) {
                case G_CREF : return 14;
                case F_CREF : return 14;
            }
        case SCORE_KEY :
            switch (type & 0x00ff) {  /* bit9-16 is used to hold last key in C_MAJOR */
                case C_MAJOR : case A_MINOR :
                    switch ((type>>8) & 0x007f) {  /* bit9-16 is used to hold last key in C_MAJOR */
                        case C_MAJOR : return  2;
                        case G_MAJOR : return 10;
                        case D_MAJOR : return 16;
                        case A_MAJOR : return 18;
                        case E_MAJOR : return 24;
                        case F_MAJOR : return 10;
                        case B_FLAT_MAJOR : return 14;
                        case E_FLAT_MAJOR : return 20;
                        case A_FLAT_MAJOR : return 24;
                        case D_FLAT_MAJOR : return 30;
                        case B_MINOR : return 14;
                        case D_MINOR : return 10;
                        case E_MINOR : return 10;
                        case G_MINOR : return 14;
                        case B_FLAT_MINOR : return 30;
                    }
                case G_MAJOR : return 10;
                case D_MAJOR : return 16;
                case A_MAJOR : return 20;
                case E_MAJOR : return 26;
                case F_MAJOR : return 10;
                case B_FLAT_MAJOR : return 14;
                case E_FLAT_MAJOR : return 20;
                case A_FLAT_MAJOR : return 24;
                case D_FLAT_MAJOR : return 30;
                case B_MINOR : return 14;
                case D_MINOR : return 10;
                case E_MINOR : return 10;
                case G_MINOR : return 14;
                case B_FLAT_MINOR : return 30;
            }
        case SCORE_RITHM :
            switch (type) {
                case BINARIA :      return 10;
                case TERNARIA :     return 10;
                case QUATERNARIA :  return 10;
                case CROME :        return 10;
                case SEMICROME :    return 10;
                case TWOFOURTH :    return 10;
                case TWELVEEIGHTH : return 10;
            }
        case SCORE_SIG :
            switch (type) {
                case SIG_SHARP :        return  6;
                case SIG_FLAT :         return  6;
                case SIG_NATURAL :      return  6;
                case SIG_DOUBLE_SHARP : return  6;
                case SIG_DOUBLE_FLAT :  return 10;
            }
        case SCORE_REST :
            switch (type) {
                case DOTTED_WHOLE :     return 10;
                case WHOLE :            return 10;
                case DOTTED_MINIM :     return 10;
                case MINIM :            return 10;
                case DOTTED_CROTCHET :  return  8;
                case CROTCHET :         return  8;
                case DOTTED_QUAVER :    return  8;
                case QUAVER :           return  8;
                case DOTTED_SIXTEENTH : return  8;
                case SIXTEENTH :        return  8;
                case THIRTY_SECOND :    return  8;
            }
        case SCORE_NOTE :
            switch (type) {
                case DOTTED_WHOLE :     return 8;
                case WHOLE :            return 8;
                case DOTTED_MINIM :     return 8;
                case MINIM :            return 8;
                case DOTTED_CROTCHET :  return 8;
                case CROTCHET :         return 8;
                case DOTTED_QUAVER :    return 8;
                case QUAVER :           return 8;
                case DOTTED_SIXTEENTH : return 8;
                case SIXTEENTH :        return 8;
                case THIRTY_SECOND :    return 8;
                case MINIM_TRIPLET :    return 8;
                case CROTCHET_TRIPLET : return 8;
            }
        case SCORE_BAR :
            return 6;
    }
    return 8;
}


/* display buffer handlings */

void set_pixel(int x, int y, unsigned int color) {

    int ix = ((x&6)>>1) + ((y&7)<< 2) + ((x&~7)<<2) + (y&~7)*112;

    char byte = buffer[ix];
    if (x&1) buffer[ix] = (byte&0xf0) | color;
    else buffer[ix] = (byte&0x0f) | (color << 4);
}


unsigned int get_pixel(int x, int y) {

    int ix = ((x&6)>>1) + ((y&7)<< 2) + ((x&~7)<<2) + (y&~7)*112;

    if (x&1) return (unsigned int)(buffer[ix]&0x0f);
    else return (unsigned int)((buffer[ix]&0xf0)>>4);
}


/* score initialization */

void init_score_t() {

    /* set example data */

    score_t[0].category = SCORE_CREF;
    score_t[0].type = G_CREF;
    score_t[0].pitch = PITCH_C6;
    score_t[0].attributes = PITCH_HIGH;

    score_t[1].category = SCORE_KEY;
    score_t[1].type = F_MAJOR;
    score_t[1].pitch = PITCH_C6;
    score_t[1].attributes = ~DIRECTION_REVERSED & ~TIE_SET;

    score_t[2].category = SCORE_RITHM;
    score_t[2].type = QUATERNARIA;
    score_t[2].pitch = PITCH_C6;
    score_t[2].attributes = ~DIRECTION_REVERSED & ~TIE_SET;

    no_of_elements_t = 3;
}


void init_score_b() {

    /* set example data */

    score_b[0].category = SCORE_CREF;
    score_b[0].type = F_CREF;
    score_b[0].pitch = PITCH_C6;
    score_b[0].attributes = PITCH_LOW;

    score_b[1].category = SCORE_KEY;
    score_b[1].type = F_MAJOR;
    score_b[1].pitch = PITCH_C6;
    score_b[1].attributes = ~DIRECTION_REVERSED & ~TIE_SET;

    score_b[2].category = SCORE_RITHM;
    score_b[2].type = QUATERNARIA;
    score_b[2].pitch = PITCH_C6;
    score_b[2].attributes = ~DIRECTION_REVERSED & ~TIE_SET;

    no_of_elements_b = 3;
}


/* delay to avoid key chattering */

void delay() {
    int i;
    for (i=0; i++; i<5000) {}
}


/* screen display infra */

void show_form(int xpos, int ypos, struct Form form, unsigned int color) {

    int i, j;  /* pixel position, column & row */
    int bits_index =0;
    int bit_count =0;
    unsigned int sixteen_bits;

    int max_i, max_j;
    int ix;
    char byte;

    /* form is placed out of screen */
    if ((xpos + form.width) < sc_from_x) return;
    if (sc_to_x <= xpos) return;
    if ((ypos + form.height) < sc_from_y) return;
    if (sc_to_y <= ypos) return;

    /* form is on the border of screen */
    if ((xpos < sc_from_x) || (ypos < sc_from_y)
    || (sc_to_x <= (xpos+form.width)) || (sc_to_y <= (ypos+form.height))) {
        sixteen_bits = form.bits[bits_index];
        max_j = ypos + form.height;
        j = ypos;
        while(j<max_j) {
            max_i = xpos + form.width;
            i = xpos;
            while(i<max_i) {
                if ((sixteen_bits & 0x8000) != 0) {
                    if ((sc_from_x <= i) && (sc_from_y <= j) && (i < sc_to_x) && (j < sc_to_y)) {
                        /* set_pixel(i++, j, color); */
                        ix = ((i&6)>>1) + ((j&7)<< 2) + ((i&~7)<<2) + (j&~7)*112;
                        byte = buffer[ix];
                        if (i&1) buffer[ix] = (byte&0xf0) | color;
                        else buffer[ix] = (byte&0x0f) | (color << 4);
                    }
                }
                i++;
                sixteen_bits = sixteen_bits << 1;
                if (++bit_count == 16) {
                    sixteen_bits = form.bits[++bits_index];
                    bit_count = 0;
                }
            }
            j++;
        }
        return;
    }

    /* form is included within screen */
    sixteen_bits = form.bits[bits_index];
    max_j = ypos + form.height;
    j = ypos;
    while(j<max_j) {
        max_i = xpos + form.width;
        i = xpos;
        while(i<max_i) {
            if ((sixteen_bits & 0x8000) != 0) {
                /* set_pixel(i++, j, color); */
                ix = ((i&6)>>1) + ((j&7)<< 2) + ((i&~7)<<2) + (j&~7)*112;
                byte = buffer[ix];
                if (i&1) buffer[ix] = (byte&0xf0) | color;
                else buffer[ix] = (byte&0x0f) | (color << 4);
            }
            i++;
            sixteen_bits = sixteen_bits << 1;
            if (++bit_count == 16) {
                sixteen_bits = form.bits[++bits_index];
                bit_count = 0;
            }
        }
        j++;
    }
}


void clear_screen() {

    int i=0;

    while (i < 32*28*18) buffer[i++]=0x11;
}


void paint_screen(int xpos, int ypos, int width, int height, unsigned int color) {

    int i, j;    /* pixel position, column & row */
    int xpos_t, ypos_t, width_t, height_t;
    int max_i, max_j;

    int ix;
    char byte;

    /* area is placed out of screen */
    if ((xpos + width) < 0) return;
    if (224 <= xpos) return;
    if ((ypos + height) < 0) return;
    if (144 <= ypos) return;

    /* check if area is on the border of screen */
    if (xpos < 0) {
        xpos_t = 0;
        if (xpos + width > 224) width_t = 224;
        else width_t = width + xpos;
    } else {
        xpos_t = xpos;
        if (xpos + width > 224) width_t = 224 - xpos;
        else width_t = width;
    }
    if (ypos < 0) {
        ypos_t = 0;
        if (ypos + height > 144) height_t = 144;
        else height_t = height + ypos;
    } else {
        ypos_t = ypos;
        if (ypos + height > 144) height_t = 144 - ypos;
        else height_t = height;
    }

    /* paint */

    max_j = ypos_t + height_t;
    j = ypos_t;
    while(j<max_j) {
        max_i = xpos_t + width_t;
        i = xpos_t;
        if (i&1) {  /* treat pixel at edge if x-coordinate is odd */
            /* set_pixel(i++, j, color); */
            ix = ((i&6)>>1) + ((j&7)<< 2) + ((i&~7)<<2) + (j&~7)*112;
            byte = buffer[ix];
            buffer[ix] = (byte&0xf0) | color;
            i++;
        }
        byte = (color << 4) | color;
        while(i<max_i) {  /* treat 2 pixels at once */
            /* set_pixel(i++, j, color); */
            ix = ((i&6)>>1) + ((j&7)<< 2) + ((i&~7)<<2) + (j&~7)*112;
            buffer[ix] = byte;
            i+=2;
        }
        if (max_i&1) {  /* treat pixel at edge if x-coordinate is odd */
            ix = ((max_i&6)>>1) + ((j&7)<< 2) + ((max_i&~7)<<2) + (j&~7)*112;
            byte = buffer[ix];
            buffer[ix] = (byte&0x0f) | (color << 4);
        }
        j++;
    }
}


void change_color(int xpos, int ypos, int width, int height, unsigned int target_color, unsigned int color) {

    int i, j;    /* pixel position, column & row */
    int xpos_t, ypos_t, width_t, height_t;
    int max_i, max_j;

    /* area is placed out of screen */
    if ((xpos + width) < 0) return;
    if (224 <= xpos) return;
    if ((ypos + height) < 0) return;
    if (144 <= ypos) return;

    /* check if area is on the border of screen */
    if (xpos < 0) {
        xpos_t = 0;
        if (xpos + width > 224) width_t = 224;
        else width_t = width + xpos;
    } else {
        xpos_t = xpos;
        if (xpos + width > 224) width_t = 224 - xpos;
        else width_t = width;
    }
    if (ypos < 0) {
        ypos_t = 0;
        if (ypos + height > 144) height_t = 144;
        else height_t = height + ypos;
    } else {
        ypos_t = ypos;
        if (ypos + height > 144) height_t = 144 - ypos;
        else height_t = height;
    }

    /* area is included within screen */
    max_j = ypos_t + height_t;
    j = ypos_t;
    while(j<max_j) {
        max_i = xpos_t + width_t;
        i = xpos_t;
        while(i<max_i) {
            if (get_pixel(i, j) == target_color)
                set_pixel(i, j, color);
            i++;
        }
        j++;
    }
}


void init_char_forms() {

    char_forms[33] = &form_char_33;
    char_forms[34] = &form_char_34;
    char_forms[35] = &form_char_35;
    char_forms[36] = &form_char_36;
    char_forms[37] = &form_char_37;
    char_forms[38] = &form_char_38;
    char_forms[40] = &form_char_40;
    char_forms[41] = &form_char_41;
    char_forms[61] = &form_char_61;
    char_forms[45] = &form_char_45;
    char_forms[32] = &form_char_32;
    char_forms[49] = &form_char_49;
    char_forms[50] = &form_char_50;
    char_forms[51] = &form_char_51;
    char_forms[52] = &form_char_52;
    char_forms[53] = &form_char_53;
    char_forms[54] = &form_char_54;
    char_forms[55] = &form_char_55;
    char_forms[56] = &form_char_56;
    char_forms[57] = &form_char_57;
    char_forms[48] = &form_char_48;
    char_forms[97] = &form_char_97;
    char_forms[98] = &form_char_98;
    char_forms[99] = &form_char_99;
    char_forms[100] = &form_char_100;
    char_forms[101] = &form_char_101;
    char_forms[102] = &form_char_102;
    char_forms[103] = &form_char_103;
    char_forms[104] = &form_char_104;
    char_forms[105] = &form_char_105;
    char_forms[106] = &form_char_106;
    char_forms[107] = &form_char_107;
    char_forms[108] = &form_char_108;
    char_forms[109] = &form_char_109;
    char_forms[110] = &form_char_110;
    char_forms[111] = &form_char_111;
    char_forms[112] = &form_char_112;
    char_forms[113] = &form_char_113;
    char_forms[114] = &form_char_114;
    char_forms[115] = &form_char_115;
    char_forms[116] = &form_char_116;
    char_forms[117] = &form_char_117;
    char_forms[118] = &form_char_118;
    char_forms[119] = &form_char_119;
    char_forms[120] = &form_char_120;
    char_forms[121] = &form_char_121;
    char_forms[122] = &form_char_122;
}


void show_string(int xpos, int ypos, char *text, unsigned int color) {

    struct Form *char_form;

    while (*text != '\0') {
        char_form = char_forms[(int)*text++];
        show_form(xpos, ypos, *char_form, color);
        xpos += char_form->width;
    }
}


void show_horizontal_line(int from_x, int width, int y, unsigned int color) {

    int i;
    int from_x_t, width_t;
    int max_i;

    if (from_x + width < sc_from_x) return;
    if (sc_to_x <= from_x) return;
    if (y < sc_from_y) return;
    if (sc_to_y <= y) return;

    /* check if area is on the border of screen */
    if (from_x < sc_from_x) {
        from_x_t = sc_from_x;
        if (from_x + width > sc_to_x) width_t = sc_to_x - sc_from_x;
        else width_t = width + from_x - sc_from_x;
    } else {
        from_x_t = from_x;
        if (from_x + width > sc_to_x) width_t = sc_to_x - from_x;
        else width_t = width;
    }

    max_i = from_x_t + width_t;
    i = from_x_t;
    while(i<max_i) {
        set_pixel(i, y, color);
        i++;
    }
}


void show_vertical_line(int x, int from_y, int to_y, unsigned int color) {

    int i;

    if (x < sc_from_x) return;
    if (sc_to_x <= x) return;

    for(i=from_y; i<to_y; i++) {
        set_pixel(x, i, color);
    }
}


/* show message */
void show_message() {
    show_msg = 0;
    paint_screen(0, 124, 224, 20, 0x01);
    show_string(message_pos, 124, message, 0x00);
    invalidate = 1;
}


void set_message(char *text) {

    int i=0;

    message_pos_min = 224 - 3;
    while((message[i++] = *text) != '\0') {
        message_pos_min -= (char_forms[(int)*text++])->width;
    }
    show_msg = 1;
    message_pos = 224;
    message_pos_timer = 3;
}


/* show selections */

int selection_position_from_t() {

    return score_t[selection_from].position;
}

int selection_position_from_b() {

    return score_b[selection_from].position;
}


int selection_position_to_t() {

    struct Score_element *element = &score_t[selection_to];

    return element->position + element_width_for_sel(element);
}

int selection_position_to_b() {

    struct Score_element *element = &score_b[selection_to];

    return element->position + element_width_for_sel(element);
}


/* I element \ */
void paint_score_selections() {

    if (t_or_b == TREBLE) {
        if (no_of_elements_t < 1) return;
        change_color(
            selection_position_from_t() - position_offset,
            10,
            selection_position_to_t() - selection_position_from_t(),
            50,
            0x01,  /* white */
            0x02   /* yellow */
        );
    } else {
        if (no_of_elements_b < 1) return;
        change_color(
            selection_position_from_b() - position_offset,
            60,
            selection_position_to_b() - selection_position_from_b(),
            50,
            0x01,  /* white */
            0x02   /* yellow */
        );
    }
    invalidate = 1;
}


/* I element ̋\NA */
void restore_score_selections() {

    if (t_or_b == TREBLE) {
        change_color(
            selection_position_from_t() - position_offset,
            10,
            selection_position_to_t() - selection_position_from_t(),
            50,
            0x02,  /* yellow */
            0x01   /* white */
        );
    } else {
        change_color(
            selection_position_from_b() - position_offset,
            60,
            selection_position_to_b() - selection_position_from_b(),
            50,
            0x02,  /* yellow */
            0x01   /* white */
        );
    }
    invalidate = 1;
}


void paint_play_selection_t() {

    struct Score_element *element = &score_t[shown_play_selection_t];

    change_color(
        element->position - position_offset,
        10,
        element_width_for_sel(element),
        50,
        0x01,  /* white */
        0x04   /* color for play selection */
    );
    invalidate = 1;
}

void paint_play_selection_b() {

    struct Score_element *element = &score_b[shown_play_selection_b];

    change_color(
        element->position - position_offset,
        60,
        element_width_for_sel(element),
        50,
        0x01,  /* white */
        0x04   /* color for play selection */
    );
    invalidate = 1;
}


void restore_play_selection_t() {

    struct Score_element *element = &score_t[shown_play_selection_t];

    change_color(
        element->position - position_offset,
        10,
        element_width_for_sel(element),
        50,
        0x04,  /* color for play selection */
        0x01   /* white */
    );
    invalidate = 1;
}

void restore_play_selection_b() {

    struct Score_element *element = &score_b[shown_play_selection_b];

    change_color(
        element->position - position_offset,
        60,
        element_width_for_sel(element),
        50,
        0x04,  /* color for play selection */
        0x01   /* white */
    );
    invalidate = 1;
}


/* score displaying */

void show_cref(struct Score_element *element) {
    treble_cref = element->type;
    switch (treble_cref) {
        case G_CREF :
            show_form(element->position - position_offset, 24, g_cref_form, 0x00);
            break;
        case F_CREF :
            show_form(element->position - position_offset, 29, f_cref_form, 0x00);
            break;
    }
}

void show_cref_b(struct Score_element *element) {
    bass_cref = element->type;
    switch (bass_cref) {
        case G_CREF :
            show_form(element->position - position_offset, 74, g_cref_form, 0x00);
            break;
        case F_CREF :
            show_form(element->position - position_offset, 79, f_cref_form, 0x00);
            break;
    }
}


/* show key */

void show_c_major_or_a_minor(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_cref_offset = 10;

    switch (treble_cref) {
        case G_CREF : y_cref_offset = 10; break;
        case F_CREF : y_cref_offset = 14; break;
    }

    switch (((element->type)>>8) & 0x00ff) {  /* bit9-16 is used to hold last key in C_MAJOR */
        case C_MAJOR : case A_MINOR :
            return;

        case E_MAJOR :
            show_form(xpos+15, 20+y_cref_offset, natural_form, 0x00);
        case A_MAJOR :
            show_form(xpos+10, 14+y_cref_offset, natural_form, 0x00);
        case D_MAJOR : case B_MINOR :
            show_form(xpos+7, 22+y_cref_offset, natural_form, 0x00);
        case G_MAJOR : case E_MINOR :
            show_form(xpos+2, 16+y_cref_offset, natural_form, 0x00);
            return;

        case D_FLAT_MAJOR : case B_FLAT_MINOR :
            show_form(xpos+22, 28+y_cref_offset, natural_form, 0x00);
        case A_FLAT_MAJOR :
            show_form(xpos+17, 20+y_cref_offset, natural_form, 0x00);
        case E_FLAT_MAJOR :
            show_form(xpos+12, 26+y_cref_offset, natural_form, 0x00);
        case B_FLAT_MAJOR : case G_MINOR :
            show_form(xpos+7, 18+y_cref_offset, natural_form, 0x00);
        case F_MAJOR : case D_MINOR :
            show_form(xpos+2, 24+y_cref_offset, natural_form, 0x00);
            return;
    }
}

void show_c_major_or_a_minor_b(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_cref_offset = 10;

    switch (treble_cref) {
        case G_CREF : y_cref_offset = 10; break;
        case F_CREF : y_cref_offset = 14; break;
    }

    switch (((element->type)>>8) & 0x00ff) {  /* bit9-16 is used to hold last key in C_MAJOR */
        case C_MAJOR : case A_MINOR :
            return;

        case E_MAJOR :
            show_form(xpos+15, 50+y_cref_offset, natural_form, 0x00);
        case A_MAJOR :
            show_form(xpos+10, 64+y_cref_offset, natural_form, 0x00);
        case D_MAJOR : case B_MINOR :
            show_form(xpos+7, 72+y_cref_offset, natural_form, 0x00);
        case G_MAJOR : case E_MINOR :
            show_form(xpos+2, 66+y_cref_offset, natural_form, 0x00);
            return;

        case D_FLAT_MAJOR : case B_FLAT_MINOR :
            show_form(xpos+22, 78+y_cref_offset, natural_form, 0x00);
        case A_FLAT_MAJOR :
            show_form(xpos+17, 70+y_cref_offset, natural_form, 0x00);
        case E_FLAT_MAJOR :
            show_form(xpos+12, 76+y_cref_offset, natural_form, 0x00);
        case B_FLAT_MAJOR : case G_MINOR :
            show_form(xpos+7, 68+y_cref_offset, natural_form, 0x00);
        case F_MAJOR : case D_MINOR :
            show_form(xpos+2, 74+y_cref_offset, natural_form, 0x00);
            return;
    }
}


void show_key(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_cref_offset = 10;

    switch (treble_cref) {
        case G_CREF : y_cref_offset = 10; break;
        case F_CREF : y_cref_offset = 14; break;
    }

    switch (element->type & 0x00ff) {  /* bit9-16 is used to hold last key in C_MAJOR */
        case C_MAJOR : case A_MINOR :
            show_c_major_or_a_minor(element);
            return;

        case E_MAJOR :
            show_form(xpos+17, 20+y_cref_offset, sharp_form, 0x00);
        case A_MAJOR :
            show_form(xpos+12, 14+y_cref_offset, sharp_form, 0x00);
        case D_MAJOR : case B_MINOR :
            show_form(xpos+7, 22+y_cref_offset, sharp_form, 0x00);
        case G_MAJOR : case E_MINOR :
            show_form(xpos+2, 16+y_cref_offset, sharp_form, 0x00);
            return;

        case D_FLAT_MAJOR : case B_FLAT_MINOR :
            show_form(xpos+22, 24+y_cref_offset, flat_form, 0x00);
        case A_FLAT_MAJOR :
            show_form(xpos+16, 16+y_cref_offset, flat_form, 0x00);
        case E_FLAT_MAJOR :
            show_form(xpos+12, 22+y_cref_offset, flat_form, 0x00);
        case B_FLAT_MAJOR : case G_MINOR :
            show_form(xpos+6, 14+y_cref_offset, flat_form, 0x00);
        case F_MAJOR : case D_MINOR :
            show_form(xpos+2, 20+y_cref_offset, flat_form, 0x00);
            return;
    }
}

void show_key_b(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_cref_offset = 10;

    switch (bass_cref) {
        case G_CREF : y_cref_offset = 10; break;
        case F_CREF : y_cref_offset = 14; break;
    }

    switch (element->type & 0x00ff) {  /* bit9-16 is used to hold last key in C_MAJOR */
        case C_MAJOR : case A_MINOR :
            show_c_major_or_a_minor_b(element);
            return;

        case E_MAJOR :
            show_form(xpos+17, 70+y_cref_offset, sharp_form, 0x00);
        case A_MAJOR :
            show_form(xpos+12, 64+y_cref_offset, sharp_form, 0x00);
        case D_MAJOR : case B_MINOR :
            show_form(xpos+7, 72+y_cref_offset, sharp_form, 0x00);
        case G_MAJOR : case E_MINOR :
            show_form(xpos+2, 66+y_cref_offset, sharp_form, 0x00);
            return;

        case D_FLAT_MAJOR : case B_FLAT_MINOR :
            show_form(xpos+22, 74+y_cref_offset, flat_form, 0x00);
        case A_FLAT_MAJOR :
            show_form(xpos+16, 66+y_cref_offset, flat_form, 0x00);
        case E_FLAT_MAJOR : 
            show_form(xpos+12, 72+y_cref_offset, flat_form, 0x00);
        case B_FLAT_MAJOR : case G_MINOR :
            show_form(xpos+6, 64+y_cref_offset, flat_form, 0x00);
        case F_MAJOR : case D_MINOR :
            show_form(xpos+2, 70+y_cref_offset, flat_form, 0x00);
            return;
    }
}


void show_rithm(struct Score_element *element) {

    int xpos = element->position - position_offset;

    treble_rithm = element->type;
    switch(treble_rithm) {
        case BINARIA : show_form(xpos+2, 29, binaria_form, 0x00); return;
        case TERNARIA : show_form(xpos+2, 29, ternaria_form, 0x00); return;
        case QUATERNARIA : show_form(xpos+2, 33, quaternaria_form, 0x00); return;
        case CROME : show_form(xpos+2, 29, crome_form, 0x00); return;
        case TWOFOURTH : show_form(xpos+2, 29, two_fourth_form, 0x00); return;
        case TWELVEEIGHTH : show_form(xpos+1, 29, twelve_eighth_form, 0x00); return;
    }
    return;
}

void show_rithm_b(struct Score_element *element) {

    int xpos = element->position - position_offset;

    bass_rithm = element->type;
    switch(bass_rithm) {
        case BINARIA : show_form(xpos+2, 79, binaria_form, 0x00); return;
        case TERNARIA : show_form(xpos+2, 79, ternaria_form, 0x00); return;
        case QUATERNARIA : show_form(xpos+2, 83, quaternaria_form, 0x00); return;
        case CROME : show_form(xpos+2, 79, crome_form, 0x00); return;
        case TWOFOURTH : show_form(xpos+2, 79, two_fourth_form, 0x00); return;
        case TWELVEEIGHTH : show_form(xpos+1, 79, twelve_eighth_form, 0x00); return;
    }
    return;
}


/* pitch to y position offset */

int pitch_to_y(int pitch) {
    switch (pitch) {
        case PITCH_C3 : return 76;
        case PITCH_D3 : return 74;
        case PITCH_E3 : return 72;
        case PITCH_F3 : return 70;
        case PITCH_G3 : return 68;
        case PITCH_A3 : return 66;
        case PITCH_B3 : return 64;
        case PITCH_C4 : return 62;
        case PITCH_D4 : return 60;
        case PITCH_E4 : return 58;
        case PITCH_F4 : return 56;
        case PITCH_G4 : return 54;
        case PITCH_A4 : return 52;
        case PITCH_B4 : return 50;
        case PITCH_C5 : return 48;
        case PITCH_D5 : return 46;
        case PITCH_E5 : return 44;
        case PITCH_F5 : return 42;
        case PITCH_G5 : return 40;
        case PITCH_A5 : return 38;
        case PITCH_B5 : return 36;
        case PITCH_C6 : return 34;
        case PITCH_D6 : return 32;
        case PITCH_E6 : return 30;
        case PITCH_F6 : return 28;
        case PITCH_G6 : return 26;
        case PITCH_A6 : return 24;
        case PITCH_B6 : return 22;
        case PITCH_C7 : return 20;
        case PITCH_D7 : return 18;
        case PITCH_E7 : return 16;
        case PITCH_F7 : return 14;
        case PITCH_G7 : return 12;
        case PITCH_A7 : return 10;
        case PITCH_B7 : return 8;
        case PITCH_C8 : return 6;
        case PITCH_D8 : return 4;
        case PITCH_E8 : return 2;
        case PITCH_F8 : return 0;
        case PITCH_G8 : return -2;
        case PITCH_A8 : return -4;
    }
    return 0;
}


/* key signature */

void show_sharp(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_offset = 12;

    switch (treble_cref) {
        case G_CREF : y_offset = 12; break;
        case F_CREF : y_offset = 16; break;
    }
    y_offset += pitch_to_y(element->pitch);
    show_form(xpos, y_offset, sharp_form, 0x00);
}

void show_sharp_b(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_offset = 62;

    switch (bass_cref) {
        case G_CREF : y_offset = 62; break;
        case F_CREF : y_offset = 66; break;
    }
    y_offset += pitch_to_y(element->pitch);
    show_form(xpos, y_offset, sharp_form, 0x00);
}


void show_flat(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_offset = 8;

    switch (treble_cref) {
        case G_CREF : y_offset = 8; break;
        case F_CREF : y_offset = 12; break;
    }
    y_offset += pitch_to_y(element->pitch);
    show_form(xpos, y_offset, flat_form, 0x00);
}

void show_flat_b(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_offset = 58;

    switch (bass_cref) {
        case G_CREF : y_offset = 58; break;
        case F_CREF : y_offset = 62; break;
    }
    y_offset += pitch_to_y(element->pitch);
    show_form(xpos, y_offset, flat_form, 0x00);
}


void show_natural(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_offset = 12;

    switch (treble_cref) {
        case G_CREF : y_offset = 12; break;
        case F_CREF : y_offset = 16; break;
    }
    y_offset += pitch_to_y(element->pitch);
    show_form(xpos, y_offset, natural_form, 0x00);
}

void show_natural_b(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_offset = 62;

    switch (bass_cref) {
        case G_CREF : y_offset = 62; break;
        case F_CREF : y_offset = 66; break;
    }
    y_offset += pitch_to_y(element->pitch);
    show_form(xpos, y_offset, natural_form, 0x00);
}


void show_double_sharp(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_offset = 12;

    switch (treble_cref) {
        case G_CREF : y_offset = 12; break;
        case F_CREF : y_offset = 16; break;
    }
    y_offset += pitch_to_y(element->pitch);
    show_form(xpos, y_offset, double_sharp_form, 0x00);
}

void show_double_sharp_b(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_offset = 62;

    switch (bass_cref) {
        case G_CREF : y_offset = 62; break;
        case F_CREF : y_offset = 66; break;
    }
    y_offset += pitch_to_y(element->pitch);
    show_form(xpos, y_offset, double_sharp_form, 0x00);
}


void show_double_flat(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_offset = 8;

    switch (treble_cref) {
        case G_CREF : y_offset = 8; break;
        case F_CREF : y_offset = 12; break;
    }
    y_offset += pitch_to_y(element->pitch);
    show_form(xpos, y_offset, double_flat_form, 0x00);
}

void show_double_flat_b(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_offset = 58;

    switch (bass_cref) {
        case G_CREF : y_offset = 58; break;
        case F_CREF : y_offset = 62; break;
    }
    y_offset += pitch_to_y(element->pitch);
    show_form(xpos, y_offset, double_flat_form, 0x00);
}


void show_key_signature(struct Score_element *element) {

    switch (element->type) {
        case SIG_SHARP        : show_sharp(element); return;
        case SIG_FLAT         : show_flat(element); return;
        case SIG_NATURAL      : show_natural(element); return;
        case SIG_DOUBLE_SHARP : show_double_sharp(element); return;
	case SIG_DOUBLE_FLAT  : show_double_flat(element); return;
    }
}

void show_key_signature_b(struct Score_element *element) {

    switch (element->type) {
        case SIG_SHARP        : show_sharp_b(element); return;
        case SIG_FLAT         : show_flat_b(element); return;
        case SIG_NATURAL      : show_natural_b(element); return;
        case SIG_DOUBLE_SHARP : show_double_sharp_b(element); return;
	case SIG_DOUBLE_FLAT  : show_double_flat_b(element); return;
    }
}


/* note displaying
   whole note       */

void show_whole_note(struct Score_element *element) {

    int xpos = element->position - position_offset;  /* display position on screen */
    int ypos;
    int y_cref_offset = 10;

    switch (treble_cref) {
        case G_CREF : y_cref_offset = 10; break;
        case F_CREF : y_cref_offset = 14; break;
    }

    ypos = 5 + pitch_to_y(element->pitch) + y_cref_offset;

    /* extra pentagramma */
    switch (ypos) {
        case 59 : case 57 :
            show_horizontal_line(xpos-1, 8, 54, 0x03);
        case 55 : case 53 :
            show_horizontal_line(xpos-1, 8, 54, 0x03);
        case 51 : case 49 :
            show_horizontal_line(xpos-1, 8, 50, 0x03);
            break;
        case 13 : case 11 :
            show_horizontal_line(xpos-1, 8, 14, 0x03);
        case 17 : case 15 :
            show_horizontal_line(xpos-1, 8, 18, 0x03);
        case 21 : case 19 :
            show_horizontal_line(xpos-1, 8, 22, 0x03);
        case 25 : case 23 :
            show_horizontal_line(xpos-1, 8, 26, 0x03);
            break;
    }

    show_form(xpos, ypos, whole_note_form, 0x00);

    return;
}

void show_whole_note_b(struct Score_element *element) {

    int xpos = element->position - position_offset;  /* display position on screen */
    int ypos;
    int y_cref_offset = 10;

    switch (bass_cref) {
        case G_CREF : y_cref_offset = 10; break;
        case F_CREF : y_cref_offset = 14; break;
    }

    ypos = 55 + pitch_to_y(element->pitch) + y_cref_offset;

    /* extra pentagramma */
    switch (ypos) {
        case 109 : case 107 :
            show_horizontal_line(xpos-1, 8, 104, 0x03);
        case 105 : case 103 :
            show_horizontal_line(xpos-1, 8, 104, 0x03);
        case 101 : case 99 :
            show_horizontal_line(xpos-1, 8, 100, 0x03);
            break;
        case 63 : case 61 :
            show_horizontal_line(xpos-1, 8, 64, 0x03);
        case 67 : case 65 :
            show_horizontal_line(xpos-1, 8, 68, 0x03);
        case 71 : case 69 :
            show_horizontal_line(xpos-1, 8, 72, 0x03);
        case 75 : case 73 :
            show_horizontal_line(xpos-1, 8, 76, 0x03);
            break;
    }

    show_form(xpos, ypos, whole_note_form, 0x00);

    return;
}


/* show dotted whole note */

void show_dotted_whole_note(struct Score_element *element) {

    int xpos = element->position - position_offset;  /* display position on screen */
    int ypos;
    int y_cref_offset = 10;

    switch (treble_cref) {
        case G_CREF : y_cref_offset = 10; break;
        case F_CREF : y_cref_offset = 14; break;
    }

    ypos = 5 + pitch_to_y(element->pitch) + y_cref_offset;

    /* extra pentagramma */
    switch (ypos) {
        case 59 : case 57 :
            show_horizontal_line(xpos-1, 8, 54, 0x03);
        case 55 : case 53 :
            show_horizontal_line(xpos-1, 8, 54, 0x03);
        case 51 : case 49 :
            show_horizontal_line(xpos-1, 8, 50, 0x03);
            break;
        case 13 : case 11 :
            show_horizontal_line(xpos-1, 8, 14, 0x03);
        case 17 : case 15 :
            show_horizontal_line(xpos-1, 8, 18, 0x03);
        case 21 : case 19 :
            show_horizontal_line(xpos-1, 8, 22, 0x03);
        case 25 : case 23 :
            show_horizontal_line(xpos-1, 8, 26, 0x03);
            break;
    }

    show_form(xpos, ypos, dotted_whole_note_form, 0x00);

    return;
}

void show_dotted_whole_note_b(struct Score_element *element) {

    int xpos = element->position - position_offset;  /* display position on screen */
    int ypos;
    int y_cref_offset = 10;

    switch (bass_cref) {
        case G_CREF : y_cref_offset = 10; break;
        case F_CREF : y_cref_offset = 14; break;
    }

    ypos = 55 + pitch_to_y(element->pitch) + y_cref_offset;

    /* extra pentagramma */
    switch (ypos) {
        case 109 : case 107 :
            show_horizontal_line(xpos-1, 8, 104, 0x03);
        case 105 : case 103 :
            show_horizontal_line(xpos-1, 8, 104, 0x03);
        case 101 : case 99 :
            show_horizontal_line(xpos-1, 8, 100, 0x03);
            break;
        case 63 : case 61 :
            show_horizontal_line(xpos-1, 8, 64, 0x03);
        case 67 : case 65 :
            show_horizontal_line(xpos-1, 8, 68, 0x03);
        case 71 : case 69 :
            show_horizontal_line(xpos-1, 8, 72, 0x03);
        case 75 : case 73 :
            show_horizontal_line(xpos-1, 8, 76, 0x03);
            break;
    }

    show_form(xpos, ypos, dotted_whole_note_form, 0x00);

    return;
}


/* extra pentagramma for minim, crotchet, quaver */

void extra_pentagramma_normal(int xpos, int y_position_offset) {

    switch (y_position_offset) {

        case 0 : case -2 :
            show_horizontal_line(xpos-1, 8, 14, 0x03);
        case 4 : case 2 :
            show_horizontal_line(xpos-1, 8, 18, 0x03);
        case 8 : case 6 :
            show_horizontal_line(xpos-1, 8, 22, 0x03);
        case 12 : case 10 :
            show_horizontal_line(xpos-1, 8, 26, 0x03);
            break;

        case 48 : case 50 :
            show_horizontal_line(xpos-1, 8, 62, 0x03);
        case 44 : case 46 :
            show_horizontal_line(xpos-1, 8, 58, 0x03);
        case 40 : case 42 :
            show_horizontal_line(xpos-1, 8, 54, 0x03);
        case 36 : case 38 :
            show_horizontal_line(xpos-1, 8, 50, 0x03);
            break;

/*      case 50 : case 48 :
            show_horizontal_line(xpos-1, 8, 64, 0x03);  */
        case 54 : case 52 :
            show_horizontal_line(xpos-1, 8, 68, 0x03);
        case 58 : case 56 :
            show_horizontal_line(xpos-1, 8, 72, 0x03);
        case 62 : case 60 :
            show_horizontal_line(xpos-1, 8, 76, 0x03);
            break;

        case 98 : case 100 :
            show_horizontal_line(xpos-1, 8, 112, 0x03);
        case 94 : case 96 :
            show_horizontal_line(xpos-1, 8, 108, 0x03);
        case 90 : case 92 :
            show_horizontal_line(xpos-1, 8, 104, 0x03);
        case 86 : case 88 :
            show_horizontal_line(xpos-1, 8, 100, 0x03);
            break;
    }
}


void extra_pentagramma_reverse(int xpos, int y_position_offset) {

    switch (y_position_offset) {

        case  4 : case  6 :
            show_horizontal_line(xpos-1, 8, 14, 0x03);
        case  8 : case 10 :
            show_horizontal_line(xpos-1, 8, 18, 0x03);
        case 12 : case 14 :
            show_horizontal_line(xpos-1, 8, 22, 0x03);
        case 16 : case 18 :
            show_horizontal_line(xpos-1, 8, 26, 0x03);
            break;

        case 54 : case 56 :
            show_horizontal_line(xpos-1, 8, 62, 0x03);
        case 50 : case 52 :
            show_horizontal_line(xpos-1, 8, 58, 0x03);
        case 46 : case 48 :
            show_horizontal_line(xpos-1, 8, 54, 0x03);
        case 42 : case 44 :
            show_horizontal_line(xpos-1, 8, 50, 0x03);
            break;

/*      case 54 : case 56 :
            show_horizontal_line(xpos-1, 8, 64, 0x03);  */
        case 58 : case 60 :
            show_horizontal_line(xpos-1, 8, 68, 0x03);
        case 62 : case 64 :
            show_horizontal_line(xpos-1, 8, 72, 0x03);
        case 66 : case 68 :
            show_horizontal_line(xpos-1, 8, 76, 0x03);
            break;

        case 104 : case 106 :
            show_horizontal_line(xpos-1, 8, 112, 0x03);
        case 100 : case 102 :
            show_horizontal_line(xpos-1, 8, 108, 0x03);
        case 96 : case 98 :
            show_horizontal_line(xpos-1, 8, 104, 0x03);
        case 92 : case 94 :
            show_horizontal_line(xpos-1, 8, 100, 0x03);
            break;
    }
}


/* show minim note */

void show_normal_minim_form(int xpos, int y_position_offset) {

    extra_pentagramma_normal(xpos, y_position_offset);
    show_form(xpos, y_position_offset, minim_noml_form, 0x00);
}


void show_reversed_minim_form(int xpos, int y_position_offset) {

    extra_pentagramma_reverse(xpos, y_position_offset);
    show_form(xpos, y_position_offset+7, minim_revs_form, 0x00);
}


void show_minim_note(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_offset = 2;

    switch (treble_cref) {
        case G_CREF : y_offset = 2; break;
        case F_CREF : y_offset = 6; break;
    }

    if ((element->attributes & DIRECTION_REVERSED) == 0)
        show_normal_minim_form(xpos, y_offset + pitch_to_y(element->pitch));
    else
        show_reversed_minim_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
}

void show_minim_note_b(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_offset = 52;

    switch (bass_cref) {
        case G_CREF : y_offset = 52; break;
        case F_CREF : y_offset = 56; break;
    }

    if ((element->attributes & DIRECTION_REVERSED) == 0)
        show_normal_minim_form(xpos, y_offset + pitch_to_y(element->pitch));
    else
        show_reversed_minim_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
}


/* show dotted minim mone */

void show_dotted_normal_minim_form(int xpos, int y_position_offset) {

    extra_pentagramma_normal(xpos, y_position_offset);
    show_form(xpos, y_position_offset, dotted_minim_noml_form, 0x00);
}


void show_dotted_reversed_minim_form(int xpos, int y_position_offset) {

    extra_pentagramma_reverse(xpos, y_position_offset);
    show_form(xpos, y_position_offset+7, dotted_minim_revs_form, 0x00);
}


void show_dotted_minim_note(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_offset = 2;

    switch (treble_cref) {
        case G_CREF : y_offset = 2; break;
        case F_CREF : y_offset = 6; break;
    }

    if ((element->attributes & DIRECTION_REVERSED) == 0)
        show_dotted_normal_minim_form(xpos, y_offset + pitch_to_y(element->pitch));
    else
        show_dotted_reversed_minim_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
}

void show_dotted_minim_note_b(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_offset = 52;

    switch (bass_cref) {
        case G_CREF : y_offset = 52; break;
        case F_CREF : y_offset = 56; break;
    }

    if ((element->attributes & DIRECTION_REVERSED) == 0)
        show_dotted_normal_minim_form(xpos, y_offset + pitch_to_y(element->pitch));
    else
        show_dotted_reversed_minim_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
}


/* show crotchet */

void show_normal_crotchet_form(int xpos, int y_position_offset) {

    extra_pentagramma_normal(xpos, y_position_offset);
    show_form(xpos, y_position_offset, crotchet_noml_form, 0x00);
}


void show_reversed_crotchet_form(int xpos, int y_position_offset) {

    extra_pentagramma_reverse(xpos, y_position_offset);
    show_form(xpos, y_position_offset+7, crotchet_revs_form, 0x00);
}


void show_crotchet_note(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_offset = 2;

    switch (treble_cref) {
        case G_CREF : y_offset = 2; break;
        case F_CREF : y_offset = 6; break;
    }

    if ((element->attributes & DIRECTION_REVERSED) == 0)
        show_normal_crotchet_form(xpos, y_offset + pitch_to_y(element->pitch));
    else
        show_reversed_crotchet_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
}

void show_crotchet_note_b(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_offset = 52;

    switch (bass_cref) {
        case G_CREF : y_offset = 52; break;
        case F_CREF : y_offset = 56; break;
    }

    if ((element->attributes & DIRECTION_REVERSED) == 0)
        show_normal_crotchet_form(xpos, y_offset + pitch_to_y(element->pitch));
    else
        show_reversed_crotchet_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
}


/* show dotted crotchet */

void show_dotted_normal_crotchet_form(int xpos, int y_position_offset) {

    extra_pentagramma_normal(xpos, y_position_offset);
    show_form(xpos, y_position_offset, dotted_crotchet_noml_form, 0x00);
}


void show_dotted_reversed_crotchet_form(int xpos, int y_position_offset) {

    extra_pentagramma_reverse(xpos, y_position_offset);
    show_form(xpos, y_position_offset+7, dotted_crotchet_revs_form, 0x00);
}


void show_dotted_crotchet_note(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_offset = 2;

    switch (treble_cref) {
        case G_CREF : y_offset = 2; break;
        case F_CREF : y_offset = 6; break;
    }

    if ((element->attributes & DIRECTION_REVERSED) == 0)
        show_dotted_normal_crotchet_form(xpos, y_offset + pitch_to_y(element->pitch));
    else
        show_dotted_reversed_crotchet_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
}

void show_dotted_crotchet_note_b(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_offset = 52;

    switch (bass_cref) {
        case G_CREF : y_offset = 52; break;
        case F_CREF : y_offset = 56; break;
    }

    if ((element->attributes & DIRECTION_REVERSED) == 0)
        show_dotted_normal_crotchet_form(xpos, y_offset + pitch_to_y(element->pitch));
    else
        show_dotted_reversed_crotchet_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
}


/* show quaver */

void show_normal_quaver_form(int xpos, int y_position_offset) {

    extra_pentagramma_normal(xpos, y_position_offset);
    show_form(xpos, y_position_offset, quaver_noml_form, 0x00);
}


void show_reversed_quaver_form(int xpos, int y_position_offset) {

    extra_pentagramma_reverse(xpos, y_position_offset);
    show_form(xpos, y_position_offset+7, quaver_revs_form, 0x00);
}


void show_quaver_note(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_offset = 2;

    switch (treble_cref) {
        case G_CREF : y_offset = 2; break;
        case F_CREF : y_offset = 6; break;
    }

    if ((element->attributes & DIRECTION_REVERSED) == 0)
        show_normal_quaver_form(xpos, y_offset + pitch_to_y(element->pitch));
    else
        show_reversed_quaver_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
}

void show_quaver_note_b(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_offset = 52;

    switch (bass_cref) {
        case G_CREF : y_offset = 52; break;
        case F_CREF : y_offset = 56; break;
    }

    if ((element->attributes & DIRECTION_REVERSED) == 0)
        show_normal_quaver_form(xpos, y_offset + pitch_to_y(element->pitch));
    else
        show_reversed_quaver_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
}


/* show dotted quaver note */

void show_dotted_normal_quaver_form(int xpos, int y_position_offset) {

    extra_pentagramma_normal(xpos, y_position_offset);
    show_form(xpos, y_position_offset, dotted_quaver_noml_form, 0x00);
}


void show_dotted_reversed_quaver_form(int xpos, int y_position_offset) {

    extra_pentagramma_reverse(xpos, y_position_offset);
    show_form(xpos, y_position_offset+7, dotted_quaver_revs_form, 0x00);
}


void show_dotted_quaver_note(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_offset = 2;

    switch (treble_cref) {
        case G_CREF : y_offset = 2; break;
        case F_CREF : y_offset = 6; break;
    }

    if ((element->attributes & DIRECTION_REVERSED) == 0)
        show_dotted_normal_quaver_form(xpos, y_offset + pitch_to_y(element->pitch));
    else
        show_dotted_reversed_quaver_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
}

void show_dotted_quaver_note_b(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_offset = 52;

    switch (bass_cref) {
        case G_CREF : y_offset = 52; break;
        case F_CREF : y_offset = 56; break;
    }

    if ((element->attributes & DIRECTION_REVERSED) == 0)
        show_dotted_normal_quaver_form(xpos, y_offset + pitch_to_y(element->pitch));
    else
        show_dotted_reversed_quaver_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
}


/* show sixteenth note 1/16 */

void show_normal_sixteenth_form(int xpos, int y_position_offset) {

    extra_pentagramma_normal(xpos, y_position_offset);
    show_form(xpos, y_position_offset, sixteenth_noml_form, 0x00);
}


void show_reversed_sixteenth_form(int xpos, int y_position_offset) {

    extra_pentagramma_reverse(xpos, y_position_offset);
    show_form(xpos, y_position_offset+7, sixteenth_revs_form, 0x00);
}


void show_sixteenth_note(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_offset = 2;

    switch (treble_cref) {
        case G_CREF : y_offset = 2; break;
        case F_CREF : y_offset = 6; break;
    }

    if ((element->attributes & DIRECTION_REVERSED) == 0)
        show_normal_sixteenth_form(xpos, y_offset + pitch_to_y(element->pitch));
    else
        show_reversed_sixteenth_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
}

void show_sixteenth_note_b(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_offset = 52;

    switch (bass_cref) {
        case G_CREF : y_offset = 52; break;
        case F_CREF : y_offset = 56; break;
    }

    if ((element->attributes & DIRECTION_REVERSED) == 0)
        show_normal_sixteenth_form(xpos, y_offset + pitch_to_y(element->pitch));
    else
        show_reversed_sixteenth_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
}


/* show dotted_sixteenth note 1/16 + 1/32 */

void show_dotted_normal_sixteenth_form(int xpos, int y_position_offset) {

    extra_pentagramma_normal(xpos, y_position_offset);
    show_form(xpos, y_position_offset, dotted_sixteenth_noml_form, 0x00);
}


void show_dotted_reversed_sixteenth_form(int xpos, int y_position_offset) {

    extra_pentagramma_reverse(xpos, y_position_offset);
    show_form(xpos, y_position_offset+7, dotted_sixteenth_revs_form, 0x00);
}


void show_dotted_sixteenth_note(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_offset = 2;

    switch (treble_cref) {
        case G_CREF : y_offset = 2; break;
        case F_CREF : y_offset = 6; break;
    }

    if ((element->attributes & DIRECTION_REVERSED) == 0)
        show_dotted_normal_sixteenth_form(xpos, y_offset + pitch_to_y(element->pitch));
    else
        show_dotted_reversed_sixteenth_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
}

void show_dotted_sixteenth_note_b(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_offset = 52;

    switch (bass_cref) {
        case G_CREF : y_offset = 52; break;
        case F_CREF : y_offset = 56; break;
    }

    if ((element->attributes & DIRECTION_REVERSED) == 0)
        show_dotted_normal_sixteenth_form(xpos, y_offset + pitch_to_y(element->pitch));
    else
        show_dotted_reversed_sixteenth_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
}


/* show thirty second note 1/32 */

void show_normal_thirty_second_form(int xpos, int y_position_offset) {

    extra_pentagramma_normal(xpos, y_position_offset);
    show_form(xpos, y_position_offset, thirty_second_noml_form, 0x00);
}


void show_reversed_thirty_second_form(int xpos, int y_position_offset) {

    extra_pentagramma_reverse(xpos, y_position_offset);
    show_form(xpos, y_position_offset+7, thirty_second_revs_form, 0x00);
}


void show_thirty_second_note(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_offset = 2;

    switch (treble_cref) {
        case G_CREF : y_offset = 2; break;
        case F_CREF : y_offset = 6; break;
    }

    if ((element->attributes & DIRECTION_REVERSED) == 0)
        show_normal_thirty_second_form(xpos, y_offset + pitch_to_y(element->pitch));
    else
        show_reversed_thirty_second_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
}

void show_thirty_second_note_b(struct Score_element *element) {

    int xpos = element->position - position_offset;
    int y_offset = 52;

    switch (bass_cref) {
        case G_CREF : y_offset = 52; break;
        case F_CREF : y_offset = 56; break;
    }

    if ((element->attributes & DIRECTION_REVERSED) == 0)
        show_normal_thirty_second_form(xpos, y_offset + pitch_to_y(element->pitch));
    else
        show_reversed_thirty_second_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
}


/* show triplet */

void set_triplet_direction_and_ypos(struct Score_element *element, int index) {

    /* if there are any triplet which is reversed, all triplets are show as reversed */
    /* the y position of triplet index will be adjusted with the pitch of notes */

    struct Score_element *posterior_element=NULL;
    struct Score_element *far_posterior_element=NULL;
    int elements_ypos;

    struct Score_element *test_element;
    int i, j;
    int elements_ypos_offset;

    if (index == 0) return;  /* palette indication */

    /* set 2nd triplet */
    for (i = index+1; i < index+10; i++) {
        if (no_of_elements_t <= i) break;
        test_element = &score_t[i];
        if (((test_element->category == SCORE_NOTE) || (test_element->category == SCORE_REST))
        && ((test_element->type == MINIM_TRIPLET) || (test_element->type == CROTCHET_TRIPLET))) {
            posterior_element = test_element;
            break;
        }
    }

    /* set 3rd triplet */
    for (j = i+1; j < i+10; j++) {
        if (no_of_elements_t <= j) break;
        test_element = &score_t[j];
        if (((test_element->category == SCORE_NOTE) || (test_element->category == SCORE_REST))
        && ((test_element->type == MINIM_TRIPLET) || (test_element->type == CROTCHET_TRIPLET))) {
            far_posterior_element = test_element;
            break;
        }
    }

    /* check direction */
    triplet_is_reversed = 0;  /* direction is NOT reversed */
    if ((element->category == SCORE_NOTE)
    && ((element->type == MINIM_TRIPLET) || (element->type == CROTCHET_TRIPLET))) {
        if ((element->attributes & DIRECTION_REVERSED) != 0) {
            triplet_is_reversed = 1;
        }
    }

    if ((posterior_element->category == SCORE_NOTE)
    && ((posterior_element->attributes & DIRECTION_REVERSED) != 0)) {
         triplet_is_reversed = 1;  /* direction is reversed */
    }

    if ((far_posterior_element->category == SCORE_NOTE)
    && ((far_posterior_element->attributes & DIRECTION_REVERSED) != 0)) {
         triplet_is_reversed = 1;  /* direction is reversed */
    }

    switch (treble_cref) {
        case G_CREF : elements_ypos_offset = 2; break;
        case F_CREF : elements_ypos_offset = 6; break;
    }

    /* set y position */
    if (triplet_is_reversed == 0) {  /* NOT reversed */
        triplet_indication_ypos = 48;
        if (element->category == SCORE_NOTE) {
            elements_ypos = elements_ypos_offset + pitch_to_y(element->pitch) + 18;
            if (triplet_indication_ypos < elements_ypos) triplet_indication_ypos = elements_ypos;
        }
        if (posterior_element != NULL) {
            if (posterior_element->category == SCORE_NOTE) {
                elements_ypos = elements_ypos_offset + pitch_to_y(posterior_element->pitch) + 18;
                if (triplet_indication_ypos < elements_ypos) triplet_indication_ypos = elements_ypos;
            }
        }
        if (far_posterior_element != NULL) {
            if (far_posterior_element->category == SCORE_NOTE) {
                elements_ypos = elements_ypos_offset + pitch_to_y(far_posterior_element->pitch) + 18;
                if (triplet_indication_ypos < elements_ypos) triplet_indication_ypos = elements_ypos;
            }
        }
    } else {  /* reversed */
        triplet_indication_ypos = 22;
        if (element->category == SCORE_NOTE) {
            elements_ypos = elements_ypos_offset + pitch_to_y(element->pitch) + 4;
            if (elements_ypos < triplet_indication_ypos) triplet_indication_ypos = elements_ypos;
        }
        if (posterior_element != NULL) {
            if (posterior_element->category == SCORE_NOTE) {
                elements_ypos = elements_ypos_offset + pitch_to_y(posterior_element->pitch) + 4;
                if (elements_ypos < triplet_indication_ypos) triplet_indication_ypos = elements_ypos;
            }
        }
        if (far_posterior_element != NULL) {
            if (far_posterior_element->category == SCORE_NOTE) {
                elements_ypos = elements_ypos_offset + pitch_to_y(far_posterior_element->pitch) + 4;
                if (elements_ypos < triplet_indication_ypos) triplet_indication_ypos = elements_ypos;
            }
        }
    }
}

void set_triplet_direction_and_ypos_b(struct Score_element *element, int index) {

    /* if there are any triplet which is reversed, all triplets are show as reversed */
    /* the y position of triplet index will be adjusted with the pitch of notes */

    struct Score_element *posterior_element=NULL;
    struct Score_element *far_posterior_element=NULL;
    int elements_ypos;

    struct Score_element *test_element;
    int i, j;
    int elements_ypos_offset;

    if (index == 0) return;  /* palette indication */

    /* set 2nd triplet */
    for (i = index+1; i < index+10; i++) {
        if (no_of_elements_b <= i) break;
        test_element = &score_b[i];
        if (((test_element->category == SCORE_NOTE) || (test_element->category == SCORE_REST))
        && ((test_element->type == MINIM_TRIPLET) || (test_element->type == CROTCHET_TRIPLET))) {
            posterior_element = test_element;
            break;
        }
    }

    /* set 3rd triplet */
    for (j = i+1; j < i+10; j++) {
        if (no_of_elements_b <= j) break;
        test_element = &score_b[j];
        if (((test_element->category == SCORE_NOTE) || (test_element->category == SCORE_REST))
        && ((test_element->type == MINIM_TRIPLET) || (test_element->type == CROTCHET_TRIPLET))) {
            far_posterior_element = test_element;
            break;
        }
    }

    /* check direction */
    triplet_is_reversed = 0;  /* direction is NOT reversed */
    if ((element->category == SCORE_NOTE)
    && ((element->type == MINIM_TRIPLET) || (element->type == CROTCHET_TRIPLET))) {
        if ((element->attributes & DIRECTION_REVERSED) != 0) {
            triplet_is_reversed = 1;
        }
    }

    if ((posterior_element->category == SCORE_NOTE)
    && ((posterior_element->attributes & DIRECTION_REVERSED) != 0)) {
         triplet_is_reversed = 1;  /* direction is reversed */
    }

    if ((far_posterior_element->category == SCORE_NOTE)
    && ((far_posterior_element->attributes & DIRECTION_REVERSED) != 0)) {
         triplet_is_reversed = 1;  /* direction is reversed */
    }

    switch (bass_cref) {
        case G_CREF : elements_ypos_offset = 2; break;
        case F_CREF : elements_ypos_offset = 6; break;
    }

    /* set y position */
    if (triplet_is_reversed == 0) {  /* NOT reversed */
        triplet_indication_ypos = 98;
        if (element->category == SCORE_NOTE) {
            elements_ypos = elements_ypos_offset + pitch_to_y(element->pitch) + 68;
            if (triplet_indication_ypos < elements_ypos) triplet_indication_ypos = elements_ypos;
        }
        if (posterior_element != NULL) {
            if (posterior_element->category == SCORE_NOTE) {
                elements_ypos = elements_ypos_offset + pitch_to_y(posterior_element->pitch) + 68;
                if (triplet_indication_ypos < elements_ypos) triplet_indication_ypos = elements_ypos;
            }
        }
        if (far_posterior_element != NULL) {
            if (far_posterior_element->category == SCORE_NOTE) {
                elements_ypos = elements_ypos_offset + pitch_to_y(far_posterior_element->pitch) + 68;
                if (triplet_indication_ypos < elements_ypos) triplet_indication_ypos = elements_ypos;
            }
        }
    } else {  /* reversed */
        triplet_indication_ypos = 72;
        if (element->category == SCORE_NOTE) {
            elements_ypos = elements_ypos_offset + pitch_to_y(element->pitch) + 54;
            if (elements_ypos < triplet_indication_ypos) triplet_indication_ypos = elements_ypos;
        }
        if (posterior_element != NULL) {
            if (posterior_element->category == SCORE_NOTE) {
                elements_ypos = elements_ypos_offset + pitch_to_y(posterior_element->pitch) + 54;
                if (elements_ypos < triplet_indication_ypos) triplet_indication_ypos = elements_ypos;
            }
        }
        if (far_posterior_element != NULL) {
            if (far_posterior_element->category == SCORE_NOTE) {
                elements_ypos = elements_ypos_offset + pitch_to_y(far_posterior_element->pitch) + 54;
                if (elements_ypos < triplet_indication_ypos) triplet_indication_ypos = elements_ypos;
            }
        }
    }
}


/* minim triplet */

void show_normal_minim_triplet_form(int xpos, int y_position_offset) {

    extra_pentagramma_normal(xpos, y_position_offset);
    show_form(xpos, y_position_offset, crotchet_noml_form, 0x00);
}


void show_reversed_minim_triplet_form(int xpos, int y_position_offset) {

    extra_pentagramma_reverse(xpos, y_position_offset);
    show_form(xpos, y_position_offset+7, crotchet_revs_form, 0x00);
}


void show_minim_triplet_note(struct Score_element *element, int index) {

    int xpos = element->position - position_offset;
    int former_xpos, width;
    int y_offset = 2;

    switch (treble_cref) {
        case G_CREF : y_offset = 2; break;
        case F_CREF : y_offset = 6; break;
    }

    switch (triplet_index) {
        case 0 :
            /* drawing area of triplet exceeds right by 100(maybe) than ordinal note */
            sc_to_x += 100;
            if (224 < sc_to_x) sc_to_x = 224;

            set_triplet_direction_and_ypos(element, index);
            if (triplet_is_reversed == 0) {  /* NOT reversed */
                show_normal_minim_triplet_form(xpos, y_offset + pitch_to_y(element->pitch));
                show_form(xpos, triplet_indication_ypos, triplet_indication_noml_1_form, 0x00);
            } else {  /* reversed */
                show_reversed_minim_triplet_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
                show_form(xpos, triplet_indication_ypos, triplet_indication_revs_1_form, 0x00);
            }
            former_element = element;
            triplet_index = 1;
            break;
        case 1 :
            /* drawing area of triplet exceeds right & left by 50(maybe) than ordinal note */
            sc_to_x += 50;
            if (224 < sc_to_x) sc_to_x = 224;
            sc_from_x -= 50;
            if (sc_from_x < 0) sc_from_x = 0;

            former_xpos = former_element->position - position_offset;
            width = xpos - former_xpos - 7;
            if (triplet_is_reversed == 0) {
                show_normal_minim_triplet_form(xpos, y_offset + pitch_to_y(element->pitch));
                show_form(xpos-2, triplet_indication_ypos, triplet_indication_noml_2_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+5, width, triplet_indication_ypos+3, 0x00);
            } else {
                show_reversed_minim_triplet_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
                show_form(xpos-2, triplet_indication_ypos, triplet_indication_revs_2_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+5, width, triplet_indication_ypos+3, 0x00);
            }
            former_element = element;
            triplet_index = 2;
            break;
        case 2 :
            /* drawing area of triplet exceeds right & left by 50(maybe) than ordinal note */
            sc_from_x -= 100;
            if (sc_from_x < 0) sc_from_x = 0;

            former_xpos = former_element->position - position_offset;
            width = xpos - former_xpos - 7;
            if (triplet_is_reversed == 0) {
                show_normal_minim_triplet_form(xpos, y_offset + pitch_to_y(element->pitch));
                show_form(xpos-6, triplet_indication_ypos, triplet_indication_noml_3_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+7, width, triplet_indication_ypos+3, 0x00);
            } else {
                show_reversed_minim_triplet_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
                show_form(xpos-6, triplet_indication_ypos, triplet_indication_revs_3_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+7, width, triplet_indication_ypos+3, 0x00);
            }
            triplet_index = 0;
            break;
        case 4 :    /* special procedure for palette display */
            if ((element->attributes & DIRECTION_REVERSED) == 0) {
                show_normal_minim_triplet_form(xpos, y_offset + pitch_to_y(element->pitch));
                show_form(xpos-2, 48, triplet_indication_noml_4_form, 0x00);
            } else {
                show_reversed_minim_triplet_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
                show_form(xpos-2, 22, triplet_indication_revs_4_form, 0x00);
            }
            former_element = element;
            triplet_index = 0;
            break;
    }
}

void show_minim_triplet_note_b(struct Score_element *element, int index) {

    int xpos = element->position - position_offset;
    int former_xpos, width;
    int y_offset = 52;

    switch (bass_cref) {
        case G_CREF : y_offset = 52; break;
        case F_CREF : y_offset = 56; break;
    }

    switch (triplet_index) {
        case 0 :
            /* drawing area of triplet exceeds right by 100(maybe) than ordinal note */
            sc_to_x += 100;
            if (224 < sc_to_x) sc_to_x = 224;

            set_triplet_direction_and_ypos_b(element, index);
            if (triplet_is_reversed == 0) {
                show_normal_minim_triplet_form(xpos, y_offset + pitch_to_y(element->pitch));
                show_form(xpos, triplet_indication_ypos, triplet_indication_noml_1_form, 0x00);
            } else {
                show_reversed_minim_triplet_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
                show_form(xpos, triplet_indication_ypos, triplet_indication_revs_1_form, 0x00);
            }
            former_element = element;
            triplet_index = 1;
            break;
        case 1 :
            /* drawing area of triplet exceeds right & left by 50(maybe) than ordinal note */
            sc_to_x += 50;
            if (224 < sc_to_x) sc_to_x = 224;
            sc_from_x -= 50;
            if (sc_from_x < 0) sc_from_x = 0;

            former_xpos = former_element->position - position_offset;
            width = xpos - former_xpos - 7;
            if (triplet_is_reversed == 0) {
                show_normal_minim_triplet_form(xpos, y_offset + pitch_to_y(element->pitch));
                show_form(xpos-2, triplet_indication_ypos, triplet_indication_noml_2_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+5, width, triplet_indication_ypos+3, 0x00);
            } else {
                show_reversed_minim_triplet_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
                show_form(xpos-2, triplet_indication_ypos, triplet_indication_revs_2_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+5, width, triplet_indication_ypos+3, 0x00);
            }
            former_element = element;
            triplet_index = 2;
            break;
        case 2 :
            /* drawing area of triplet exceeds right & left by 50(maybe) than ordinal note */
            sc_from_x -= 100;
            if (sc_from_x < 0) sc_from_x = 0;

            former_xpos = former_element->position - position_offset;
            width = xpos - former_xpos - 7;
            if (triplet_is_reversed == 0) {
                show_normal_minim_triplet_form(xpos, y_offset + pitch_to_y(element->pitch));
                show_form(xpos-6, triplet_indication_ypos, triplet_indication_noml_3_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+7, width, triplet_indication_ypos+3, 0x00);
            } else {
                show_reversed_minim_triplet_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
                show_form(xpos-6, triplet_indication_ypos, triplet_indication_revs_3_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+7, width, triplet_indication_ypos+3, 0x00);
            }
            triplet_index = 0;
            break;
        case 4 :    /* special procedure for palette display */
            if ((element->attributes & DIRECTION_REVERSED) == 0) {
                show_normal_minim_triplet_form(xpos, y_offset + pitch_to_y(element->pitch));
                show_form(xpos-2, 98, triplet_indication_noml_4_form, 0x00);
            } else {
                show_reversed_minim_triplet_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
                show_form(xpos-2, 72, triplet_indication_revs_4_form, 0x00);
            }
            former_element = element;
            triplet_index = 0;
            break;
    }
}


/* show crotchet triplet */

void show_normal_crotchet_triplet_form(int xpos, int y_position_offset) {

    extra_pentagramma_normal(xpos, y_position_offset);
    show_form(xpos, y_position_offset, quaver_noml_form, 0x00);
}


void show_reversed_crotchet_triplet_form(int xpos, int y_position_offset) {

    extra_pentagramma_reverse(xpos, y_position_offset);
    show_form(xpos, y_position_offset+7, quaver_revs_form, 0x00);
}


void show_crotchet_triplet_note(struct Score_element *element, int index) {

    int xpos = element->position - position_offset;
    int former_xpos, width;
    int y_offset = 2;

    switch (treble_cref) {
        case G_CREF : y_offset = 2; break;
        case F_CREF : y_offset = 6; break;
    }

    switch (triplet_index) {
        case 0 :
            /* drawing area of triplet exceeds right by 100(maybe) than ordinal note */
            sc_to_x += 100;
            if (224 < sc_to_x) sc_to_x = 224;

            set_triplet_direction_and_ypos(element, index);
            if (triplet_is_reversed == 0) {
                show_normal_crotchet_triplet_form(xpos, y_offset + pitch_to_y(element->pitch));
                show_form(xpos, triplet_indication_ypos, triplet_indication_noml_1_form, 0x00);
            } else {
                show_reversed_crotchet_triplet_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
                show_form(xpos, triplet_indication_ypos, triplet_indication_revs_1_form, 0x00);
            }
            former_element = element;
            triplet_index = 1;
            break;
        case 1 :
            /* drawing area of triplet exceeds right & left by 50(maybe) than ordinal note */
            sc_to_x += 50;
            if (224 < sc_to_x) sc_to_x = 224;
            sc_from_x -= 50;
            if (sc_from_x < 0) sc_from_x = 0;

            former_xpos = former_element->position - position_offset;
            width = xpos - former_xpos - 7;
            if (triplet_is_reversed == 0) {
                show_normal_crotchet_triplet_form(xpos, y_offset + pitch_to_y(element->pitch));
                show_form(xpos-2, triplet_indication_ypos, triplet_indication_noml_2_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+5, width, triplet_indication_ypos+3, 0x00);
            } else {
                show_reversed_crotchet_triplet_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
                show_form(xpos-2, triplet_indication_ypos, triplet_indication_revs_2_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+5, width, triplet_indication_ypos+3, 0x00);
            }
            former_element = element;
            triplet_index = 2;
            break;
        case 2 :
            /* drawing area of triplet exceeds left by 100(maybe) than ordinal note */
            sc_from_x -= 100;
            if (sc_from_x < 0) sc_from_x = 0;

            former_xpos = former_element->position - position_offset;
            width = xpos - former_xpos - 7;
            if (triplet_is_reversed == 0) {
                show_normal_crotchet_triplet_form(xpos, y_offset + pitch_to_y(element->pitch));
                show_form(xpos-6, triplet_indication_ypos, triplet_indication_noml_3_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+7, width, triplet_indication_ypos+3, 0x00);
            } else {
                show_reversed_crotchet_triplet_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
                show_form(xpos-6, triplet_indication_ypos, triplet_indication_revs_3_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+7, width, triplet_indication_ypos+3, 0x00);
            }
            triplet_index = 0;
            break;
        case 4 :    /* special procedure for palette display */
            if ((element->attributes & DIRECTION_REVERSED) == 0) {
                show_normal_crotchet_triplet_form(xpos, y_offset + pitch_to_y(element->pitch));
                show_form(xpos-2, 48, triplet_indication_noml_4_form, 0x00);
            } else {
                show_reversed_crotchet_triplet_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
                show_form(xpos-2, 22, triplet_indication_revs_4_form, 0x00);
            }
            former_element = element;
            triplet_index = 0;
            break;
    }
}

void show_crotchet_triplet_note_b(struct Score_element *element, int index) {

    int xpos = element->position - position_offset;
    int former_xpos, width;
    int y_offset = 52;

    switch (bass_cref) {
        case G_CREF : y_offset = 52; break;
        case F_CREF : y_offset = 56; break;
    }

    switch (triplet_index) {
        case 0 :
            set_triplet_direction_and_ypos_b(element, index);
            if (triplet_is_reversed == 0) {
                show_normal_crotchet_triplet_form(xpos, y_offset + pitch_to_y(element->pitch));
                show_form(xpos, triplet_indication_ypos, triplet_indication_noml_1_form, 0x00);
            } else {
                show_reversed_crotchet_triplet_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
                show_form(xpos, triplet_indication_ypos, triplet_indication_revs_1_form, 0x00);
            }
            former_element = element;
            triplet_index = 1;
            break;
        case 1 :
            former_xpos = former_element->position - position_offset;
            width = xpos - former_xpos - 7;
            if (triplet_is_reversed == 0) {
                show_normal_crotchet_triplet_form(xpos, y_offset + pitch_to_y(element->pitch));
                show_form(xpos-2, triplet_indication_ypos, triplet_indication_noml_2_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+5, width, triplet_indication_ypos+3, 0x00);
            } else {
                show_reversed_crotchet_triplet_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
                show_form(xpos-2, triplet_indication_ypos, triplet_indication_revs_2_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+5, width, triplet_indication_ypos+3, 0x00);
            }
            former_element = element;
            triplet_index = 2;
            break;
        case 2 :
            former_xpos = former_element->position - position_offset;
            width = xpos - former_xpos - 7;
            if (triplet_is_reversed == 0) {
                show_normal_crotchet_triplet_form(xpos, y_offset + pitch_to_y(element->pitch));
                show_form(xpos-6, triplet_indication_ypos, triplet_indication_noml_3_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+7, width, triplet_indication_ypos+3, 0x00);
            } else {
                show_reversed_crotchet_triplet_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
                show_form(xpos-6, triplet_indication_ypos, triplet_indication_revs_3_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+7, width, triplet_indication_ypos+3, 0x00);
            }
            triplet_index = 0;
            break;
        case 4 :    /* special procedure for palette display */
            if ((element->attributes & DIRECTION_REVERSED) == 0) {
                show_normal_crotchet_triplet_form(xpos, y_offset + pitch_to_y(element->pitch));
                show_form(xpos-2, 98, triplet_indication_noml_4_form, 0x00);
            } else {
                show_reversed_crotchet_triplet_form(xpos, y_offset + pitch_to_y(element->pitch) + 6);
                show_form(xpos-2, 72, triplet_indication_revs_4_form, 0x00);
            }
            former_element = element;
            triplet_index = 0;
            break;
    }
}


/* show tie */

void show_tie_t(struct Score_element *element) {
    int xpos = element->position - position_offset;
    int y_cref_offset = 0;
    int former_xpos, width, ypos;

    switch (treble_cref) {
        case G_CREF : y_cref_offset = 0; break;
        case F_CREF : y_cref_offset = 4; break;
    }

    if (tie_display_index == 1) {
        former_xpos = former_element->position - position_offset;
        width = xpos - former_xpos - 12;

        if ((former_element->attributes & TIE_UPPER) != 0) {

            ypos = -2 + pitch_to_y(element->pitch) + y_cref_offset;
            show_form(former_xpos+3, ypos+7, tie_top_left_s_form, 0x00);
            if (width > -6) {    /* _s_ means "small" tie form */
                show_form(former_xpos+3, ypos+7, tie_top_left_form, 0x00);
                show_form(xpos-9, ypos+7, tie_top_right_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+9, width, ypos+11, 0x00);
                else show_horizontal_line(former_xpos+9, width, ypos+12, 0x00);
            } else show_form(xpos-9, ypos+7, tie_top_right_s_form, 0x00);

        } else {

            ypos = -8 + pitch_to_y(element->pitch) + y_cref_offset;
            show_form(former_xpos+3, ypos+28, tie_bottom_left_s_form, 0x00);
            if (width > -6) {    /* _s_ means "small" tie form */
                show_form(former_xpos+3, ypos+28, tie_bottom_left_form, 0x00);
                show_form(xpos-9, ypos+28, tie_bottom_right_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+9, width, ypos+31, 0x00);
                else show_horizontal_line(former_xpos+9, width, ypos+30, 0x00);
            } else show_form(xpos-9, ypos+28, tie_bottom_right_s_form, 0x00);

        }
    }

    if ((element->attributes & TIE_SET) != 0) {
        tie_display_index = 1;
        former_element = element;
    } else {
        tie_display_index = 0;
    }
}


void show_tie_b(struct Score_element *element) {
    int xpos = element->position - position_offset;
    int y_cref_offset = 0;
    int former_xpos, width, ypos;

    switch (bass_cref) {
        case G_CREF : y_cref_offset = 0; break;
        case F_CREF : y_cref_offset = 4; break;
    }

    if (tie_display_index == 1) {
        former_xpos = former_element->position - position_offset;
        width = xpos - former_xpos - 12;

        if ((former_element->attributes & TIE_UPPER) != 0) {

            ypos = 48 + pitch_to_y(element->pitch) + y_cref_offset;
            show_form(former_xpos+3, ypos+7, tie_top_left_s_form, 0x00);
            if (width > -6) {    /* _s_ means "small" tie form */
                show_form(former_xpos+3, ypos+7, tie_top_left_form, 0x00);
                show_form(xpos-9, ypos+7, tie_top_right_form, 0x00);
                if (width > -2) show_horizontal_line(former_xpos+9, width, ypos+11, 0x00);
                else show_horizontal_line(former_xpos+9, width, ypos+12, 0x00);
            } else show_form(xpos-9, ypos+7, tie_top_right_s_form, 0x00);

        } else {

            ypos = 42 + pitch_to_y(element->pitch) + y_cref_offset;
            show_form(former_xpos+3, ypos+28, tie_bottom_left_s_form, 0x00);
            if (width > -6) {    /* _s_ means "small" tie form */
                show_form(former_xpos+3, ypos+28, tie_bottom_left_form, 0x00);
                show_form(xpos-9, ypos+28, tie_bottom_right_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+9, width, ypos+31, 0x00);
                else show_horizontal_line(former_xpos+9, width, ypos+30, 0x00);
            } else show_form(xpos-9, ypos+28, tie_bottom_right_s_form, 0x00);

        }
    }

    if ((element->attributes & TIE_SET) != 0) {
        tie_display_index = 1;
        former_element = element;
    } else {
        tie_display_index = 0;
    }
}


/* show note */

void show_note(struct Score_element *element, int index) {

    show_tie_t(element);

    switch(element->type) {
        case MINIM_TRIPLET :
            show_minim_triplet_note(element, index);
            return;
        case CROTCHET_TRIPLET :
            show_crotchet_triplet_note(element, index);
            return;
    }

    if ((element->position - position_offset) < -40) return;
    if (224 < (element->position - position_offset)) return;

    switch(element->type) {
        case WHOLE :
            show_whole_note(element);
            return;
        case DOTTED_WHOLE :
            show_dotted_whole_note(element);
            return;
        case MINIM :
            show_minim_note(element);
            return;
        case DOTTED_MINIM :
            show_dotted_minim_note(element);
            return;
        case CROTCHET :
            show_crotchet_note(element);
            return;
        case DOTTED_CROTCHET :
            show_dotted_crotchet_note(element);
            return;
        case QUAVER :
            show_quaver_note(element);
            return;
        case DOTTED_QUAVER :
            show_dotted_quaver_note(element);
            return;
        case SIXTEENTH :
            show_sixteenth_note(element);
            return;
        case DOTTED_SIXTEENTH :
            show_dotted_sixteenth_note(element);
            return;
        case THIRTY_SECOND :
            show_thirty_second_note(element);
            return;
    }
    return;
}

void show_note_b(struct Score_element *element, int index) {

    show_tie_b(element);

    switch(element->type) {
        case MINIM_TRIPLET :
            show_minim_triplet_note_b(element, index);
            return;
        case CROTCHET_TRIPLET :
            show_crotchet_triplet_note_b(element, index);
            return;
    }

    if ((element->position - position_offset) < -40) return;
    if (224 < (element->position - position_offset)) return;

    switch(element->type) {
        case WHOLE :
            show_whole_note_b(element);
            return;
        case DOTTED_WHOLE :
            show_dotted_whole_note_b(element);
            return;
        case MINIM :
            show_minim_note_b(element);
            return;
        case DOTTED_MINIM :
            show_dotted_minim_note_b(element);
            return;
        case CROTCHET :
            show_crotchet_note_b(element);
            return;
        case DOTTED_CROTCHET :
            show_dotted_crotchet_note_b(element);
            return;
        case QUAVER :
            show_quaver_note_b(element);
            return;
        case DOTTED_QUAVER :
            show_dotted_quaver_note_b(element);
            return;
        case SIXTEENTH :
            show_sixteenth_note_b(element);
            return;
        case DOTTED_SIXTEENTH :
            show_dotted_sixteenth_note_b(element);
            return;
        case THIRTY_SECOND :
            show_thirty_second_note_b(element);
            return;
    }
    return;
}


/* rest */

/* minim triplet rest */

void show_minim_triplet_rest(struct Score_element *element, int index) {

    int xpos = element->position - position_offset;
    int former_xpos, width;

    switch (triplet_index) {
        case 0 :
            set_triplet_direction_and_ypos(element, index);
            if (triplet_is_reversed == 0) {
                show_form(xpos, 30, crotchet_rest_form, 0x00);
                show_form(xpos, triplet_indication_ypos, triplet_indication_noml_1_form, 0x00);
            } else {
                show_form(xpos, 30, crotchet_rest_form, 0x00);
                show_form(xpos, triplet_indication_ypos, triplet_indication_revs_1_form, 0x00);
            }
            former_element = element;
            triplet_index = 1;
            break;
        case 1 :
            former_xpos = former_element->position - position_offset;
            width = xpos - former_xpos - 7;
            if (triplet_is_reversed == 0) {
                show_form(xpos, 30, crotchet_rest_form, 0x00);
                show_form(xpos-2, triplet_indication_ypos, triplet_indication_noml_2_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+5, width, triplet_indication_ypos+3, 0x00);
            } else {
                show_form(xpos, 30, crotchet_rest_form, 0x00);
                show_form(xpos-2, triplet_indication_ypos, triplet_indication_revs_2_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+5, width, triplet_indication_ypos+3, 0x00);
            }
            former_element = element;
            triplet_index = 2;
            break;
        case 2 :
            former_xpos = former_element->position - position_offset;
            width = xpos - former_xpos - 7;
            if (triplet_is_reversed == 0) {
                show_form(xpos, 30, crotchet_rest_form, 0x00);
                show_form(xpos-6, triplet_indication_ypos, triplet_indication_noml_3_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+7, width, triplet_indication_ypos+3, 0x00);
            } else {
                show_form(xpos, 30, crotchet_rest_form, 0x00);
                show_form(xpos-6, triplet_indication_ypos, triplet_indication_revs_3_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+7, width, triplet_indication_ypos+3, 0x00);
            }
            triplet_index = 0;
            break;
        case 4 :    /* special procedure for palette display */
            if ((element->attributes & DIRECTION_REVERSED) == 0) {
                show_form(xpos, 30, crotchet_rest_form, 0x00);
                show_form(xpos-2, 48, triplet_indication_noml_4_form, 0x00);
            } else {
                show_form(xpos, 30, crotchet_rest_form, 0x00);
                show_form(xpos-2, 22, triplet_indication_revs_4_form, 0x00);
            }
            former_element = element;
            triplet_index = 0;
            break;
    }
}

void show_minim_triplet_rest_b(struct Score_element *element, int index) {

    int xpos = element->position - position_offset;
    int former_xpos, width;

    switch (triplet_index) {
        case 0 :
            set_triplet_direction_and_ypos_b(element, index);
            if (triplet_is_reversed == 0) {
                show_form(xpos, 80, crotchet_rest_form, 0x00);
                show_form(xpos, triplet_indication_ypos, triplet_indication_noml_1_form, 0x00);
            } else {
                show_form(xpos, 80, crotchet_rest_form, 0x00);
                show_form(xpos, triplet_indication_ypos, triplet_indication_revs_1_form, 0x00);
            }
            former_element = element;
            triplet_index = 1;
            break;
        case 1 :
            former_xpos = former_element->position - position_offset;
            width = xpos - former_xpos - 7;
            if (triplet_is_reversed == 0) {
                show_form(xpos, 80, crotchet_rest_form, 0x00);
                show_form(xpos-2, triplet_indication_ypos, triplet_indication_noml_2_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+5, width, triplet_indication_ypos+3, 0x00);
            } else {
                show_form(xpos, 80, crotchet_rest_form, 0x00);
                show_form(xpos-2, triplet_indication_ypos, triplet_indication_revs_2_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+5, width, triplet_indication_ypos+3, 0x00);
            }
            former_element = element;
            triplet_index = 2;
            break;
        case 2 :
            former_xpos = former_element->position - position_offset;
            width = xpos - former_xpos - 7;
            if (triplet_is_reversed == 0) {
                show_form(xpos, 80, crotchet_rest_form, 0x00);
                show_form(xpos-6, triplet_indication_ypos, triplet_indication_noml_3_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+7, width, triplet_indication_ypos+3, 0x00);
            } else {
                show_form(xpos, 80, crotchet_rest_form, 0x00);
                show_form(xpos-6, triplet_indication_ypos, triplet_indication_revs_3_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+7, width, triplet_indication_ypos+3, 0x00);
            }
            triplet_index = 0;
            break;
        case 4 :    /* special procedure for palette display */
            if ((element->attributes & DIRECTION_REVERSED) == 0) {
                show_form(xpos, 80, crotchet_rest_form, 0x00);
                show_form(xpos-2, 98, triplet_indication_noml_4_form, 0x00);
            } else {
                show_form(xpos, 80, crotchet_rest_form, 0x00);
                show_form(xpos-2, 72, triplet_indication_revs_4_form, 0x00);
            }
            former_element = element;
            triplet_index = 0;
            break;
    }
}


/* show crotchet triplet rest */

void show_crotchet_triplet_rest(struct Score_element *element, int index) {

    int xpos = element->position - position_offset;
    int former_xpos, width;

    switch (triplet_index) {
        case 0 :
            set_triplet_direction_and_ypos(element, index);
            if (triplet_is_reversed == 0) {
                show_form(xpos, 30, quaver_rest_form, 0x00);
                show_form(xpos, triplet_indication_ypos, triplet_indication_noml_1_form, 0x00);
            } else {
                show_form(xpos, 30, quaver_rest_form, 0x00);
                show_form(xpos, triplet_indication_ypos, triplet_indication_revs_1_form, 0x00);
            }
            former_element = element;
            triplet_index = 1;
            break;
        case 1 :
            former_xpos = former_element->position - position_offset;
            width = xpos - former_xpos - 7;
            if (triplet_is_reversed == 0) {
                show_form(xpos, 30, quaver_rest_form, 0x00);
                show_form(xpos-2, triplet_indication_ypos, triplet_indication_noml_2_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+5, width, triplet_indication_ypos+3, 0x00);
            } else {
                show_form(xpos, 30, quaver_rest_form, 0x00);
                show_form(xpos-2, triplet_indication_ypos, triplet_indication_revs_2_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+5, width, triplet_indication_ypos+3, 0x00);
            }
            former_element = element;
            triplet_index = 2;
            break;
        case 2 :
            former_xpos = former_element->position - position_offset;
            width = xpos - former_xpos - 7;
            if (triplet_is_reversed == 0) {
                show_form(xpos, 30, quaver_rest_form, 0x00);
                show_form(xpos-6, triplet_indication_ypos, triplet_indication_noml_3_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+7, width, triplet_indication_ypos+3, 0x00);
            } else {
                show_form(xpos, 30, quaver_rest_form, 0x00);
                show_form(xpos-6, triplet_indication_ypos, triplet_indication_revs_3_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+7, width, triplet_indication_ypos+3, 0x00);
            }
            triplet_index = 0;
            break;
        case 4 :    /* special procedure for palette display */
            if ((element->attributes & DIRECTION_REVERSED) == 0) {
                show_form(xpos, 30, quaver_rest_form, 0x00);
                show_form(xpos-2, 48, triplet_indication_noml_4_form, 0x00);
            } else {
                show_form(xpos, 30, quaver_rest_form, 0x00);
                show_form(xpos-2, 22, triplet_indication_revs_4_form, 0x00);
            }
            former_element = element;
            triplet_index = 0;
            break;
    }
}

void show_crotchet_triplet_rest_b(struct Score_element *element, int index) {

    int xpos = element->position - position_offset;
    int former_xpos, width;

    switch (triplet_index) {
        case 0 :
            set_triplet_direction_and_ypos_b(element, index);
            if (triplet_is_reversed == 0) {
                show_form(xpos, 80, quaver_rest_form, 0x00);
                show_form(xpos, triplet_indication_ypos, triplet_indication_noml_1_form, 0x00);
            } else {
                show_form(xpos, 80, quaver_rest_form, 0x00);
                show_form(xpos, triplet_indication_ypos, triplet_indication_revs_1_form, 0x00);
            }
            former_element = element;
            triplet_index = 1;
            break;
        case 1 :
            former_xpos = former_element->position - position_offset;
            width = xpos - former_xpos - 7;
            if (triplet_is_reversed == 0) {
                show_form(xpos, 80, quaver_rest_form, 0x00);
                show_form(xpos-2, triplet_indication_ypos, triplet_indication_noml_2_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+5, width, triplet_indication_ypos+3, 0x00);
            } else {
                show_form(xpos, 80, quaver_rest_form, 0x00);
                show_form(xpos-2, triplet_indication_ypos, triplet_indication_revs_2_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+5, width, triplet_indication_ypos+3, 0x00);
            }
            former_element = element;
            triplet_index = 2;
            break;
        case 2 :
            former_xpos = former_element->position - position_offset;
            width = xpos - former_xpos - 7;
            if (triplet_is_reversed == 0) {
                show_form(xpos, 80, quaver_rest_form, 0x00);
                show_form(xpos-6, triplet_indication_ypos, triplet_indication_noml_3_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+7, width, triplet_indication_ypos+3, 0x00);
            } else {
                show_form(xpos, 80, quaver_rest_form, 0x00);
                show_form(xpos-6, triplet_indication_ypos, triplet_indication_revs_3_form, 0x00);
                if (width > 2) show_horizontal_line(former_xpos+7, width, triplet_indication_ypos+3, 0x00);
            }
            triplet_index = 0;
            break;
        case 4 :    /* special procedure for palette display */
            if ((element->attributes & DIRECTION_REVERSED) == 0) {
                show_form(xpos, 80, quaver_rest_form, 0x00);
                show_form(xpos-2, 98, triplet_indication_noml_4_form, 0x00);
            } else {
                show_form(xpos, 80, quaver_rest_form, 0x00);
                show_form(xpos-2, 72, triplet_indication_revs_4_form, 0x00);
            }
            former_element = element;
            triplet_index = 0;
            break;
    }
}


void show_rest(struct Score_element *element, int index) {

    int xpos = element->position - position_offset;

    switch(element->type) {
        case WHOLE : 
            show_form(xpos, 34, whole_rest_form, 0x00);
            return;
        case DOTTED_WHOLE : 
            show_form(xpos, 34, dotted_whole_rest_form, 0x00);
            return;
        case MINIM :
            show_form(xpos, 36, minim_rest_form, 0x00);
            return;
        case DOTTED_MINIM :
            show_form(xpos, 36, dotted_minim_rest_form, 0x00);
            return;
        case CROTCHET :
            show_form(xpos, 30, crotchet_rest_form, 0x00);
            return;
        case DOTTED_CROTCHET :
            show_form(xpos, 30, dotted_crotchet_rest_form, 0x00);
            return;
        case QUAVER       : 
            show_form(xpos, 30, quaver_rest_form, 0x00);
            return;
        case DOTTED_QUAVER :
            show_form(xpos, 30, dotted_quaver_rest_form, 0x00);
            return;
        case SIXTEENTH : 
            show_form(xpos, 30, sixteenth_rest_form, 0x00);
            return;
        case DOTTED_SIXTEENTH : 
            show_form(xpos, 30, dotted_sixteenth_rest_form, 0x00);
            return;
        case THIRTY_SECOND : 
            show_form(xpos, 30, thirty_second_rest_form, 0x00);
            return;
        case MINIM_TRIPLET :
            show_minim_triplet_rest(element, index);
            return;
        case CROTCHET_TRIPLET :
            show_crotchet_triplet_rest(element, index);
            return;
    }
}

void show_rest_b(struct Score_element *element, int index) {

    int xpos = element->position - position_offset;

    switch(element->type) {
        case WHOLE : 
            show_form(xpos, 84, whole_rest_form, 0x00);
            return;
        case DOTTED_WHOLE : 
            show_form(xpos, 84, dotted_whole_rest_form, 0x00);
            return;
        case MINIM :
            show_form(xpos, 86, minim_rest_form, 0x00);
            return;
        case DOTTED_MINIM :
            show_form(xpos, 86, dotted_minim_rest_form, 0x00);
            return;
        case CROTCHET :
            show_form(xpos, 80, crotchet_rest_form, 0x00);
            return;
        case DOTTED_CROTCHET :
            show_form(xpos, 80, dotted_crotchet_rest_form, 0x00);
            return;
        case QUAVER       : 
            show_form(xpos, 80, quaver_rest_form, 0x00);
            return;
        case DOTTED_QUAVER :
            show_form(xpos, 80, dotted_quaver_rest_form, 0x00);
            return;
        case SIXTEENTH : 
            show_form(xpos, 80, sixteenth_rest_form, 0x00);
            return;
        case DOTTED_SIXTEENTH : 
            show_form(xpos, 80, dotted_sixteenth_rest_form, 0x00);
            return;
        case THIRTY_SECOND : 
            show_form(xpos, 80, thirty_second_rest_form, 0x00);
            return;
        case MINIM_TRIPLET :
            show_minim_triplet_rest_b(element, index);
            return;
        case CROTCHET_TRIPLET :
            show_crotchet_triplet_rest_b(element, index);
            return;
    }
}


/* show bar section mark */

void show_bar_t(struct Score_element *element) {

    if (element->type == BAR_DOUBLE) {
        /* double */
        show_vertical_line(element->position - position_offset, 30, 47, 0x03);
        show_vertical_line(element->position - position_offset +2, 30, 47, 0x03);
    } else {
        /* single */
        show_vertical_line(element->position - position_offset +1, 30, 47, 0x03);
    }
}

void show_bar_b(struct Score_element *element) {

    if (element->type == BAR_DOUBLE) {
        /* double */
        show_vertical_line(element->position - position_offset, 80, 97, 0x03);
        show_vertical_line(element->position - position_offset +2, 80, 97, 0x03);
    } else {
        /* single */
        show_vertical_line(element->position - position_offset +1, 80, 97, 0x03);
    }
}


/* show score */

void show_element_t(struct Score_element *element, int index) {

    switch(element->category) {
        case SCORE_CREF  : show_cref(element); return;
        case SCORE_KEY   : show_key(element); return;
        case SCORE_RITHM : show_rithm(element); return;
        case SCORE_NOTE  : show_note(element, index); return;
        case SCORE_REST  : show_rest(element, index); return;
        case SCORE_SIG   : show_key_signature(element); return;
        case SCORE_BAR   : show_bar_t(element); return;
    }
}

void show_element_b(struct Score_element *element, int index) {

    switch(element->category) {
        case SCORE_CREF  : show_cref_b(element); return;
        case SCORE_KEY   : show_key_b(element); return;
        case SCORE_RITHM : show_rithm_b(element); return;
        case SCORE_NOTE  : show_note_b(element, index); return;
        case SCORE_REST  : show_rest_b(element, index); return;
        case SCORE_SIG   : show_key_signature_b(element); return;
        case SCORE_BAR   : show_bar_b(element); return;
    }
}


void show_pentagramma_t(int from_x, int width) {

    int i;
    int y = 30;

    for(i=0; i<5; i++) {
        show_horizontal_line(from_x, width, y, 0x03);
        y = y+4;
    }
}

void show_pentagramma_b(int from_x, int width) {

    int i;
    int y = 80;

    for(i=0; i<5; i++) {
        show_horizontal_line(from_x, width, y, 0x03);
        y = y+4;
    }
}


void show_score_t() {

    int i;

    triplet_index = 0;
    tie_display_index = 0;

    /* show pentagramma */
    show_pentagramma_t(5 - position_offset, score_end_position_t - 5);

    /* show each element */
    for (i=0; i < no_of_elements_t; i++) show_element_t(&score_t[i], i);

    /* show end of score */
    show_vertical_line(score_end_position_t - position_offset, 30, 47, 0x03);
    show_vertical_line(score_end_position_t-1 - position_offset, 30, 47, 0x03);
    show_vertical_line(score_end_position_t-3 - position_offset, 30, 47, 0x03);
}

void show_score_b() {

    int i;

    triplet_index = 0;
    tie_display_index = 0;

    /* show pentagramma */
    show_pentagramma_b(5 - position_offset, score_end_position_b - 5);

    /* show each element */
    for (i=0; i < no_of_elements_b; i++) show_element_b(&score_b[i], i);

    /* show end of score */
    show_vertical_line(score_end_position_b - position_offset, 80, 97, 0x03);
    show_vertical_line(score_end_position_b-1 - position_offset, 80, 97, 0x03);
    show_vertical_line(score_end_position_b-3 - position_offset, 80, 97, 0x03);
}

void show_score() {
    show_score_t();
    show_score_b();
}


/* show_all */
void show_all() {
    clear_screen();
    show_score();
    paint_score_selections();
    wwc_font_set_colordata(0, 28*18, buffer);
}


/* durations */

int display_duration_t(struct Score_element *element) {
    switch (element->category) {
        case SCORE_NOTE :
            return element->type; /* elment type ̎Ԃ duration Œ` */
        case SCORE_REST :
            if (element->type == WHOLE) {
                switch (treble_rithm) {
                    case BINARIA      : return 2*960;    /*  2/2  */
                    case TERNARIA     : return 3*480;    /*  3/4  */
                    case QUATERNARIA  : return 4*480;    /*  4/4  */
                    case CROME        : return 6*240;    /*  6/8  */
                    case SEMICROME    : return 9*120;    /*  9/16 */
                    case TWOFOURTH    : return 2*480;    /*  2/4  */
                    case TWELVEEIGHTH : return 12*240;   /*  12/8  */
                }
            } else {
                return element->type; /* elment type ̎Ԃ duration Œ` */
            }
    }
    return 0;
}

int display_duration_b(struct Score_element *element) {
     switch (element->category) {
        case SCORE_NOTE :
            return element->type; /* elment type ̎Ԃ duration Œ` */
        case SCORE_REST :
            if (element->type == WHOLE) {
                switch (bass_rithm) {
                    case BINARIA      : return 2*960;    /*  2/2  */
                    case TERNARIA     : return 3*480;    /*  3/4  */
                    case QUATERNARIA  : return 4*480;    /*  4/4  */
                    case CROME        : return 6*240;    /*  6/8  */
                    case SEMICROME    : return 9*120;    /*  9/16 */
                    case TWOFOURTH    : return 2*480;    /*  2/4  */
                    case TWELVEEIGHTH : return 12*240;  /*  12/8  */
                }
            } else {
                return element->type; /* elment type ̎Ԃ duration Œ` */
            }
    }
    return 0;
}


int play_duration_t(struct Score_element *element) {
    switch (element->category) {
        case SCORE_NOTE :
            return element->type; /* elment type ̎Ԃ duration Œ` */
        case SCORE_REST :
            if (element->type == WHOLE) {
                switch (treble_rithm_play) {
                    case BINARIA      : return 2*960;    /*  2/2  */
                    case TERNARIA     : return 3*480;    /*  3/4  */
                    case QUATERNARIA  : return 4*480;    /*  4/4  */
                    case CROME        : return 6*240;    /*  6/8  */
                    case SEMICROME    : return 9*120;    /*  9/16 */
                    case TWOFOURTH    : return 2*480;    /*  2/4  */
                    case TWELVEEIGHTH : return 12*240;   /*  12/8  */
                }
            } else {
                return element->type; /* elment type ̎Ԃ duration Œ` */
            }
    }
    return 0;
}

int play_duration_b(struct Score_element *element) {
     switch (element->category) {
        case SCORE_NOTE :
            return element->type; /* elment type ̎Ԃ duration Œ` */
        case SCORE_REST :
            if (element->type == WHOLE) {
                switch (bass_rithm_play) {
                    case BINARIA      : return 2*960;    /*  2/2  */
                    case TERNARIA     : return 3*480;    /*  3/4  */
                    case QUATERNARIA  : return 4*480;    /*  4/4  */
                    case CROME        : return 6*240;    /*  6/8  */
                    case SEMICROME    : return 9*120;    /*  9/16 */
                    case TWOFOURTH    : return 2*480;    /*  2/4  */
                    case TWELVEEIGHTH : return 12*240;   /*  12/8  */
                }
            } else {
                return element->type; /* elment type ̎Ԃ duration Œ` */
            }
    }
    return 0;
}


/* adjust element positions */


void adjust_positions() {

    int i_t_1 = 0;  /* index treble 1st pass */
    int i_b_1 = 0;  /* index bass 1st pass */
    int notes_and_rests_t; /* no of notest and rests within the bar */
    int bar_width_t;  /* width of bar */
    int duration_t;  /* total duration of notes and rests within the bar */
    int notes_and_rests_b;
    int bar_width_b;
    int duration_b;
    int bar_width;
    int offset_t;
    int offset_b;
    int i_t_2 = 0;  /* index treble 2nd pass */
    int i_b_2 = 0;  /* index treble 2nd pass */
    int current_bar_pos = 5; /* current bar position (same in treble and bass) */
    int current_pos_t = 5; /* current element position */
    int current_pos_b = 5;
    int element_width;

    while ((i_t_1 < no_of_elements_t) || (i_b_1 < no_of_elements_b)) {

        /* check no of notes and rests and width within the bar in treble score */
        notes_and_rests_t = 0;
        bar_width_t = 0;
        duration_t = 0;
        while ((i_t_1 < no_of_elements_t) && (score_t[i_t_1].category != SCORE_BAR)) {
            switch (score_t[i_t_1].category) {
                case SCORE_NOTE : case SCORE_REST :
                    notes_and_rests_t++;
                    duration_t += display_duration_t(&score_t[i_t_1]);
                    break;
            }
            bar_width_t += element_width_for_adj(&score_t[i_t_1]);
            i_t_1++;
        }
        i_t_1++;

        /* check no of notes and rests and width of bar in bass score */
        notes_and_rests_b = 0;
        bar_width_b = 0;
        duration_b = 0;
        while ((i_b_1 < no_of_elements_b) && (score_b[i_b_1].category != SCORE_BAR)) {
            switch (score_b[i_b_1].category) {
                case SCORE_NOTE : case SCORE_REST :
                    notes_and_rests_b++;
                    duration_b += display_duration_b(&score_b[i_b_1]);
                    break;
            }
            bar_width_b += element_width_for_adj(&score_b[i_b_1]);
            i_b_1++;
        }
        i_b_1++;

        /* compare width of treble and bass bar */
        if (bar_width_t > bar_width_b) {
            bar_width = bar_width_t;
            offset_t = 0;
            if (duration_t > duration_b) 
                offset_b = (bar_width_t - bar_width_b) * 6 / notes_and_rests_b
                    * duration_b / duration_t / 6;
            else offset_b = (bar_width_t - bar_width_b) / notes_and_rests_b;
        } else {
            bar_width = bar_width_b;
            if (duration_b > duration_t)
                offset_t = (bar_width_b - bar_width_t) * 6 / notes_and_rests_t
                    * duration_t / duration_b / 6;
            else offset_t = (bar_width_b - bar_width_t) / notes_and_rests_t;
            offset_b = 0;
        }
        if (bar_width == 0) bar_width = 4;
        current_bar_pos += bar_width;
        current_bar_pos += 8;

        /* set position of elements in treble */
        while ((i_t_2 < no_of_elements_t) && (score_t[i_t_2].category != SCORE_BAR)) {
            switch (score_t[i_t_2].category) {
                case SCORE_NOTE : case SCORE_REST :
                    element_width = element_width_for_adj(&score_t[i_t_2]) + offset_t;
                    score_t[i_t_2].position = current_pos_t + element_width/3;
                    if (1 <= i_t_2)
                        if (score_t[i_t_2 - 1].category == SCORE_SIG) {
                            /* re-adjust for position of key signature */
                            score_t[i_t_2 - 1].position += element_width/3;
                            if (2 <= i_t_2)
                                if (score_t[i_t_2 - 2].category == SCORE_SIG)
                                /* re-adjust for position of key signature */
                                score_t[i_t_2 - 2].position += element_width/3;
                        }
                    current_pos_t += element_width;
                    break;
                default :
                    element_width = element_width_for_adj(&score_t[i_t_2]);
                    score_t[i_t_2].position = current_pos_t;
                    current_pos_t += element_width;
                    break;
            }
            i_t_2++;
        }
        if (no_of_elements_t <= i_t_2) {
            score_end_position_t = current_bar_pos + 6;
        } else {
            score_t[i_t_2].position = current_bar_pos;
            current_pos_t = current_bar_pos + 2;
        }
        i_t_2++;

        /* set position of elements in bass */
        while ((i_b_2 < no_of_elements_b) && (score_b[i_b_2].category != SCORE_BAR)) {
            switch (score_b[i_b_2].category) {
                case SCORE_NOTE : case SCORE_REST :
                    element_width = element_width_for_adj(&score_b[i_b_2]) + offset_b;
                    score_b[i_b_2].position = current_pos_b + element_width/3;
                    if (1 <= i_b_2)
                        if (score_b[i_b_2 - 1].category == SCORE_SIG) {
                            /* re-adjust for position of key signature */
                            score_b[i_b_2 - 1].position += element_width/3;
                            if (2 <= i_b_2)
                                if (score_b[i_b_2 - 2].category == SCORE_SIG)
                                /* re-adjust for position of key signature */
                                score_b[i_b_2 - 2].position += element_width/3;
                        }
                    current_pos_b += element_width;
                    break;
                default :
                    element_width = element_width_for_adj(&score_b[i_b_2]);
                    score_b[i_b_2].position = current_pos_b;
                    current_pos_b += element_width;
                    break;
            }
            i_b_2++;
        }
        if (no_of_elements_b <= i_b_2) {
            score_end_position_b = current_bar_pos + 6;
        } else {
            score_b[i_b_2].position = current_bar_pos;
            current_pos_b = current_bar_pos + 2;
        }
        i_b_2++;

    }
}


/* set last key in bit9-16 of type of key_c_major and key_a_minor */
void set_c_major_and_a_minor() {

    int i;
    int current_key;

    current_key = C_MAJOR;
    for (i=0; i<no_of_elements_t; i++) {
        if (score_t[i].category == SCORE_KEY) {
            switch (score_t[i].type & 0x00ff) {
                case C_MAJOR : case A_MINOR :
                    score_t[i].type = (score_t[i].type & 0x00ff) | ((current_key<<8) & 0xff00);
            }
            current_key = score_t[i].type & 0x00ff;
        }
    }
    current_key = C_MAJOR;
    for (i=0; i<no_of_elements_b; i++) {
        if (score_b[i].category == SCORE_KEY) {
            switch (score_b[i].type & 0x00ff) {
                case C_MAJOR : case A_MINOR :
                    score_b[i].type = (score_b[i].type & 0x00ff) | ((current_key<<8) & 0xff00);
            }
            current_key = score_b[i].type & 0x00ff;
        }
    }
}


/* playing sound */

void play_simple_sound(int pitch_value, int length) {
    sound_set_pitch(CHANNEL1, pitch_value);
    sound_set_volume(CHANNEL1, 0x55);
    vcount_1 = 10;
}

void play_simple_sound_b(int pitch_value, int length) {
    sound_set_pitch(CHANNEL2, pitch_value);
    sound_set_volume(CHANNEL2, 0x55);
    vcount_2 = 10;
}


void play_rest() {
    sound_set_volume(CHANNEL1, 0x00);

}

void play_rest_b() {
    sound_set_volume(CHANNEL2, 0x00);
}


void play_ok_button() {
    sound_set_pitch(CHANNEL3, 1993);  /* PITCH_A7 */
    sound_set_volume(CHANNEL3, 0x22);
    vcount_3 = 3;
}


void play_cancel_button() {
    sound_set_pitch(CHANNEL3, 1939);  /* PITCH_A6 */
    sound_set_volume(CHANNEL3, 0x22);
    vcount_3 = 3;
}


int pitch_value_t(struct Score_element *element) {

    int i;
    int pitch;

    /* adjustment by octave */
    switch (treble_pitch) {
        case PITCH_HIGH :   pitch = element->pitch; break;
        case PITCH_MIDDLE : pitch = element->pitch - 12; break;
        case PITCH_LOW :    pitch = element->pitch - 24; break;
    }

    for (i=0; i<41; i++) {
        if (pitch == bar_pitchs_t[i].name)
            return bar_pitchs_t[i].value;
    }
    return 44; /* default PITCH_C3 */
}

int pitch_value_b(struct Score_element *element) {

    int i;
    int pitch;

    /* adjustment by octave */
    switch (bass_pitch) {
        case PITCH_HIGH :   pitch = element->pitch; break;
        case PITCH_MIDDLE : pitch = element->pitch - 12; break;
        case PITCH_LOW :    pitch = element->pitch - 24; break;
    }

    for (i=0; i<41; i++) {
        if (pitch == bar_pitchs_b[i].name)
            return bar_pitchs_b[i].value;
    }
    return 44; /* default PITCH_C3 */
}


void set_sharp_pitch_t(int pitch) {  /* destination is score pitch table */
    int i;
    int value;

    for (i=0; i<41; i++)
        if (sharp_pitchs[i].name == pitch) value = sharp_pitchs[i].value;
    for (i=0; i<41; i++)
        if (score_pitchs_t[i].name == pitch) score_pitchs_t[i].value = value;
}

void set_sharp_pitch_b(int pitch) {  /* destination is score pitch table */
    int i;
    int value;

    for (i=0; i<41; i++)
        if (sharp_pitchs[i].name == pitch) value = sharp_pitchs[i].value;
    for (i=0; i<41; i++)
        if (score_pitchs_b[i].name == pitch) score_pitchs_b[i].value = value;
}


void set_flat_pitch_t(int pitch) {  /* destination is score pitch table */
    int i;
    int value;

    for (i=0; i<41; i++)
        if (flat_pitchs[i].name == pitch) value = flat_pitchs[i].value;
    for (i=0; i<41; i++)
        if (score_pitchs_t[i].name == pitch) score_pitchs_t[i].value = value;
}

void set_flat_pitch_b(int pitch) {  /* destination is score pitch table */
    int i;
    int value;

    for (i=0; i<41; i++)
        if (flat_pitchs[i].name == pitch) value = flat_pitchs[i].value;
    for (i=0; i<41; i++)
        if (score_pitchs_b[i].name == pitch) score_pitchs_b[i].value = value;
}


/* change pitches for key scale */

void change_score_pitch_t(struct Score_element *element) {
    int i;

    for (i=0; i<41; i++) score_pitchs_t[i] = natural_pitchs[i];
    switch (element-> type) {
        case E_MAJOR :
            set_sharp_pitch_t(PITCH_D3);
            set_sharp_pitch_t(PITCH_D4);
            set_sharp_pitch_t(PITCH_D5);
            set_sharp_pitch_t(PITCH_D6);
            set_sharp_pitch_t(PITCH_D7);
            set_sharp_pitch_t(PITCH_D8);
        case A_MAJOR :
            set_sharp_pitch_t(PITCH_G3);
            set_sharp_pitch_t(PITCH_G4);
            set_sharp_pitch_t(PITCH_G5);
            set_sharp_pitch_t(PITCH_G6);
            set_sharp_pitch_t(PITCH_G7);
            set_sharp_pitch_t(PITCH_G8);
        case D_MAJOR : case B_MINOR :
            set_sharp_pitch_t(PITCH_C3);
            set_sharp_pitch_t(PITCH_C4);
            set_sharp_pitch_t(PITCH_C5);
            set_sharp_pitch_t(PITCH_C6);
            set_sharp_pitch_t(PITCH_C7);
            set_sharp_pitch_t(PITCH_C8); /* D_MAJOR  G_MAJOR ƓV[v̂ break Ȃ */
        case G_MAJOR : case E_MINOR :
            set_sharp_pitch_t(PITCH_F3);
            set_sharp_pitch_t(PITCH_F4);
            set_sharp_pitch_t(PITCH_F5);
            set_sharp_pitch_t(PITCH_F6);
            set_sharp_pitch_t(PITCH_F7);
            set_sharp_pitch_t(PITCH_F8);
            break;
        case D_FLAT_MAJOR : case B_FLAT_MINOR :
            set_flat_pitch_t(PITCH_G3);
            set_flat_pitch_t(PITCH_G4);
            set_flat_pitch_t(PITCH_G5);
            set_flat_pitch_t(PITCH_G6);
            set_flat_pitch_t(PITCH_G7); /* B_FLAT_MAJOR  F_MAJOR Ɠtbĝ break Ȃ */
            set_flat_pitch_t(PITCH_G8);
        case A_FLAT_MAJOR :
            set_flat_pitch_t(PITCH_D3);
            set_flat_pitch_t(PITCH_D4);
            set_flat_pitch_t(PITCH_D5);
            set_flat_pitch_t(PITCH_D6);
            set_flat_pitch_t(PITCH_D7);
            set_flat_pitch_t(PITCH_D8); /* B_FLAT_MAJOR  F_MAJOR Ɠtbĝ break Ȃ */
        case E_FLAT_MAJOR :
            set_flat_pitch_t(PITCH_A3);
            set_flat_pitch_t(PITCH_A4);
            set_flat_pitch_t(PITCH_A5);
            set_flat_pitch_t(PITCH_A6);
            set_flat_pitch_t(PITCH_A7); /* B_FLAT_MAJOR  F_MAJOR Ɠtbĝ break Ȃ */
            set_flat_pitch_t(PITCH_A8);
        case B_FLAT_MAJOR : case G_MINOR :
            set_flat_pitch_t(PITCH_E3);
            set_flat_pitch_t(PITCH_E4);
            set_flat_pitch_t(PITCH_E5);
            set_flat_pitch_t(PITCH_E6);
            set_flat_pitch_t(PITCH_E7);
            set_flat_pitch_t(PITCH_E8); /* B_FLAT_MAJOR  F_MAJOR Ɠtbĝ break Ȃ */
        case F_MAJOR : case D_MINOR :
            set_flat_pitch_t(PITCH_B3);
            set_flat_pitch_t(PITCH_B4);
            set_flat_pitch_t(PITCH_B5);
            set_flat_pitch_t(PITCH_B6);
            set_flat_pitch_t(PITCH_B7);
            break;
    }
}

void change_score_pitch_b(struct Score_element *element) {
    int i;

    for (i=0; i<41; i++) score_pitchs_b[i] = natural_pitchs[i];
    switch (element-> type) {
        case E_MAJOR :
            set_sharp_pitch_b(PITCH_D3);
            set_sharp_pitch_b(PITCH_D4);
            set_sharp_pitch_b(PITCH_D5);
            set_sharp_pitch_b(PITCH_D6);
            set_sharp_pitch_b(PITCH_D7);
            set_sharp_pitch_b(PITCH_D8);
        case A_MAJOR :
            set_sharp_pitch_b(PITCH_G3);
            set_sharp_pitch_b(PITCH_G4);
            set_sharp_pitch_b(PITCH_G5);
            set_sharp_pitch_b(PITCH_G6);
            set_sharp_pitch_b(PITCH_G7);
            set_sharp_pitch_b(PITCH_G8);
        case D_MAJOR : case B_MINOR :
            set_sharp_pitch_b(PITCH_C3);
            set_sharp_pitch_b(PITCH_C4);
            set_sharp_pitch_b(PITCH_C5);
            set_sharp_pitch_b(PITCH_C6);
            set_sharp_pitch_b(PITCH_C7);
            set_sharp_pitch_b(PITCH_C8); /* D_MAJOR  G_MAJOR ƓV[v̂ break Ȃ */
        case G_MAJOR : case E_MINOR :
            set_sharp_pitch_b(PITCH_F3);
            set_sharp_pitch_b(PITCH_F4);
            set_sharp_pitch_b(PITCH_F5);
            set_sharp_pitch_b(PITCH_F6);
            set_sharp_pitch_b(PITCH_F7);
            set_sharp_pitch_b(PITCH_F8);
            break;
        case D_FLAT_MAJOR : case B_FLAT_MINOR :
            set_flat_pitch_b(PITCH_G3);
            set_flat_pitch_b(PITCH_G4);
            set_flat_pitch_b(PITCH_G5);
            set_flat_pitch_b(PITCH_G6);
            set_flat_pitch_b(PITCH_G7); /* B_FLAT_MAJOR  F_MAJOR Ɠtbĝ break Ȃ */
            set_flat_pitch_b(PITCH_G8);
        case A_FLAT_MAJOR :
            set_flat_pitch_b(PITCH_D3);
            set_flat_pitch_b(PITCH_D4);
            set_flat_pitch_b(PITCH_D5);
            set_flat_pitch_b(PITCH_D6);
            set_flat_pitch_b(PITCH_D7);
            set_flat_pitch_b(PITCH_D8); /* B_FLAT_MAJOR  F_MAJOR Ɠtbĝ break Ȃ */
        case E_FLAT_MAJOR :
            set_flat_pitch_b(PITCH_A3);
            set_flat_pitch_b(PITCH_A4);
            set_flat_pitch_b(PITCH_A5);
            set_flat_pitch_b(PITCH_A6);
            set_flat_pitch_b(PITCH_A7); /* B_FLAT_MAJOR  F_MAJOR Ɠtbĝ break Ȃ */
            set_flat_pitch_b(PITCH_A8);
        case B_FLAT_MAJOR : case G_MINOR :
            set_flat_pitch_b(PITCH_E3);
            set_flat_pitch_b(PITCH_E4);
            set_flat_pitch_b(PITCH_E5);
            set_flat_pitch_b(PITCH_E6);
            set_flat_pitch_b(PITCH_E7);
            set_flat_pitch_b(PITCH_E8); /* B_FLAT_MAJOR  F_MAJOR Ɠtbĝ break Ȃ */
        case F_MAJOR : case D_MINOR :
            set_flat_pitch_b(PITCH_B3);
            set_flat_pitch_b(PITCH_B4);
            set_flat_pitch_b(PITCH_B5);
            set_flat_pitch_b(PITCH_B6);
            set_flat_pitch_b(PITCH_B7);
            break;
    }
}


/* change bar pitches for key signeture */

void change_bar_pitch_t(struct Score_element *element) {
    int i;
    int pitch;
    int value;

    /* adjust by octave */
    switch (treble_pitch) {
        case PITCH_HIGH :   pitch = element->pitch; break;
        case PITCH_MIDDLE : pitch = element->pitch - 12; break;
        case PITCH_LOW :    pitch = element->pitch - 24; break;
    }

    switch (element->type) {
        case SIG_SHARP :
            for (i=0; i<41; i++)
                if (sharp_pitchs[i].name == pitch)
                    value = sharp_pitchs[i].value;
            break;
        case SIG_FLAT :
            for (i=0; i<41; i++)
                if (flat_pitchs[i].name == pitch)
                    value = flat_pitchs[i].value;
            break;
        case SIG_NATURAL :
            for (i=0; i<41; i++)
                if (natural_pitchs[i].name == pitch)
                    value = natural_pitchs[i].value;
            break;
        case SIG_DOUBLE_SHARP :
            for (i=0; i<41; i++)
                if (double_sharp_pitchs[i].name == pitch)
                    value = double_sharp_pitchs[i].value;
            break;
        case SIG_DOUBLE_FLAT :
            for (i=0; i<41; i++)
                if (double_flat_pitchs[i].name == pitch)
                    value = double_flat_pitchs[i].value;
            break;
    }
    for (i=0; i<41; i++)
        if (bar_pitchs_t[i].name == pitch)
            bar_pitchs_t[i].value = value;

}

void change_bar_pitch_b(struct Score_element *element) {
    int i;
    int pitch;
    int value;

    /* adjust by octave */
    switch (bass_pitch) {
        case PITCH_HIGH :   pitch = element->pitch; break;
        case PITCH_MIDDLE : pitch = element->pitch - 12; break;
        case PITCH_LOW :    pitch = element->pitch - 24; break;
    }

    switch (element->type) {
        case SIG_SHARP :
            for (i=0; i<41; i++)
                if (sharp_pitchs[i].name == pitch)
                    value = sharp_pitchs[i].value;
            break;
        case SIG_FLAT :
            for (i=0; i<41; i++)
                if (flat_pitchs[i].name == pitch)
                    value = flat_pitchs[i].value;
            break;
        case SIG_NATURAL :
            for (i=0; i<41; i++)
                if (natural_pitchs[i].name == pitch)
                    value = natural_pitchs[i].value;
            break;
        case SIG_DOUBLE_SHARP :
            for (i=0; i<41; i++)
                if (double_sharp_pitchs[i].name == pitch)
                    value = double_sharp_pitchs[i].value;
            break;
        case SIG_DOUBLE_FLAT :
            for (i=0; i<41; i++)
                if (double_flat_pitchs[i].name == pitch)
                    value = double_flat_pitchs[i].value;
            break;
    }
    for (i=0; i<41; i++)
        if (bar_pitchs_b[i].name == pitch)
            bar_pitchs_b[i].value = value;
}


/* update playing selection */

void update_play_selection() {

    int sel_position;

    if (play_selection_t != shown_play_selection_t) {
        switch (score_t[shown_play_selection_t].category) {
           case SCORE_NOTE : case SCORE_REST :
           restore_play_selection_t();
        }
        shown_play_selection_t = play_selection_t;

        /* show new play selection if play continue */
        if ((tick_switch != 0) && (play_selection_t < no_of_elements_t)) {

            /* scroll */
            sel_position = score_t[shown_play_selection_t].position;
            if (sel_position - position_offset + 20 >= 224) {
                position_offset = sel_position - 224 + 200;
                if (position_offset > score_end_position_t - 224 + 5)
                    position_offset = score_end_position_t - 224 + 5;
                clear_screen();
                show_score();
            }

            switch (score_t[shown_play_selection_t].category) {
                case SCORE_NOTE : case SCORE_REST :
                    paint_play_selection_t(); break;
            }
        }
    }

    if (play_selection_b != shown_play_selection_b) {
        switch (score_b[shown_play_selection_b].category) {
           case SCORE_NOTE : case SCORE_REST :
           restore_play_selection_b();
        }
        shown_play_selection_b = play_selection_b;

        /* show new play selection if play continue */
        if ((tick_switch != 0) && (play_selection_b < no_of_elements_b)) {

            /* scroll */
            sel_position = score_b[shown_play_selection_b].position;
            if (sel_position - position_offset + 20 >= 224) {
                position_offset = sel_position - 224 + 200;
                if (position_offset > score_end_position_b - 224 + 5)
                    position_offset = score_end_position_b - 224 + 5;
                clear_screen();
                show_score();
            }

            switch (score_b[shown_play_selection_b].category) {
                case SCORE_NOTE : case SCORE_REST :
                    paint_play_selection_b(); break;
            }
        }
    }
}


/* do add */

/* palette  elements_menu_selection ۂݒ肷 */
void fit_for_add() {

    int i;
    int current_duration = 0;
    int bar_duration = CROTCHET * 4;

    int category, type, attributes;

    /* if score is empty, specify palette selection as CREF */
    if (t_or_b == TREBLE) {
        if (no_of_elements_t < 1) {
            elements_menu_selection = 0;
            return;
        }
    } else {
        if (no_of_elements_b < 1) {
            elements_menu_selection = 0;
            return;
        }
    }

    if (t_or_b == TREBLE) {
        /* check the duration and if it is large enough, set element as bar */
        for (i=0; i <= selection_from; i++) {
            switch (score_t[i].category) {
                case SCORE_BAR : current_duration = 0; break;  /* reset duration count */
                case SCORE_RITHM :  /* set duration of each bar according to rithm */
                    switch (score_t[i].type) {
                        case BINARIA      : bar_duration = MINIM * 2; break;
                        case TERNARIA     : bar_duration = CROTCHET * 3; break;
                        case QUATERNARIA  : bar_duration = CROTCHET * 4; break;
                        case CROME        : bar_duration = QUAVER * 6; break;
                        case SEMICROME    : bar_duration = SIXTEENTH * 9; break;
                        case TWOFOURTH    : bar_duration = CROTCHET * 2; break;
                        case TWELVEEIGHTH : bar_duration = QUAVER * 12; break;
                    }
                    break;
                case SCORE_NOTE : case SCORE_REST :
                    current_duration += score_t[i].type;
                    break;
            }
        }
        /* these values will be used later */
        category = score_t[selection_from].category;
        type = score_t[selection_from].type;
        attributes = score_t[selection_from].attributes;
    } else {
        /* check the duration and if it is large enough, set element as bar */
        for (i=0; i <= selection_from; i++) {
            switch (score_b[i].category) {
                case SCORE_BAR : current_duration = 0; break;  /* reset duration count */
                case SCORE_RITHM :  /* set duration of each bar according to rithm */
                    switch (score_b[i].type) {
                        case BINARIA      : bar_duration = MINIM * 2; break;
                        case TERNARIA     : bar_duration = CROTCHET * 3; break;
                        case QUATERNARIA  : bar_duration = CROTCHET * 4; break;
                        case CROME        : bar_duration = QUAVER * 6; break;
                        case SEMICROME    : bar_duration = SIXTEENTH * 9; break;
                        case TWOFOURTH    : bar_duration = CROTCHET * 2; break;
                        case TWELVEEIGHTH : bar_duration = QUAVER * 12; break;
                    }
                    break;
                case SCORE_NOTE : case SCORE_REST :
                    current_duration += score_b[i].type;
                    break;
            }
        }
        /* these values will be used later */
        category = score_b[selection_from].category;
        type = score_b[selection_from].type;
        attributes = score_b[selection_from].attributes;
    }

    if (current_duration >= bar_duration) {
        elements_menu_selection = 18;  /* SCORE_BAR */
        return;
    }

    /* refer former element of score */
    switch (category) {
        case SCORE_CREF : elements_menu_selection = 2;  return;  /* SCORE_KEY, F_MAJOR */
        case SCORE_KEY  : elements_menu_selection = 14; return;  /* SCORE_RITHM, QUATERNARIA */
        case SCORE_BAR  : case SCORE_RITHM :
            elements_menu_selection = 32;  /* SCORE_NOTE, QUAVER NORMAL */
            return;
        case SCORE_SIG :
            elements_menu_selection = 32;  /* SCORE_NOTE, QUAVER NORMAL */
            return;
        case SCORE_NOTE : case SCORE_REST :
            if ((attributes & DIRECTION_REVERSED) != 0) {
                /* reversed */
                switch (type) {
                    case DOTTED_MINIM     : elements_menu_selection = 27; return;  /* CROTCHET REVERSED */
                    case MINIM            : elements_menu_selection = 31; return;  /* QUAVER REVERSED */
                    case DOTTED_CROTCHET  : elements_menu_selection = 31; return;  /* QUAVER REVERSED */
                    case CROTCHET         : elements_menu_selection = 27; return;  /* CROTCHET */
                    case DOTTED_QUAVER    : elements_menu_selection = 35; return;  /* SIXTEENTH */
                    case QUAVER           : elements_menu_selection = 31; return;  /* QUAVER REVERSED */
                    case DOTTED_SIXTEENTH : elements_menu_selection = 37; return;  /* THIRTY SECOND */
                    case SIXTEENTH        : elements_menu_selection = 35; return;  /* SIXTEENTH */
                    case THIRTY_SECOND    : elements_menu_selection = 37; return;  /* THIRTY SCCOND */
                    return;
                }
            } else {
                /* normal (not reversed) */
                switch (type) {
                    case DOTTED_MINIM     : elements_menu_selection = 28; return;  /* CROTCHET NORMAL */
                    case MINIM            : elements_menu_selection = 32; return;  /* QUAVER NORMAL */
                    case DOTTED_CROTCHET  : elements_menu_selection = 32; return;  /* QUAVER NORMAL */
                    case CROTCHET         : elements_menu_selection = 28; return;  /* CROTCHET NORMAL */
                    case DOTTED_QUAVER    : elements_menu_selection = 36; return;  /* SIXTEENTH NORMAL */
                    case QUAVER           : elements_menu_selection = 32; return;  /* QUAVER NORMAL */
                    case DOTTED_SIXTEENTH : elements_menu_selection = 38; return;  /* THIRTY SECOND */
                    case SIXTEENTH        : elements_menu_selection = 36; return;  /* SIXTEENTH */
                    case THIRTY_SECOND    : elements_menu_selection = 38; return;  /* THIRTY SCCOND */
                    return;
                }
            }
    }
}


void fit_for_edit_t() {

    int i;

    for (i=0; i<60; i++) {
        if (score_t[selection_from].category == elements_menu[i].category) {
            if (score_t[selection_from].type == elements_menu[i].type) {
                if (score_t[selection_from].category != SCORE_NOTE) {
                    elements_menu_selection = i;
                    return;
                } else {
                    if ((score_t[selection_from].attributes & DIRECTION_REVERSED)
                        == (elements_menu[i].attributes & DIRECTION_REVERSED)) {
                        elements_menu_selection = i;
                        return;
                    }
                }
            }
        }
    }
}

void fit_for_edit_b() {

    int i;

    for (i=0; i<60; i++) {
        if (score_b[selection_from].category == elements_menu[i].category) {
            if (score_b[selection_from].type == elements_menu[i].type) {
                if (score_b[selection_from].category != SCORE_NOTE) {
                    elements_menu_selection = i;
                    return;
                } else {
                    if ((score_b[selection_from].attributes & DIRECTION_REVERSED)
                        == (elements_menu[i].attributes & DIRECTION_REVERSED)) {
                        elements_menu_selection = i;
                        return;
                    }
                }
            }
        }
    }
}

void fit_for_edit() {
    if (t_or_b == TREBLE) fit_for_edit_t();
    else fit_for_edit_b();
}


/* fit pitch */

void fit_pitch_t() {
    switch (score_t[selection_from-1].category) {
        case SCORE_NOTE : case SCORE_REST : case SCORE_SIG :
            score_t[selection_from].pitch = score_t[selection_from-1].pitch;
            break;
        default :
            score_t[selection_from].pitch = PITCH_A6;
    }
    if (score_t[selection_from].pitch > PITCH_A8)
        score_t[selection_from].pitch = PITCH_A8;
    if (score_t[selection_from].pitch < PITCH_C3)
        score_t[selection_from].pitch = PITCH_C3;
}

void fit_pitch_b() {
    switch (score_b[selection_from-1].category) {
        case SCORE_NOTE : case SCORE_REST : case SCORE_SIG :
            score_b[selection_from].pitch = score_b[selection_from-1].pitch;
            break;
        default :
            score_b[selection_from].pitch = PITCH_A6;
    }
    if (score_b[selection_from].pitch > PITCH_A8)
        score_b[selection_from].pitch = PITCH_A8;
    if (score_b[selection_from].pitch < PITCH_C3)
        score_b[selection_from].pitch = PITCH_C3;
}


/* init elements menu positions */
void init_elements_menu() {

    int i;
    int xpos = 0;
    struct Score_element *element;

    xpos = 0;
    for (i=0; i<2; i++) {  /* cref */
        element = &elements_menu[i];
        element->position = xpos;
        xpos += element_width_for_sel(element) +3;
    }

    xpos = 0;
    for (i=2; i<7; i++) {  /* key, sharp # sequence */
        element = &elements_menu[i];
        element->position = xpos;
        xpos += element_width_for_sel(element) +3;
    }

    /* init key c major */
    elements_menu[6].type = elements_menu[6].type | (E_MAJOR << 8);

    xpos = 0;
    for (i=7; i<12; i++) {  /* key, flat sequence */
        element = &elements_menu[i];
        element->position = xpos;
        xpos += element_width_for_sel(element) +3;
    }

    xpos = 0;
    for (i=12; i<18; i++) {  /* rithm */
        element = &elements_menu[i];
        element->position = xpos;
        xpos += element_width_for_sel(element) +3;
    }

    xpos = 0;
    for (i=18; i<20; i++) {  /* bar */
        element = &elements_menu[i];
        element->position = xpos;
        xpos += element_width_for_sel(element) +3;
    }

    xpos = 0;
    for (i=20; i<33; i++) {  /* note */
        element = &elements_menu[i];
        element->position = xpos;
        xpos += element_width_for_sel(element) +1;
    }

    xpos = 0;
    for (i=33; i<43; i++) {  /* note 2 */
        element = &elements_menu[i];
        element->position = xpos;
        xpos += element_width_for_sel(element) +1;
    }

    xpos = 0;
    for (i=43; i<53; i++) {  /* rest */
        element = &elements_menu[i];
        element->position = xpos;
        xpos += element_width_for_sel(element) +3;
    }

    xpos = 0;
    for (i=53; i<55; i++) {  /* rest 2 */
        element = &elements_menu[i];
        element->position = xpos;
        xpos += element_width_for_sel(element) +3;
    }

    xpos = 0;
    for (i=55; i<60; i++) {  /* signature */
        element = &elements_menu[i];
        element->position = xpos;
        xpos += element_width_for_sel(element) +3;
    }
}


void show_palette() {

    int i;

    paint_screen(20, 10, 184, 100, 0x01);  /* x, y, width, height */
    show_horizontal_line(20, 184, 10, 0x05); /* x, width, y, color */
    show_horizontal_line(20, 184, 110, 0x05);
    show_horizontal_line(21, 185, 111, 0x03);  /* shadow */
    show_vertical_line(20, 10, 110, 0x05); /* x, y_top, y_bottom, color */
    show_vertical_line(204, 10, 111, 0x05);
    show_vertical_line(205, 11, 112, 0x03);  /* shadow */

    show_pentagramma_t(65, 130);  /* x, width */
    show_pentagramma_b(65, 130);

    show_string(23, 6, "cref", 0x00);
    show_string(23, 18, "key", 0x00);
    show_string(23, 30, "rithm", 0x00);
    show_string(23, 42, "bar", 0x00);
    show_string(23, 54, "note", 0x00);
    show_string(23, 66, "rest", 0x00);
    show_string(23, 78, "sig", 0x00);

    treble_cref = G_CREF;
    bass_cref = G_CREF;

    switch (menu_selection) {
        case 1 :  /* cref */
            for (i=0; i<2; i++) show_element_t(&elements_menu[i], 0);
            break;
        case 2 :  /* key */
            for (i=2; i<7; i++) show_element_t(&elements_menu[i], 0);
            for (i=7; i<12; i++) show_element_b(&elements_menu[i], 0);
            break;
        case 3 :  /* rithm */
            for (i=12; i<18; i++) show_element_t(&elements_menu[i], 0);
            break;
        case 4 :  /* bar */
            for (i=18; i<20; i++) show_element_t(&elements_menu[i], 0);
            break;
        case 5 :  /* note */
            for (i=20; i<33; i++) show_element_t(&elements_menu[i], 0);
            for (i=33; i<43; i++) {
                triplet_index = 4;  /* specify special treatment for triplet */
                show_element_b(&elements_menu[i], 0);
            }
            break;
        case 6 :  /* rest */
            for (i=43; i<53; i++) show_element_t(&elements_menu[i], 0);
            for (i=53; i<55; i++) {
		triplet_index = 4; /* specify special treatment for triplet */
                show_element_b(&elements_menu[i], 0);
            }
            break;
        case 7 :  /* signature */
            for (i=55; i<60; i++) show_element_t(&elements_menu[i], 0);
            break;
    }
}


void paint_palette_selection() {

    struct Score_element *element;
    int upper = 0;

    switch (menu_selection) {
        case 1 :  /* cref */
            change_color(20, 10, 40, 12, 0x01, 0x02);
            upper = 1;
            break;
        case 2 :  /* key */
            change_color(20, 22, 40, 12, 0x01, 0x02);
            if (elements_menu_selection < 7) upper = 1;
            break;
        case 3 :  /* rithm */
            change_color(20, 34, 40, 12, 0x01, 0x02);
            upper = 1;
            break;
        case 4 :  /* bar */
            change_color(20, 46, 40, 12, 0x01, 0x02);
            upper = 1;
            break;
        case 5 :  /* note */
            change_color(20, 58, 40, 12, 0x01, 0x02);
            if (elements_menu_selection < 33) upper = 1;
            break;
        case 6 :  /* rest */
            change_color(20, 70, 40, 12, 0x01, 0x02);
            if (elements_menu_selection < 53) upper = 1;
            break;
        case 7 :  /* signature */
            change_color(20, 82, 40, 12, 0x01, 0x02);
            upper = 1;
            break;
    }

    element = &elements_menu[elements_menu_selection];
    if (upper != 0) {
        change_color(
            element->position - position_offset,
            10,
            element_width_for_sel(element),
            50,
            0x01,  /* white */
            0x02   /* blue */
        );
    } else {
        change_color(
            element->position - position_offset,
            60,
            element_width_for_sel(element),
            50,
            0x01,  /* white */
            0x02   /* blue */
        );
    }

    wwc_font_set_colordata(0, 28*18, buffer);
    invalidate = 0;
}


void repair_palette_selection() {

    struct Score_element *element;
    int upper = 0;

    switch (menu_selection) {
        case 1 :  /* cref */
            upper = 1;
            break;
        case 2 :  /* key */
            if (elements_menu_selection < 7) upper = 1;
            break;
        case 3 :  /* rithm */
            upper = 1;
            break;
        case 4 :  /* bar */
            upper = 1;
            break;
        case 5 :  /* note */
            if (elements_menu_selection < 33) upper = 1;
            break;
        case 6 :  /* rest */
            if (elements_menu_selection < 53) upper = 1;
            break;
        case 7 :  /* signature */
            upper = 1;
            break;
    }

    element = &elements_menu[elements_menu_selection];
    if (upper != 0) {
        change_color(
            element->position - position_offset,
            10,
            element_width_for_sel(element),
            50,
            0x02,   /* blue */
            0x01    /* white */
        );
    } else {
        change_color(
            element->position - position_offset,
            60,
            element_width_for_sel(element),
            50,
            0x02,   /* blue */
            0x01    /* white */
        );
    }

    wwc_font_set_colordata(0, 28*18, buffer);
    invalidate = 0;
}


void handle_palette() {

    int position_offset_backup;
    int elements_menu_selection_backup;
    char  initial_press = 0;    /* auto repeat */
    int   repeat_times, i;  /* auto repeat */

    position_offset_backup = position_offset;
    position_offset = -70;

    elements_menu_selection_backup = elements_menu_selection;

    if (elements_menu_selection < 2) menu_selection = 1;        /* cref  */
    else if (elements_menu_selection < 12) menu_selection = 2;  /* key   */
    else if (elements_menu_selection < 18) menu_selection = 3;  /* rithm */
    else if (elements_menu_selection < 20) menu_selection = 4;  /* bar   */
    else if (elements_menu_selection < 43) menu_selection = 5;  /* note  */
    else if (elements_menu_selection < 55) menu_selection = 6;  /* rest  */
    else menu_selection = 7;                                    /* sig.  */

    show_palette();
    paint_palette_selection();
    while((key_press_check() & KEY_A) != 0)
        {}
    delay();
    while(1) {
        if((key_press_check() & KEY_X1) != 0)
        {
            if (menu_selection > 1) {
                menu_selection--;
                switch (menu_selection) {
                    case 1 : elements_menu_selection = 0; break;   /* cref  */
                    case 2 : elements_menu_selection = 2; break;   /* key   */
                    case 3 : elements_menu_selection = 12; break;  /* rithm */
                    case 4 : elements_menu_selection = 18; break;  /* bar   */
                    case 5 : elements_menu_selection = 20; break;  /* note  */
                    case 6 : elements_menu_selection = 43; break;  /* rest  */
                    case 7 : elements_menu_selection = 55; break;  /* sig.  */
                }
                show_palette();
                paint_palette_selection();
            }
        }
        if((key_press_check() & KEY_X3) != 0)
        {
            if (menu_selection < 7) {
                menu_selection++;
                switch (menu_selection) {
                    case 1 : elements_menu_selection = 0; break;
                    case 2 : elements_menu_selection = 2; break;
                    case 3 : elements_menu_selection = 12; break;
                    case 4 : elements_menu_selection = 18; break;
                    case 5 : elements_menu_selection = 20; break;
                    case 6 : elements_menu_selection = 43; break;
                    case 7 : elements_menu_selection = 55; break;
                }
                show_palette();
                paint_palette_selection();
            }
        }
        if((key_press_check() & KEY_X4) != 0)
        {
            switch (menu_selection) {
                case 1 :
                    if (elements_menu_selection > 0) {
                        repair_palette_selection();
                        elements_menu_selection--;
                    }
                    break;
                case 2 :
                    if (elements_menu_selection > 2) {
                        repair_palette_selection();
                        elements_menu_selection--;
                    }
                    break;
                case 3 :
                    if (elements_menu_selection > 12) {
                        repair_palette_selection();
                        elements_menu_selection--;
                    }
                    break;
                case 4 :
                    if (elements_menu_selection > 18) {
                        repair_palette_selection();
                        elements_menu_selection--;
                    }
                    break;
                case 5 :
                    if (elements_menu_selection > 20) {
                        repair_palette_selection();
                        elements_menu_selection--;
                    }
                    break;
                case 6 :
                    if (elements_menu_selection > 43) {
                        repair_palette_selection();
                        elements_menu_selection--;
                    }
                    break;
                case 7 :
                    if (elements_menu_selection > 55) {
                        repair_palette_selection();
                        elements_menu_selection--;
                    }
                    break;
            }
            paint_palette_selection();

            /* key repeat */
            if (initial_press == 0) {
                repeat_times = 3000;
                initial_press = 1;
            } else {
                repeat_times = 100;
            }
            for (i=0; i<repeat_times; i++) {
                if ((key_press_check() & KEY_X4) == 0) break;
            }
        }
        if((key_press_check() & KEY_X2) != 0)
        {
            switch (menu_selection) {
                case 1 :
                    if (elements_menu_selection < 1) {
                        repair_palette_selection();
                        elements_menu_selection++;
                    }
                    break;
                case 2 :
                    if (elements_menu_selection < 11) {
                        repair_palette_selection();
                        elements_menu_selection++;
                    }
                    break;
                case 3 :
                    if (elements_menu_selection < 17) {
                        repair_palette_selection();
                        elements_menu_selection++;
                    }
                    break;
                case 4 :
                    if (elements_menu_selection < 19) {
                        repair_palette_selection();
                        elements_menu_selection++;
                    }
                    break;
                case 5 :
                    if (elements_menu_selection < 42) {
                        repair_palette_selection();
                        elements_menu_selection++;
                    }
                    break;
                case 6 :
                    if (elements_menu_selection < 54) {
                        repair_palette_selection();
                        elements_menu_selection++;
                    }
                    break;
                case 7 :
                    if (elements_menu_selection < 59) {
                        repair_palette_selection();
                        elements_menu_selection++;
                    }
                    break;
            }
            paint_palette_selection();

            /* key repeat */
            if (initial_press == 0) {
                repeat_times = 3000;
                initial_press = 1;
            } else {
                repeat_times = 100;
            }
            for (i=0; i<repeat_times; i++) {
                if ((key_press_check() & KEY_X2) == 0) break;
            }
        }
        if((key_press_check() & KEY_A) != 0)
        {
            play_ok_button();
            position_offset = position_offset_backup;

            return;
        }
        if((key_press_check() & KEY_B) != 0)
        {
            play_cancel_button();
            menu_selection = 0;  /* cancel is judged by menu_selection==0 */
            elements_menu_selection = elements_menu_selection_backup;
            position_offset = position_offset_backup;

            return;
        }

#ifdef SHOT_ENABLE
        if ((key_press_check() & KEY_Y1) != 0) do_shot();
#endif
        /* cancel auto repeat */
        if (((key_press_check() & KEY_X4) == 0)
        && ((key_press_check() & KEY_X2) == 0))
            initial_press = 0;

        if (show_msg != 0) show_message();
        wwc_font_set_colordata(0, 28*18, buffer);
        invalidate = 0;

    }
}


/* element  selection ̌ɒǉ */

void show_size_limit_error() {

    set_message("score size limit");

    while ((key_press_check() & KEY_A) != 0)
        {}
    delay();

    while (1) {
        if ((key_press_check() & KEY_A) != 0)
            break;
        if ((key_press_check() & KEY_B) != 0)
            break;
        if (show_msg != 0) {
            show_message();
            wwc_font_set_colordata(0, 28*18, buffer);
        }
    }
    play_cancel_button();

    set_message("add mode");

    while ((key_press_check() & KEY_A) != 0)
        {}
    while ((key_press_check() & KEY_B) != 0)
        {}
    delay();

}


void add_element_t() {

    int i;

    if (SCORE_SIZE - 1 <= no_of_elements_t) {
        /* score size exceed error */
        show_size_limit_error();
        return;
    }

    for (i = no_of_elements_t; selection_to < i; i--) score_t[i] = score_t[i-1];
    no_of_elements_t++;

    /* fix selection */
    if (no_of_elements_t == 1) {
        /* special case, initial element is added to empty score */
        selection_to=0;
    } else {
        selection_to++;
    }
    selection_from = selection_to;

    fit_pitch_t();

}

void add_element_b() {
    int i;

    if (SCORE_SIZE - 1 <= no_of_elements_b) {
        /* score size exceed error */
        show_size_limit_error();
        return;
    }

    for (i = no_of_elements_b; selection_to < i; i--) score_b[i] = score_b[i-1];
    no_of_elements_b++;

    /* fix selection */
    if (no_of_elements_b == 1) {
        /* special case, initial element is added to empty score */
        selection_to=0;
    } else {
        selection_to++;
    }

    selection_from = selection_to;

    fit_pitch_b();
}

void add_element() {
    if (t_or_b == TREBLE) add_element_t();
    else add_element_b();
}


/* add element which is selected on the palette  */
void select_and_add_element() {
    struct Score_element *element;

    fit_for_add();
    handle_palette();

    if (menu_selection != 0) {   /* cancel is judged by menu_selection==0 */

        add_element();

        if (t_or_b == TREBLE) element = &score_t[selection_from];
        else element = &score_b[selection_from];

        element->category = elements_menu[elements_menu_selection].category;
        element->type = elements_menu[elements_menu_selection].type;
/*      element->pitch = elements_menu[elements_menu_selection].pitch; */
        element->attributes = elements_menu[elements_menu_selection].attributes;

        adjust_positions();
        set_c_major_and_a_minor();
    }

    /* scroll */
    if (t_or_b == TREBLE) {
        if (selection_position_from_t() - position_offset + 20 >= 224) {
            position_offset = selection_position_from_t() - 224 + 100;
            if (position_offset > score_end_position_t - 224 + 5)
                position_offset = score_end_position_t - 224 + 5;
        }
    } else {
        if (selection_position_from_b() - position_offset + 20 >= 224) {
            position_offset = selection_position_from_b() - 224 + 100;
            if (position_offset > score_end_position_b - 224 + 5)
                position_offset = score_end_position_b - 224 + 5;
        }
    }

    clear_screen();
    show_score();
    paint_score_selections();
    show_message();
    wwc_font_set_colordata(0, 28*18, buffer);
}

/* set element which is selected on the palette */
void select_and_set_element() {
    struct Score_element *element;

    fit_for_edit();
    handle_palette();

    if (menu_selection != 0) {   /* cancel is judged by menu_selection==0 */

        if (t_or_b == TREBLE) element = &score_t[selection_from];
        else element = &score_b[selection_from];

        element->category = elements_menu[elements_menu_selection].category;
        element->type = elements_menu[elements_menu_selection].type;
/*      element->pitch = elements_menu[elements_menu_selection].pitch; */
        element->attributes = elements_menu[elements_menu_selection].attributes;

        adjust_positions();
        set_c_major_and_a_minor();
    }

    /* scroll */
    if (t_or_b == TREBLE) {
        if (selection_position_from_t() - position_offset + 20 >= 224) {
            position_offset = selection_position_from_t() - 224 + 100;
            if (position_offset > score_end_position_t - 224 + 5)
                position_offset = score_end_position_t - 224 + 5;
        }
    } else {
        if (selection_position_from_b() - position_offset + 20 >= 224) {
            position_offset = selection_position_from_b() - 224 + 100;
            if (position_offset > score_end_position_b - 224 + 5)
                position_offset = score_end_position_b - 224 + 5;
        }
    }

    clear_screen();
    show_score();
    paint_score_selections();
    show_message();
    wwc_font_set_colordata(0, 28*18, buffer);
}

/* {^Ńsb`̕ύX */
void change_element_up() {
    struct Score_element *element;

    if (t_or_b == TREBLE) element = &score_t[selection_from];
    else element = &score_b[selection_from];

    switch(element->category) {
        case SCORE_NOTE : case SCORE_SIG :
            switch (element->pitch) {
                case PITCH_F5 : element->pitch = PITCH_G5; break;
                case PITCH_G5 : element->pitch = PITCH_A5; break;
                case PITCH_A5 : element->pitch = PITCH_B5; break;
                case PITCH_B5 : element->pitch = PITCH_C6; break;
                case PITCH_C6 : element->pitch = PITCH_D6; break;
                case PITCH_D6 : element->pitch = PITCH_E6; break;
                case PITCH_E6 : element->pitch = PITCH_F6; break;
                case PITCH_F6 : element->pitch = PITCH_G6; break;
                case PITCH_G6 : element->pitch = PITCH_A6; break;
                case PITCH_A6 : element->pitch = PITCH_B6; break;
                case PITCH_B6 : element->pitch = PITCH_C7; break;
                case PITCH_C7 : element->pitch = PITCH_D7; break;
                case PITCH_D7 : element->pitch = PITCH_E7; break;
                case PITCH_E7 : element->pitch = PITCH_F7; break;
                case PITCH_F7 : element->pitch = PITCH_G7; break;
                case PITCH_G7 : element->pitch = PITCH_A7; break;
                case PITCH_A7 : element->pitch = PITCH_B7; break;
                case PITCH_B7 : element->pitch = PITCH_C8; break;
                case PITCH_C8 : element->pitch = PITCH_D8; break;
                case PITCH_D8 : element->pitch = PITCH_E8; break;
                case PITCH_E8 : element->pitch = PITCH_F8; break;
                case PITCH_F8 : element->pitch = PITCH_G8; break;
                case PITCH_G8 : element->pitch = PITCH_A8; break;
            }
    }

    /* redraw */
    if (t_or_b == TREBLE) {
        sc_from_x = selection_position_from_t() - position_offset - 40;
        sc_to_x = selection_position_to_t() - position_offset + 40;
        sc_from_y = 0;
        sc_to_y = 80;
    } else {
        sc_from_x = selection_position_from_b() - position_offset - 40;
        sc_to_x = selection_position_to_b() - position_offset + 40;
        sc_from_y = 40;
        sc_to_y = 124;
    }
    if (sc_from_x < 0) sc_from_x = 0;
    if (224 < sc_to_x) sc_to_x = 224;
    paint_screen(sc_from_x+2, sc_from_y, sc_to_x-sc_from_x-4, sc_to_y-sc_from_y, 0x01);
    show_score();
    paint_score_selections();
    sc_from_x = 0;
    sc_to_x = 224;
    sc_from_y = 0;
    sc_to_y = 144;
}

/* {^Ńsb`̕ύX */
void change_element_down() {
    struct Score_element *element;

    if (t_or_b == TREBLE) element = &score_t[selection_from];
    else element = &score_b[selection_from];

    switch(element->category) {
        case SCORE_NOTE : case SCORE_SIG :
            switch (element->pitch) {
                case PITCH_G5 : element->pitch = PITCH_F5; break;
                case PITCH_A5 : element->pitch = PITCH_G5; break;
                case PITCH_B5 : element->pitch = PITCH_A5; break;
                case PITCH_C6 : element->pitch = PITCH_B5; break;
                case PITCH_D6 : element->pitch = PITCH_C6; break;
                case PITCH_E6 : element->pitch = PITCH_D6; break;
                case PITCH_F6 : element->pitch = PITCH_E6; break;
                case PITCH_G6 : element->pitch = PITCH_F6; break;
                case PITCH_A6 : element->pitch = PITCH_G6; break;
                case PITCH_B6 : element->pitch = PITCH_A6; break;
                case PITCH_C7 : element->pitch = PITCH_B6; break;
                case PITCH_D7 : element->pitch = PITCH_C7; break;
                case PITCH_E7 : element->pitch = PITCH_D7; break;
                case PITCH_F7 : element->pitch = PITCH_E7; break;
                case PITCH_G7 : element->pitch = PITCH_F7; break;
                case PITCH_A7 : element->pitch = PITCH_G7; break;
                case PITCH_B7 : element->pitch = PITCH_A7; break;
                case PITCH_C8 : element->pitch = PITCH_B7; break;
                case PITCH_D8 : element->pitch = PITCH_C8; break;
                case PITCH_E8 : element->pitch = PITCH_D8; break;
                case PITCH_F8 : element->pitch = PITCH_E8; break;
                case PITCH_G8 : element->pitch = PITCH_F8; break;
                case PITCH_A8 : element->pitch = PITCH_G8; break;
            }
    }

    /* redraw */
    if (t_or_b == TREBLE) {
        sc_from_x = selection_position_from_t() - position_offset - 40;
        sc_to_x = selection_position_to_t() - position_offset + 40;
        sc_from_y = 0;
        sc_to_y = 80;
    } else {
        sc_from_x = selection_position_from_b() - position_offset - 40;
        sc_to_x = selection_position_to_b() - position_offset + 40;
        sc_from_y = 40;
        sc_to_y = 124;
    }
    if (sc_from_x < 0) sc_from_x = 0;
    if (224 < sc_to_x) sc_to_x = 224;
    paint_screen(sc_from_x+2, sc_from_y, sc_to_x-sc_from_x-4, sc_to_y-sc_from_y, 0x01);
    show_score();
    paint_score_selections();
    sc_from_x = 0;
    sc_to_x = 224;
    sc_from_y = 0;
    sc_to_y = 144;
}


/* increment target of edit */
void increment_selection_t() {

    if (selection_from + 1 < no_of_elements_t) selection_from++;
    selection_to = selection_from;

    /* scroll */
    if (selection_position_to_t() - position_offset + 30 >= 224) {
        position_offset = selection_position_to_t() - 224 + 100;
        if (position_offset > score_end_position_t - 224 + 5)
            position_offset = score_end_position_t - 224 + 5;
        clear_screen();
        show_score();
    }
}

void increment_selection_b() {

    if (selection_from + 1 < no_of_elements_b) selection_from++;
    selection_to = selection_from;

    /* scroll */
    if (selection_position_to_b() - position_offset + 30 >= 224) {
        position_offset = selection_position_to_b() - 224 + 100;
        if (position_offset > score_end_position_b - 224 + 5)
            position_offset = score_end_position_b - 224 + 5;
        clear_screen();
        show_score();
    }
}

void increment_selection() {
    restore_score_selections();
    if (t_or_b == TREBLE) increment_selection_t();
    else increment_selection_b();
    paint_score_selections();
}


/* menu processing */

void do_add() {

    char  initial_press = 0;    /* auto repeat */
    int   repeat_times, i;  /* auto repeat */

    /* redraw */
    show_all();
    set_message("add mode");

    select_and_add_element();

    while(1) {
        if((key_press_check() & KEY_X2) != 0)  /* type and duration */
        {
            select_and_set_element();
        }
        if((key_press_check() & KEY_X4) != 0) /* type and duration */
        {
            select_and_set_element();
        }
        if((key_press_check() & KEY_X1) != 0)  /* pitch */
        {
            change_element_up();

            /* key repeat */
            if (initial_press == 0) {
                repeat_times = 10000;
                initial_press = 1;
            } else {
                repeat_times = 1000;
            }
            for (i=0; i<repeat_times; i++) {
                if ((key_press_check() & KEY_X4) == 0) break;
            }
        }
        if((key_press_check() & KEY_X3) != 0)  /* pitch */
        {
            change_element_down();

            /* key repeat */
            if (initial_press == 0) {
                repeat_times = 10000;
                initial_press = 1;
            } else {
                repeat_times = 1000;
            }
            for (i=0; i<repeat_times; i++) {
                if ((key_press_check() & KEY_X4) == 0) break;
            }
        }
        if((key_press_check() & KEY_A) != 0) /* element  selection ̌ɒǉ */
        {
            play_ok_button();
            select_and_add_element();

            while((key_press_check() & KEY_A) != 0)
                {}
            delay();  /* to avoid key chattering */
        }
        if((key_press_check() & KEY_B) != 0)
        {
            play_cancel_button();
            set_message("exited the add mode");
            return;
        }

        /* cancel auto repeat */
        if (((key_press_check() & KEY_X4) == 0)
        && ((key_press_check() & KEY_X2) == 0))
            initial_press = 0;

        if (show_msg != 0) show_message();
        wwc_font_set_colordata(0, 28*18, buffer);
        invalidate = 0;
    }
}


/* do edit */

void do_edit() {

    char  initial_press = 0;    /* auto repeat */
    int   repeat_times, i;  /* auto repeat */

    show_all();
    set_message("edit mode");

    select_and_set_element();

    while((key_press_check() & KEY_A) != 0)  /* wait until button is released */
        {}

    delay();  /* to avoid key chattering */

    while(1) {
        if((key_press_check() & KEY_X2) != 0)  /* type and duration */
        {
            select_and_set_element();
        }
        if((key_press_check() & KEY_X4) != 0) /* type and duration */
        {
            select_and_set_element();
        }
        if((key_press_check() & KEY_X1) != 0)  /* pitch */
        {
            change_element_up();

            /* key repeat */
            if (initial_press == 0) {
                repeat_times = 10000;
                initial_press = 1;
            } else {
                repeat_times = 1000;
            }
            for (i=0; i<repeat_times; i++) {
                if ((key_press_check() & KEY_X4) == 0) break;
            }
        }
        if((key_press_check() & KEY_X3) != 0)  /* pitch */
        {
            change_element_down();

            /* key repeat */
            if (initial_press == 0) {
                repeat_times = 10000;
                initial_press = 1;
            } else {
                repeat_times = 1000;
            }
            for (i=0; i<repeat_times; i++) {
                if ((key_press_check() & KEY_X4) == 0) break;
            }
        }
        if((key_press_check() & KEY_A) != 0)
        {
            play_ok_button();
            if (((t_or_b == TREBLE) && (selection_to == no_of_elements_t - 1))
            || ((t_or_b == BASS) && (selection_to == no_of_elements_b - 1))) {
                while((key_press_check() & KEY_A) != 0)
                    {}
                delay();  /* to avoid key chattering */
                set_message("exited the edit mode");
                return;
            }
            increment_selection();
            select_and_set_element();
            while((key_press_check() & KEY_A) != 0)
                {}
            delay();  /* to avoid key chattering */
        }
        if((key_press_check() & KEY_B) != 0)
        {
            play_cancel_button();
            set_message("exited the edit mode");
            return;
        }

        /* cancel auto repeat */
        if (((key_press_check() & KEY_X4) == 0)
        && ((key_press_check() & KEY_X2) == 0))
            initial_press = 0;

        if (show_msg != 0) show_message();
        wwc_font_set_colordata(0, 28*18, buffer);
        invalidate = 0;
    }
}


/* do delete */

void do_delete_t() {
    int i, j;

    if (no_of_elements_t < 1) return;
    j = selection_from;
    for (i = selection_to+1; i < no_of_elements_t; i++)
        score_t[j++] = score_t[i];
    no_of_elements_t -= selection_to - selection_from + 1;

    if (selection_from >= no_of_elements_t) selection_from = no_of_elements_t - 1;
    selection_to = selection_from;
}

void do_delete_b() {
    int i, j;

    if (no_of_elements_b < 1) return;
    j = selection_from;
    for (i = selection_to+1; i < no_of_elements_b; i++)
        score_b[j++] = score_b[i];
    no_of_elements_b -= selection_to - selection_from + 1;

    if (selection_from >= no_of_elements_b) selection_from = no_of_elements_b - 1;
    selection_to = selection_from;
}

void do_delete() {
    if (t_or_b == TREBLE) do_delete_t();
    else do_delete_b();

    adjust_positions();
    set_c_major_and_a_minor();

    show_all();
}


/* do play */

/* play a note */
void play_note_t(char psuedo) {

    int i;

    struct Score_element *element;

    /* if time is come treat next note */
    while (next_tick_t <= total_tick) {

        if (just_started_t) {
            just_started_t = 0;
        } else {
            /* ȊO play_selection  increment */
            ++play_selection_t;
            if (play_selection_t >= no_of_elements_t) return;
        }

        element = &score_t[play_selection_t];

        switch (element->category) {
            case SCORE_CREF :
                treble_pitch = element->attributes;  /* set pitch */
                break;
            case SCORE_RITHM :
                treble_rithm_play = element->type;  /* set rithm for the duration of whole rest */
                break;
            case SCORE_BAR :
                /* bar_pitches initialization at the top of the bar */
                for (i=0; i<41; i++) bar_pitchs_t[i] = score_pitchs_t[i];
                break;
            case SCORE_KEY :
                change_score_pitch_t(element);
                for (i=0; i<41; i++) bar_pitchs_t[i] = score_pitchs_t[i];
                break;
            case SCORE_SIG :
                change_bar_pitch_t(element);
                break;
            case SCORE_NOTE :
                if (tie_play_t == 1) tie_play_t = 0; /* tie ̂Ƃ͉ZbgȂ */
                else if (psuedo == 0) play_simple_sound(pitch_value_t(element), 12); /* psuedo łȂΉZbg */
                next_tick_t += (long)play_duration_t(element);
                if ((element->attributes & TIE_SET) != 0) tie_play_t = 1;
                break;
            case SCORE_REST :
                if (psuedo == 0) play_rest(); /* psuedo łȂrestZbg */
                next_tick_t += (long)play_duration_t(element);
                break;
        }
    }
}


void play_note_b(char psuedo) {

    int i;

    struct Score_element *element;

    /* if time is come treat next note */
    while (next_tick_b <= total_tick) {

        if (just_started_b) {
            just_started_b = 0;
        } else {
            /* ȊO play_selection  increment */
            ++play_selection_b;
            if (play_selection_b >= no_of_elements_b) return;
        }

        element = &score_b[play_selection_b];

        switch (element->category) {
            case SCORE_CREF :
                bass_pitch = element->attributes;  /* set pitch */
                break;
            case SCORE_RITHM :
                bass_rithm_play = element->type;  /* set rithm for the duration of whole rest */
                break;
            case SCORE_BAR :
                /* bar_pitches initialization at the top of the bar */
                for (i=0; i<41; i++) bar_pitchs_b[i] = score_pitchs_b[i];
                break;
            case SCORE_KEY :
                change_score_pitch_b(element);
                for (i=0; i<41; i++) bar_pitchs_b[i] = score_pitchs_b[i];
                break;
            case SCORE_SIG :
                change_bar_pitch_b(element);
                break;
            case SCORE_NOTE :
                if (tie_play_b == 1) tie_play_b = 0; /* tie ̂Ƃ͉ZbgȂ */
                else if (psuedo == 0) play_simple_sound_b(pitch_value_b(element), 12); /* psuedo łȂΉZbg */
                next_tick_b += (long)play_duration_b(element);
                if ((element->attributes & TIE_SET) != 0) tie_play_b = 1;
                break;
            case SCORE_REST :
                if (psuedo == 0) play_rest_b(); /* psuedo łȂrestZbg */
                next_tick_b += (long)play_duration_b(element);
                break;
        }
    }
}


void do_play() {
    int i;

    if (tick_switch != 0) {
        tick_switch = 0;

        /* trigger for display update */
        play_selection_t = 0;
        play_selection_b = 0;

    } else {

        while ((int)sound_get_volume(CHANNEL3) != 0) {}; 
        sys_interrupt_reset_hook(SYS_INT_VBLANK, NULL);

        for (i=0; i<41; i++) {
            score_pitchs_t[i] = natural_pitchs[i];
            score_pitchs_b[i] = natural_pitchs[i];

            bar_pitchs_t[i] = score_pitchs_t[i];
            bar_pitchs_b[i] = score_pitchs_b[i];
        }

        total_tick = 0;
        tick_switch = 1;
        next_tick_t = 0;
        next_tick_b = 0;

        play_selection_t = 0;
        play_selection_b = 0;

        shown_play_selection_t = 0;
        shown_play_selection_b = 0;

        just_started_t = 1;
        just_started_b = 1;

        if (t_or_b == TREBLE) {
            while (play_selection_t+1 < selection_from) {
                total_tick += 10;
                play_note_t(1); /* (1) means psuedo play note */
                play_note_b(1); /* (1) means psuedo play note */
            }
        } else {
            while (play_selection_b+1 < selection_from) {
                total_tick += 10;
                play_note_t(1); /* (1) means psuedo play note */
                play_note_b(1); /* (1) means psuedo play note */
            }
        }

        sys_interrupt_set_hook(SYS_INT_VBLANK, &intVector, NULL);
    }
}


/* set tie */

void show_tie_menu(int x, int y) {

    int x_t, y_t;

    x_t = x;
    y_t = y;

    if (x < 45) x_t = 45;
    if (y < 5) y_t = 5;
    if (179 < x) x_t = 179;    /* 224 - 40 - 5 = 179 */
    if (103 < y) y_t = 103;      /* 144 - 12*3 - 5 = 103 */

    if (menu_selection != 0) {
        paint_screen(x_t, y_t, 40, 36, 0x01);
        show_horizontal_line(x_t, 40, y_t, 0x05);
        show_horizontal_line(x_t, 40, y_t+36, 0x05);
        show_horizontal_line(x_t+1, 40, y_t+37, 0x03);  /* shadow */
        show_vertical_line(x_t, y_t, y_t+36, 0x05);
        show_vertical_line(x_t+40, y_t, y_t+37, 0x05);
        show_vertical_line(x_t+41, y_t+1, y_t+38, 0x03);  /* shadow */

        /* upper */
        show_form(x_t+5, y_t, tie_top_left_form, 0x00);
        show_form(x_t+13, y_t, tie_top_right_form, 0x00);
        show_horizontal_line(x_t+11, 8, y_t+4, 0x00);

        /* lower */
        show_form(x_t+5, y_t+16, tie_bottom_left_form, 0x00);
        show_form(x_t+13, y_t+16, tie_bottom_right_form, 0x00);
        show_horizontal_line(x_t+11, 8, y_t+19, 0x00);

        show_string(x_t+3, y_t+20, "none", 0x00);
    }
    wwc_font_set_colordata(0, 28*18, buffer);
}


void paint_tie_menu_selection(int x, int y) {

    int x_t, y_t;

    x_t = x;
    y_t = y;

    if (x < 45) x_t = 45;
    if (y < 5) y_t = 5;
    if (179 < x) x_t = 179;    /* 224 - 40 - 5 = 179 */
    if (103 < y) y_t = 103;      /* 144 - 12*3 - 5 = 103 */

    switch (menu_selection) {
        case 1 : 
            change_color(x_t, y_t, 40, 12, 0x01, 0x02); break;
        case 2 :
            change_color(x_t, y_t+12, 40, 12, 0x01, 0x02); break;
        case 3 :
            change_color(x_t, y_t+24, 40, 12, 0x01, 0x02); break;
    }
    wwc_font_set_colordata(0, 28*18, buffer);
}


void repair_tie_menu_selection(int x, int y) {

    int x_t, y_t;

    x_t = x;
    y_t = y;

    if (x < 45) x_t = 45;
    if (y < 5) y_t = 5;
    if (179 < x) x_t = 179;    /* 224 - 40 - 5 = 179 */
    if (103 < y) y_t = 103;      /* 144 - 12*3 - 5 = 103 */

    switch (menu_selection) {
        case 1 : 
            change_color(x_t, y_t, 40, 12, 0x02, 0x01); break;
        case 2 :
            change_color(x_t, y_t+12, 40, 12, 0x02, 0x01); break;
        case 3 :
            change_color(x_t, y_t+24, 40, 12, 0x02, 0x01); break;
    }
    wwc_font_set_colordata(0, 28*18, buffer);
}


void handle_tie_menu(int x, int y) {

    struct Score_element *element;

    switch (t_or_b) {
        case TREBLE :
            menu_selection = treble_pitch;
            element = &score_t[selection_from];
            break;
        case BASS   :
            menu_selection = bass_pitch;
            element = &score_b[selection_to];
            break;
    }

    /* tie is set only for note */
    if (element->category != SCORE_NOTE) {
        set_message("tie is available only for note");
        return;
    }

    if (element->attributes & TIE_SET) {
        if (element->attributes & TIE_UPPER) {
            menu_selection = 1;
        } else {
            menu_selection = 2;
        }
    } else {
        menu_selection = 3;
    }

    show_tie_menu(x, y);
    paint_tie_menu_selection(x, y);
    while((key_press_check() & KEY_A) != 0)
        {}
    delay();
    while(1) {
        if((key_press_check() & KEY_X1) != 0)
        {
            if (menu_selection > 1) {
                repair_tie_menu_selection(x, y);
                menu_selection--;
            }
            paint_tie_menu_selection(x, y);
            while((key_press_check() & KEY_X1) != 0)
                {}
            delay();  /* to avoid key chattering */
        }
        if((key_press_check() & KEY_X3) != 0)
        {
            if (menu_selection < 3) {
                repair_tie_menu_selection(x, y);
                menu_selection++;
            }
            paint_tie_menu_selection(x, y);
            while((key_press_check() & KEY_X3) != 0)
                {}
            delay();  /* to avoid key chattering */
        }
        if((key_press_check() & KEY_A) != 0)
        {
            play_ok_button();
            switch (menu_selection) {
                case 1 :
                    element->attributes |= TIE_SET;
                    element->attributes |= TIE_UPPER;
                    break;
                case 2 :
                    element->attributes |= TIE_SET;
                    element->attributes &= ~TIE_UPPER;
                    break;
                case 3 :
                    element->attributes &= ~TIE_SET;
                    break;
            }

            show_all();

            while((key_press_check() & KEY_A) != 0) {
            }
            delay();

            return;
        }
        if((key_press_check() & KEY_B) != 0)
        {
            play_cancel_button();
            menu_selection = 0;
            show_all();
            return;
        }
    }
}


/* tempo menu */

void show_tempo_menu(int x, int y) {

    int x_t, y_t;

    x_t = x;
    y_t = y;

    if (x < 45) x_t = 45;
    if (y < 5) y_t = 5;
    if (179 < x) x_t = 179;    /* 224 - 40 - 5 = 179 */
    if (7 < y) y_t = 7;      /* 144 - 12*11 - 5 = 7 */

    if (menu_selection != 0) {
        paint_screen(x_t, y_t, 40, 132, 0x01);
        show_horizontal_line(x_t, 40, y_t, 0x05);
        show_horizontal_line(x_t, 40, y_t+132, 0x05);
        show_horizontal_line(x_t+1, 40, y_t+133, 0x03);  /* shadow */
        show_vertical_line(x_t, y_t, y_t+132, 0x05);
        show_vertical_line(x_t+40, y_t, y_t+133, 0x05);
        show_vertical_line(x_t+41, y_t+1, y_t+134, 0x03);  /* shadow */

        show_form(x_t+3, y_t+56, crotchet_noml_form, 0x00);
        show_string(x_t+12, y_t+56, "=", 0x00);

        show_string(x_t+21, y_t-4, "60", 0x00);
        show_string(x_t+21, y_t+8, "70", 0x00);
        show_string(x_t+21, y_t+20, "80", 0x00);
        show_string(x_t+21, y_t+32, "90", 0x00);
        show_string(x_t+21, y_t+44, "100", 0x00);
        show_string(x_t+21, y_t+56, "110", 0x00);
        show_string(x_t+21, y_t+68, "120", 0x00);
        show_string(x_t+21, y_t+80, "130", 0x00);
        show_string(x_t+21, y_t+92, "140", 0x00);
        show_string(x_t+21, y_t+104, "150", 0x00);
        show_string(x_t+21, y_t+116, "160", 0x00);
    }
    wwc_font_set_colordata(0, 28*18, buffer);
}


void paint_tempo_menu_selection(int x, int y) {

    int x_t, y_t;

    x_t = x;
    y_t = y;

    if (x < 45) x_t = 45;
    if (y < 5) y_t = 5;
    if (179 < x) x_t = 179;    /* 224 - 40 - 5 = 179 */
    if (7 < y) y_t = 7;      /* 144 - 12*11 - 5 = 7 */

    switch (menu_selection) {
        case 1 : 
            change_color(x_t+18, y_t, 22, 12, 0x01, 0x02); break;
        case 2 : 
            change_color(x_t+18, y_t+12, 22, 12, 0x01, 0x02); break;
        case 3 : 
            change_color(x_t+18, y_t+24, 22, 12, 0x01, 0x02); break;
        case 4 : 
            change_color(x_t+18, y_t+36, 22, 12, 0x01, 0x02); break;
        case 5 : 
            change_color(x_t+18, y_t+48, 22, 12, 0x01, 0x02); break;
        case 6 : 
            change_color(x_t+18, y_t+60, 22, 12, 0x01, 0x02); break;
        case 7 : 
            change_color(x_t+18, y_t+72, 22, 12, 0x01, 0x02); break;
        case 8 :
            change_color(x_t+18, y_t+84, 22, 12, 0x01, 0x02); break;
        case 9 :
            change_color(x_t+18, y_t+96, 22, 12, 0x01, 0x02); break;
        case 10 :
            change_color(x_t+18, y_t+108, 22, 12, 0x01, 0x02); break;
        case 11 :
            change_color(x_t+18, y_t+120, 22, 12, 0x01, 0x02); break;
    }
    wwc_font_set_colordata(0, 28*18, buffer);
}


void repair_tempo_menu_selection(int x, int y) {

    int x_t, y_t;

    x_t = x;
    y_t = y;

    if (x < 45) x_t = 45;
    if (y < 5) y_t = 5;
    if (179 < x) x_t = 179;    /* 224 - 40 - 5 = 179 */
    if (7 < y) y_t = 7;      /* 144 - 12*11 - 5 = 7 */

    switch (menu_selection) {
        case 1 : 
            change_color(x_t+18, y_t, 22, 12, 0x02, 0x01); break;
        case 2 : 
            change_color(x_t+18, y_t+12, 22, 12, 0x02, 0x01); break;
        case 3 : 
            change_color(x_t+18, y_t+24, 22, 12, 0x02, 0x01); break;
        case 4 : 
            change_color(x_t+18, y_t+36, 22, 12, 0x02, 0x01); break;
        case 5 : 
            change_color(x_t+18, y_t+48, 22, 12, 0x02, 0x01); break;
        case 6 : 
            change_color(x_t+18, y_t+60, 22, 12, 0x02, 0x01); break;
        case 7 : 
            change_color(x_t+18, y_t+72, 22, 12, 0x02, 0x01); break;
        case 8 :
            change_color(x_t+18, y_t+84, 22, 12, 0x02, 0x01); break;
        case 9 :
            change_color(x_t+18, y_t+96, 22, 12, 0x02, 0x01); break;
        case 10 :
            change_color(x_t+18, y_t+108, 22, 12, 0x02, 0x01); break;
        case 11 :
            change_color(x_t+18, y_t+120, 22, 12, 0x02, 0x01); break;
    }
    wwc_font_set_colordata(0, 28*18, buffer);
}


void handle_tempo_menu(int x, int y) {
    switch (tempo) {
        case  60 : menu_selection =  1; break;
        case  70 : menu_selection =  2; break;
        case  80 : menu_selection =  3; break;
        case  90 : menu_selection =  4; break;
        case 100 : menu_selection =  5; break;
        case 110 : menu_selection =  6; break;
        case 120 : menu_selection =  7; break;
        case 130 : menu_selection =  8; break;
        case 140 : menu_selection =  9; break;
        case 150 : menu_selection = 10; break;
        case 160 : menu_selection = 11; break;
    }
    show_tempo_menu(x, y);
    paint_tempo_menu_selection(x, y);
    while((key_press_check() & KEY_A) != 0)
        {}
    delay();
    while(1) {
        if((key_press_check() & KEY_X1) != 0)
        {
            if (menu_selection > 1) {
                repair_tempo_menu_selection(x, y);
                menu_selection--;
            }
            paint_tempo_menu_selection(x, y);
            while((key_press_check() & KEY_X1) != 0)
                {}
            delay();  /* to avoid key chattering */
        }
        if((key_press_check() & KEY_X3) != 0)
        {
            if (menu_selection < 11) {
                repair_tempo_menu_selection(x, y);
                menu_selection++;
            }
            paint_tempo_menu_selection(x, y);
            while((key_press_check() & KEY_X3) != 0)
                {}
            delay();  /* to avoid key chattering */
        }
        if((key_press_check() & KEY_A) != 0)
        {
            play_ok_button();
            switch (menu_selection) {
                case  1 : tempo =  60; break;
                case  2 : tempo =  70; break;
                case  3 : tempo =  80; break;
                case  4 : tempo =  90; break;
                case  5 : tempo = 100; break;
                case  6 : tempo = 110; break;
                case  7 : tempo = 120; break;
                case  8 : tempo = 130; break;
                case  9 : tempo = 140; break;
                case 10 : tempo = 150; break;
                case 11 : tempo = 160; break;
            }
            show_all();
            return;
        }
        if((key_press_check() & KEY_B) != 0)
        {
            play_cancel_button();
            menu_selection = 0;

            show_all();
            return;
        }
    }
}


/* pitch menu */

void show_pitch_menu(int x, int y) {

    int x_t, y_t;

    x_t = x;
    y_t = y;

    if (x < 45) x_t = 45;
    if (y < 5) y_t = 5;
    if (179 < x) x_t = 179;    /* 224 - 40 - 5 = 179 */
    if (103 < y) y_t = 103;      /* 144 - 12*3 - 5 = 103 */

    if (menu_selection != 0) {
        paint_screen(x_t, y_t, 40, 36, 0x01);
        show_horizontal_line(x_t, 40, y_t, 0x05);
        show_horizontal_line(x_t, 40, y_t+36, 0x05);
        show_horizontal_line(x_t+1, 40, y_t+37, 0x03);  /* shadow */
        show_vertical_line(x_t, y_t, y_t+36, 0x05);
        show_vertical_line(x_t+40, y_t, y_t+37, 0x05);
        show_vertical_line(x_t+41, y_t+1, y_t+38, 0x03);  /* shadow */

        show_string(x_t+3, y_t-4, "high", 0x00);
        show_string(x_t+3, y_t+8, "middle", 0x00);
        show_string(x_t+3, y_t+20, "low", 0x00);
    }
    wwc_font_set_colordata(0, 28*18, buffer);
}


void paint_pitch_menu_selection(int x, int y) {

    int x_t, y_t;

    x_t = x;
    y_t = y;

    if (x < 45) x_t = 45;
    if (y < 5) y_t = 5;
    if (179 < x) x_t = 179;    /* 224 - 40 - 5 = 179 */
    if (103 < y) y_t = 103;      /* 144 - 12*3 - 5 = 103 */

    switch (menu_selection) {
        case PITCH_HIGH : 
            change_color(x_t, y_t, 40, 12, 0x01, 0x02); break;
        case PITCH_MIDDLE :
            change_color(x_t, y_t+12, 40, 12, 0x01, 0x02); break;
        case PITCH_LOW :
            change_color(x_t, y_t+24, 40, 12, 0x01, 0x02); break;
    }
    wwc_font_set_colordata(0, 28*18, buffer);
}


void repair_pitch_menu_selection(int x, int y) {

    int x_t, y_t;

    x_t = x;
    y_t = y;

    if (x < 45) x_t = 45;
    if (y < 5) y_t = 5;
    if (179 < x) x_t = 179;    /* 224 - 40 - 5 = 179 */
    if (103 < y) y_t = 103;      /* 144 - 12*3 - 5 = 103 */

    switch (menu_selection) {
        case PITCH_HIGH : 
            change_color(x_t, y_t, 40, 12, 0x02, 0x01); break;
        case PITCH_MIDDLE :
            change_color(x_t, y_t+12, 40, 12, 0x02, 0x01); break;
        case PITCH_LOW :
            change_color(x_t, y_t+24, 40, 12, 0x02, 0x01); break;
    }
    wwc_font_set_colordata(0, 28*18, buffer);
}


void handle_pitch_menu(int x, int y) {

    struct Score_element *element;

    switch (t_or_b) {
        case TREBLE : element = &score_t[selection_from]; break;
        case BASS   : element = &score_b[selection_to]; break;
    }

    /* pitch is set only in CREF */
    if (element->category != SCORE_CREF) {
        set_message("pitch is available only in cref");
        return;
    }

    menu_selection = element->attributes;
    /* backward compatibility */
    if (menu_selection < 1) menu_selection = 1;
    if (menu_selection > 3) menu_selection = 3;

    show_pitch_menu(x, y);
    paint_pitch_menu_selection(x, y);
    while((key_press_check() & KEY_A) != 0)
        {}
    delay();
    while(1) {
        if((key_press_check() & KEY_X1) != 0)
        {
            if (menu_selection > 1) {
                repair_pitch_menu_selection(x, y);
                menu_selection--;
            }
            paint_pitch_menu_selection(x, y);
            while((key_press_check() & KEY_X1) != 0)
                {}
            delay();  /* to avoid key chattering */
        }
        if((key_press_check() & KEY_X3) != 0)
        {
            if (menu_selection < 3) {
                repair_pitch_menu_selection(x, y);
                menu_selection++;
            }
            paint_pitch_menu_selection(x, y);
            while((key_press_check() & KEY_X3) != 0)
                {}
            delay();  /* to avoid key chattering */
        }
        if((key_press_check() & KEY_A) != 0)
        {
            play_ok_button();
            element->attributes = menu_selection;

            show_all();
            return;
        }
        if((key_press_check() & KEY_B) != 0)
        {
            play_cancel_button();
            menu_selection = 0;
            show_all();
            return;
        }
    }
}


/* save and load */

/* save and load progress box */
void show_progress_box() {

    paint_screen(42, 57, 138, 12, 0x01);
    show_horizontal_line(41, 139, 57, 0x05);
    show_horizontal_line(41, 139, 69, 0x05);
    show_horizontal_line(42, 139, 70, 0x03);  /* shadow */
    show_vertical_line(41, 57, 70, 0x05);
    show_vertical_line(180, 57, 70, 0x05);
    show_vertical_line(181, 58, 71, 0x03);  /* shadow */

}


void do_save(int saver_selection) {

    FILE far *out;
    int i;

    set_message("saving");
    show_all();
    show_progress_box();
    show_message();
    wwc_font_set_colordata(0, 28*18, buffer);

    /* wait until ok_button_sound stop */
    while (((int)sound_get_volume(CHANNEL3) != 0) || (message_pos_min < message_pos)) {
        if (show_msg != 0) {
            show_message();
            wwc_font_set_colordata(0, 28*18, buffer);
        }
    }

    sys_interrupt_reset_hook(SYS_INT_VBLANK, NULL);

    show_message();
    wwc_font_set_colordata(0, 28*18, buffer);

    switch (saver_selection) {
        case 0 :
            creat("/ram0/sfg_sv1", O_RDWR, 96);
            out = fopen("/ram0/sfg_sv1", "wb"); break;
        case 1 :
            creat("/ram0/sfg_sv2", O_RDWR, 96);
            out = fopen("/ram0/sfg_sv2", "wb"); break;
        case 2 :
            creat("/ram0/sfg_sv3", O_RDWR, 96);
            out = fopen("/ram0/sfg_sv3", "wb"); break;
    }

    if (out == NULL) {
        while((key_press_check() & KEY_A) != 0)
            {}
        delay();
        sys_interrupt_set_hook(SYS_INT_VBLANK, &intVector, NULL);
        set_message("save failed");

        return;
    }

    fputc((char)123, out);

    fputc((char)(tempo>>8), out);
    fputc((char)tempo, out);
    fputc((char)(treble_pitch>>8), out);
    fputc((char)treble_pitch, out);
    fputc((char)(no_of_elements_t >> 8), out);
    fputc((char)no_of_elements_t, out);

    for (i=0; i<no_of_elements_t; i++) {
        fputc((char)(score_t[i].category>>8), out);
        fputc((char)score_t[i].category, out);
        fputc((char)(score_t[i].type>>8), out);
        fputc((char)score_t[i].type, out);
        fputc((char)(score_t[i].pitch>>8), out);
        fputc((char)score_t[i].pitch, out);
        fputc((char)(score_t[i].attributes>>8), out);
        fputc((char)score_t[i].attributes, out);
        if (score_t[i].category == SCORE_BAR) {
            paint_screen(42, 58, 18*i/no_of_elements_t*4 -2, 11, 0x04);
            wwc_font_set_colordata(0, 28*18, buffer);
        }
    }

    fputc((char)(bass_pitch>>8), out);
    fputc((char)bass_pitch, out);
    fputc((char)(no_of_elements_b >> 8), out);
    fputc((char)no_of_elements_b, out);

    for (i=0; i<no_of_elements_b; i++) {

        fputc((char)(score_b[i].category>>8), out);
        fputc((char)score_b[i].category, out);
        fputc((char)(score_b[i].type>>8), out);
        fputc((char)score_b[i].type, out);
        fputc((char)(score_b[i].pitch>>8), out);
        fputc((char)score_b[i].pitch, out);
        fputc((char)(score_b[i].attributes>>8), out);
        fputc((char)score_b[i].attributes, out);
        if (score_b[i].category == SCORE_BAR) {
            paint_screen(42, 58, 18*i/no_of_elements_b*4 +70, 11, 0x04);
            wwc_font_set_colordata(0, 28*18, buffer);
        }
        fflush(out);
    }

    fclose(out);

    while((key_press_check() & KEY_A) != 0)
        {}
    delay();
    sys_interrupt_set_hook(SYS_INT_VBLANK, &intVector, NULL);
    set_message("save complete");
}


void show_saver_menu(int x, int y) {

    int x_t, y_t;

    x_t = x;
    y_t = y;

    if (x < 45) x_t = 45;
    if (y < 5) y_t = 5;
    if (179 < x) x_t = 179;    /* 224 - 40 - 5 = 179 */
    if (103 < y) y_t = 103;      /* 144 - 12*3 - 5 = 103 */

    if (menu_selection != 0) {
        paint_screen(x_t, y_t, 40, 36, 0x01);
        show_horizontal_line(x_t, 40, y_t, 0x05);
        show_horizontal_line(x_t, 40, y_t+36, 0x05);
        show_horizontal_line(x_t+1, 40, y_t+37, 0x03);  /* shadow */
        show_vertical_line(x_t, y_t, y_t+36, 0x05);
        show_vertical_line(x_t+40, y_t, y_t+37, 0x05);
        show_vertical_line(x_t+41, y_t+1, y_t+38, 0x03);  /* shadow */

        show_string(x_t+3, y_t-4, "1", 0x00);
        show_string(x_t+3, y_t+8, "2", 0x00);
        show_string(x_t+3, y_t+20, "3", 0x00);
    }
    wwc_font_set_colordata(0, 28*18, buffer);
}


void paint_saver_menu_selection(int x, int y) {

    int x_t, y_t;

    x_t = x;
    y_t = y;

    if (x < 45) x_t = 45;
    if (y < 5) y_t = 5;
    if (179 < x) x_t = 179;    /* 224 - 40 - 5 = 179 */
    if (103 < y) y_t = 103;      /* 144 - 12*3 - 5 = 103 */

    switch (menu_selection) {
        case 1 : 
            change_color(x_t, y_t, 40, 12, 0x01, 0x02); break;
        case 2 : 
            change_color(x_t, y_t+12, 40, 12, 0x01, 0x02); break;
        case 3 : 
            change_color(x_t, y_t+24, 40, 12, 0x01, 0x02); break;
    }
    wwc_font_set_colordata(0, 28*18, buffer);
}


void repair_saver_menu_selection(int x, int y) {

    int x_t, y_t;

    x_t = x;
    y_t = y;

    if (x < 45) x_t = 45;
    if (y < 5) y_t = 5;
    if (179 < x) x_t = 179;    /* 224 - 40 - 5 = 179 */
    if (103 < y) y_t = 103;      /* 144 - 12*3 - 5 = 103 */

    switch (menu_selection) {
        case 1 : 
            change_color(x_t, y_t, 50, 12, 0x02, 0x01); break;
        case 2 : 
            change_color(x_t, y_t+12, 50, 12, 0x02, 0x01); break;
        case 3 : 
            change_color(x_t, y_t+24, 50, 12, 0x02, 0x01); break;
    }
    wwc_font_set_colordata(0, 28*18, buffer);
}

void handle_save_menu(int x, int y) {
    menu_selection = 1;
    show_saver_menu(x, y);
    paint_saver_menu_selection(x, y);
    while((key_press_check() & KEY_A) != 0) {
    }
    delay();
    while(1) {
        if((key_press_check() & KEY_X1) != 0)
        {
            if (menu_selection > 1) {
                repair_saver_menu_selection(x, y);
                menu_selection--;
            }
            paint_saver_menu_selection(x, y);
            while((key_press_check() & KEY_X1) != 0)
                {}
            delay();  /* to avoid key chattering */
        }
        if((key_press_check() & KEY_X3) != 0)
        {
            if (menu_selection < 3) {
                repair_saver_menu_selection(x, y);
                menu_selection++;
            }
            paint_saver_menu_selection(x, y);
            while((key_press_check() & KEY_X3) != 0)
                {}
            delay();  /* to avoid key chattering */
        }
        if((key_press_check() & KEY_A) != 0)
        {
            play_ok_button();
            switch (menu_selection) {
                case 1 : do_save(0); break;
                case 2 : do_save(1); break;
                case 3 : do_save(2); break;
            }
            show_all();
            return;
        }
        if((key_press_check() & KEY_B) != 0)
        {
            play_cancel_button();
            menu_selection = 0;
            show_all();
            return;
        }
    }
}


void do_load(int saver_selection) {

    FILE far *in;
    int i;

    set_message("loading");
    show_all();
    show_progress_box();
    show_message();
    wwc_font_set_colordata(0, 28*18, buffer);

    /* wait until ok_button_sound stop */
    while (((int)sound_get_volume(CHANNEL3) != 0) || (message_pos_min < message_pos)) {
        if (show_msg != 0) {
            show_message();
            wwc_font_set_colordata(0, 28*18, buffer);
        }
    }

    sys_interrupt_reset_hook(SYS_INT_VBLANK, NULL);

    show_message();
    wwc_font_set_colordata(0, 28*18, buffer);

    switch (saver_selection) {
        case 0 : in = fopen("/ram0/sfg_sv1", "rb"); break;
        case 1 : in = fopen("/ram0/sfg_sv2", "rb"); break;
        case 2 : in = fopen("/ram0/sfg_sv3", "rb"); break;
    }

    if (in == NULL) {
        while((key_press_check() & KEY_A) != 0)
            {}
        delay();
        sys_interrupt_set_hook(SYS_INT_VBLANK, &intVector, NULL);
        set_message("saver not found");

        return;
    }

    if (fgetc(in) != (char)123) return;

    tempo = (int)(fgetc(in))<<8;
    tempo += (int)fgetc(in);
    treble_pitch = (int)(fgetc(in))<<8;
    treble_pitch += (int)fgetc(in);
    no_of_elements_t = (int)(fgetc(in))<<8;
    no_of_elements_t += (int)fgetc(in);

    for (i=0; i<no_of_elements_t; i++) {
        score_t[i].category = (int)(fgetc(in))<<8;
        score_t[i].category += (int)fgetc(in);
        score_t[i].type = (int)(fgetc(in))<<8;
        score_t[i].type += (int)fgetc(in);
        score_t[i].pitch = (int)(fgetc(in))<<8;
        score_t[i].pitch += (int)fgetc(in);
        score_t[i].attributes = (int)(fgetc(in))<<8;
        score_t[i].attributes += (int)fgetc(in);
        if (score_t[i].category == SCORE_BAR) {
            paint_screen(42, 58, 18*i/(long)no_of_elements_t*4 -2, 11, 0x04);
            wwc_font_set_colordata(0, 28*18, buffer);
        }
    }

    bass_pitch = (int)(fgetc(in))<<8;
    bass_pitch += (int)fgetc(in);
    no_of_elements_b = (int)(fgetc(in))<<8;
    no_of_elements_b += (int)fgetc(in);

    for (i=0; i<no_of_elements_b; i++) {
        score_b[i].category = (int)(fgetc(in))<<8;
        score_b[i].category += (int)fgetc(in);
        score_b[i].type = (int)(fgetc(in))<<8;
        score_b[i].type += (int)fgetc(in);
        score_b[i].pitch = (int)(fgetc(in))<<8;
        score_b[i].pitch += (int)fgetc(in);
        score_b[i].attributes = (int)(fgetc(in))<<8;
        score_b[i].attributes += (int)fgetc(in);
        if (score_b[i].category == SCORE_BAR) {
            paint_screen(42, 58, 18*i/no_of_elements_b*4 +70, 11, 0x04);
            wwc_font_set_colordata(0, 28*18, buffer);
        }
    }

    fclose(in);


    /* selection adjustment */
    selection_from = 0;
    selection_to = selection_from;
    t_or_b = TREBLE;

    position_offset = 0;

    /* redraw score */
    adjust_positions();
    set_c_major_and_a_minor();

    show_all();

    while((key_press_check() & KEY_A) != 0)
        {}
    delay();
    sys_interrupt_set_hook(SYS_INT_VBLANK, &intVector, NULL);
    set_message("load complete");
}


void handle_load_menu(int x, int y) {
    menu_selection = 1;
    show_saver_menu(x, y);
    paint_saver_menu_selection(x, y);
    while((key_press_check() & KEY_A) != 0) {
    }
    delay();
    while(1) {
        if((key_press_check() & KEY_X1) != 0)
        {
            if (menu_selection > 1) {
                repair_saver_menu_selection(x, y);
                menu_selection--;
            }
            paint_saver_menu_selection(x, y);
            while((key_press_check() & KEY_X1) != 0)
                {}
            delay();  /* to avoid key chattering */
        }
        if((key_press_check() & KEY_X3) != 0)
        {
            if (menu_selection < 3) {
                repair_saver_menu_selection(x, y);
                menu_selection++;
            }
            paint_saver_menu_selection(x, y);
            while((key_press_check() & KEY_X3) != 0)
                {}
            delay();  /* to avoid key chattering */
        }
        if((key_press_check() & KEY_A) != 0)
        {
            play_ok_button();
            switch (menu_selection) {
                case 1 : do_load(0); break;
                case 2 : do_load(1); break;
                case 3 : do_load(2); break;
            }
            show_all();
            return;
        }
        if((key_press_check() & KEY_B) != 0)
        {
            play_cancel_button();
            menu_selection = 0;
            show_all();
            return;
        }
#ifdef SHOT_ENABLE
        if ((key_press_check() & KEY_Y1) != 0) do_shot();
#endif
    }
}


/* main menu */

void show_main_menu(int x, int y) {

    int x_t, y_t;

    x_t = x;
    y_t = y;

    if (x < 5) x_t = 5;
    if (y < 5) y_t = 5;
    if (139 < x) x_t = 139;    /* 224 - 40 - 40 - 5 = 139 */
    if (31 < y) y_t = 31;      /* 144 - 12*9 - 5 = 31 */

    if (menu_selection != 0) {
        paint_screen(x_t, y_t, 40, 108, 0x01);
        show_horizontal_line(x_t, 40, y_t, 0x05);
        show_horizontal_line(x_t, 40, y_t+108, 0x05);
        show_horizontal_line(x_t+1, 40, y_t+109, 0x03);  /* shadow */
        show_vertical_line(x_t, y_t, y_t+108, 0x05);
        show_vertical_line(x_t+40, y_t, y_t+109, 0x05);
        show_vertical_line(x_t+41, y_t+1, y_t+110, 0x03);  /* shadow */

        show_string(x_t+3, y_t-4, "add", 0x00);
        show_string(x_t+3, y_t+8, "edit", 0x00);
        show_string(x_t+3, y_t+20, "tie", 0x00);
        show_string(x_t+3, y_t+32, "pitch", 0x00);
        show_string(x_t+3, y_t+44, "delete", 0x00);
        show_string(x_t+3, y_t+56, "tempo", 0x00);
        show_string(x_t+3, y_t+68, "play", 0x00);
        show_string(x_t+3, y_t+80, "save", 0x00);
        show_string(x_t+3, y_t+92, "load", 0x00);
    }
    wwc_font_set_colordata(0, 28*18, buffer);
}


void paint_main_menu_selection(int x, int y) {

    int x_t, y_t;

    x_t = x;
    y_t = y;

    if (x < 5) x_t = 5;
    if (y < 5) y_t = 5;
    if (139 < x) x_t = 139;    /* 224 - 40 - 40 - 5 = 139 */
    if (31 < y) y_t = 31;      /* 144 - 12*9 - 5 = 31 */

    switch (menu_selection) {
        case 1 : 
            change_color(x_t, y_t, 40, 12, 0x01, 0x02); break;
        case 2 : 
            change_color(x_t, y_t+12, 40, 12, 0x01, 0x02); break;
        case 3 : 
            change_color(x_t, y_t+24, 40, 12, 0x01, 0x02); break;
        case 4 : 
            change_color(x_t, y_t+36, 40, 12, 0x01, 0x02); break;
        case 5 : 
            change_color(x_t, y_t+48, 40, 12, 0x01, 0x02); break;
        case 6 : 
            change_color(x_t, y_t+60, 40, 12, 0x01, 0x02); break;
        case 7 :
            change_color(x_t, y_t+72, 40, 12, 0x01, 0x02); break;
        case 8 :
            change_color(x_t, y_t+84, 40, 12, 0x01, 0x02); break;
        case 9 :
            change_color(x_t, y_t+96, 40, 12, 0x01, 0x02); break;
    }
    wwc_font_set_colordata(0, 28*18, buffer);
}


void repair_main_menu_selection(int x, int y) {

    int x_t, y_t;

    x_t = x;
    y_t = y;

    if (x < 5) x_t = 5;
    if (y < 5) y_t = 5;
    if (139 < x) x_t = 139;    /* 224 - 40 - 40 - 5 = 139 */
    if (31 < y) y_t = 31;      /* 144 - 12*9 - 5 = 31 */

    switch (menu_selection) {
        case 1 : 
            change_color(x_t, y_t, 40, 12, 0x02, 0x01); break;
        case 2 : 
            change_color(x_t, y_t+12, 40, 12, 0x02, 0x01); break;
        case 3 : 
            change_color(x_t, y_t+24, 40, 12, 0x02, 0x01); break;
        case 4 : 
            change_color(x_t, y_t+36, 40, 12, 0x02, 0x01); break;
        case 5 : 
            change_color(x_t, y_t+48, 40, 12, 0x02, 0x01); break;
        case 6 : 
            change_color(x_t, y_t+60, 40, 12, 0x02, 0x01); break;
        case 7 :
            change_color(x_t, y_t+72, 40, 12, 0x02, 0x01); break;
        case 8 :
            change_color(x_t, y_t+84, 40, 12, 0x02, 0x01); break;
        case 9 :
            change_color(x_t, y_t+96, 40, 12, 0x02, 0x01); break;
    }
    wwc_font_set_colordata(0, 28*18, buffer);
}


void handle_main_menu(int x, int y) {

    menu_selection = 1;
    show_main_menu(x, y);
    paint_main_menu_selection(x, y);
    while((key_press_check() & KEY_A) != 0)
        {}
    delay();
    while(1) {
        if((key_press_check() & KEY_X1) != 0)
        {
            if (menu_selection > 1) {
                repair_main_menu_selection(x, y);
                menu_selection--;
                paint_main_menu_selection(x, y);
            }
            while((key_press_check() & KEY_X1) != 0)
                {}
            delay();  /* to avoid key chattering */
        }
        if((key_press_check() & KEY_X3) != 0)
        {
            if (menu_selection < 9) {
                repair_main_menu_selection(x, y);
                menu_selection++;
                paint_main_menu_selection(x, y);
            }
            while((key_press_check() & KEY_X3) != 0)
                {}
            delay();  /* to avoid key chattering */
        }
        if((key_press_check() & KEY_A) != 0)
        {
            play_ok_button();
            switch(menu_selection) {
                case 1 : do_add(); break;
                case 2 : do_edit(); break;
                case 3 : handle_tie_menu(x+40, y+24); break;
                case 4 : handle_pitch_menu(x+40, y+36); break;
                case 5 : do_delete(); break;
                case 6 : handle_tempo_menu(x+40, y+60); break;
                case 7 : do_play(); break;
                case 8 : handle_save_menu(x+40, y+84); break;
                case 9 : handle_load_menu(x+40, y+96); break;
            }
            show_all();
            return;
        }
        if((key_press_check() & KEY_B) != 0)
        {
            play_cancel_button();
            menu_selection = 0;
            show_all();
            return;
        }
#ifdef SHOT_ENABLE
        if ((key_press_check() & KEY_Y1) != 0) do_shot();
#endif
    }
}


/* change selection */

void count_up_selection_to_t() {
    if (selection_to < no_of_elements_t - 1) selection_to++;
}

void count_up_selection_to_b() {
    if (selection_to < no_of_elements_b - 1) selection_to++;
}


void count_down_selection_from_t() {
    if (selection_from > 0) selection_from--;
}

void count_down_selection_from_b() {
    if (selection_from > 0) selection_from--;
}


void change_selections_by_right_button_t() {

    /* if selection_from is end of score, do nothing */
    if (selection_from >= no_of_elements_t-1) return;

    if((key_press_check() & KEY_B) != 0)
    {
        count_up_selection_to_t();
    } else {
        restore_score_selections();
        count_up_selection_to_t();
        selection_from = selection_to;
    }

    /* scroll */
    if (selection_position_to_t() - position_offset + 20 >= 224) {
        position_offset = selection_position_to_t() - 224 + 100;

        if (position_offset > score_end_position_t - 224 + 5)
            position_offset = score_end_position_t - 224 + 5;
        show_all();

    } else {
        paint_score_selections();
    }

    /* scroll */
    if (selection_position_from_t() - position_offset < 0) {
        position_offset = selection_position_from_t() - 100;

        if (position_offset < 0) position_offset = 0;
        show_all();

    } else {
        paint_score_selections();
    }
}

void change_selections_by_right_button_b() {

    /* if selection_from is end of score, do nothing */
    if (selection_from >= no_of_elements_b-1) return;

    if((key_press_check() & KEY_B) != 0)
    {
        count_up_selection_to_b();
    } else {
        restore_score_selections();
        count_up_selection_to_b();
        selection_from = selection_to;
    }

    /* scroll */
    if (selection_position_to_b() - position_offset + 20 >= 224) {
        position_offset = selection_position_to_b() - 224 + 100;

        if (position_offset > score_end_position_b - 224 + 5)
            position_offset = score_end_position_b - 224 + 5;
        show_all();

    } else {
        paint_score_selections();
    }

    /* scroll */
    if (selection_position_from_b() - position_offset < 0) {
        position_offset = selection_position_from_b() - 100;

        if (position_offset < 0) position_offset = 0;
        show_all();

    } else {
        paint_score_selections();
    }
}


void change_selections_by_left_button_t() {

    /* if selection_to is start of score, do nothing */
    if ((selection_to <= 0) && (position_offset == 0)) return;

    if((key_press_check() & KEY_B) != 0)
    {
        count_down_selection_from_t();
    } else {
        restore_score_selections();
        count_down_selection_from_t();
        selection_to = selection_from;
    }

    /* scroll */
    if (selection_position_from_t() - position_offset < 0) {
        position_offset = selection_position_from_t() - 100;

        if (position_offset < 0) position_offset = 0;
        show_all();

    } else {
        paint_score_selections();
    }
}

void change_selections_by_left_button_b() {

    /* if selection_to is start of score, do nothing */
    if ((selection_to <= 0) && (position_offset == 0)) return;

    if((key_press_check() & KEY_B) != 0)
    {
        count_down_selection_from_b();
    } else {
        restore_score_selections();
        count_down_selection_from_b();
        selection_to = selection_from;
    }

    /* scroll */
    if (selection_position_from_b() - position_offset < 0) {
        position_offset = selection_position_from_b() - 100;

        if (position_offset < 0) position_offset = 0;
        show_all();

    } else {
        paint_score_selections();
    }
}


/* change selection by bar as unit, right */

void count_up_selection_to_by_bar_t() {
    int i;

    for (i = selection_to + 1; i < no_of_elements_t; i++) {
        if (score_t[i].category == SCORE_BAR) {
            selection_to = i;
            return;
        }
    }

    selection_to = no_of_elements_t - 1;
    return;
}

void count_up_selection_to_by_bar_b() {
    int i;

    for (i = selection_to + 1; i < no_of_elements_b; i++) {
        if (score_b[i].category == SCORE_BAR) {
            selection_to = i;
            return;
        }
    }

    selection_to = no_of_elements_b - 1;
    return;
}


void change_selections_by_R_button_t() {

    /* if selection_from is end of score, do nothing */
    if (selection_from >= no_of_elements_t -1) return;

    if((key_press_check() & KEY_B) != 0)
    {
        count_up_selection_to_by_bar_t();
    } else {
        restore_score_selections();
        count_up_selection_to_by_bar_t();
        selection_from = selection_to;
    }

    /* scroll */
    if (selection_position_to_t() - position_offset + 20 >= 224) {
        position_offset = selection_position_to_t() - 224 + 100;

        if (position_offset > score_end_position_t - 224 + 5)
            position_offset = score_end_position_t - 224 + 5;
        show_all();

    } else {
        paint_score_selections();
    }

    /* scroll */
    if (selection_position_from_t() - position_offset < 0) {
        position_offset = selection_position_from_t() - 100;

        if (position_offset < 0) position_offset = 0;
        show_all();

    } else {
        paint_score_selections();
    }
}

void change_selections_by_R_button_b() {

    /* if selection_from is end of score, do nothing */
    if (selection_from >= no_of_elements_b -1) return;

    if((key_press_check() & KEY_B) != 0)
    {
        count_up_selection_to_by_bar_b();
    } else {
        restore_score_selections();
        count_up_selection_to_by_bar_b();
        selection_from = selection_to;
    }

    /* scroll */
    if (selection_position_to_b() - position_offset + 20 >= 224) {
        position_offset = selection_position_to_b() - 224 + 100;

        if (position_offset > score_end_position_b - 224 + 5)
            position_offset = score_end_position_b - 224 + 5;
        show_all();

    } else {
        paint_score_selections();
    }

    /* scroll */
    if (selection_position_from_b() - position_offset < 0) {
        position_offset = selection_position_from_b() - 100;

        if (position_offset < 0) position_offset = 0;
        show_all();

    } else {
        paint_score_selections();
    }
}

void change_selections_by_R_button() {
    if (t_or_b == TREBLE) change_selections_by_R_button_t();
    else change_selections_by_R_button_b();
}


/* change selection by bar as unit, left */

void count_down_selection_by_bar_from_t() {
    int i;

    for (i = selection_from; 0 <= i; i--) {
         if (score_t[i].category == SCORE_BAR) {
             selection_from = i - 1;
             return;
         }
    }

    selection_from = 0;
    return;
}

void count_down_selection_by_bar_from_b() {
    int i;

    for (i = selection_from; 0 <= i; i--) {
         if (score_b[i].category == SCORE_BAR) {
             selection_from = i - 1;
             return;
         }
    }

    selection_from = 0;
    return;
}


void change_selections_by_L_button_t() {

    /* if selection_to is start of score, do nothing */
    if ((selection_to <= 0) && (position_offset == 0)) return;

    if((key_press_check() & KEY_B) != 0)
    {
        count_down_selection_by_bar_from_t();
    } else {
        restore_score_selections();
        count_down_selection_by_bar_from_t();
        selection_to = selection_from;
    }

    /* scroll */
    if (selection_position_from_t() - position_offset < 0) {
        position_offset = selection_position_from_t() - 100;

        if (position_offset < 0) position_offset = 0;
        show_all();

    } else {
        paint_score_selections();
    }

}

void change_selections_by_L_button_b() {

    /* if selection_to is start of score, do nothing */
    if ((selection_to <= 0) && (position_offset == 0)) return;

    if((key_press_check() & KEY_B) != 0)
    {
        count_down_selection_by_bar_from_b();
    } else {
        restore_score_selections();
        count_down_selection_by_bar_from_b();
        selection_to = selection_from;
    }

    /* scroll */
    if (selection_position_from_b() - position_offset < 0) {
        position_offset = selection_position_from_b() - 100;

        if (position_offset < 0) position_offset = 0;
        show_all();

    } else {
        paint_score_selections();
    }
}

void change_selections_by_L_button() {
    if (t_or_b == TREBLE) change_selections_by_L_button_t();
    else change_selections_by_L_button_b();
}


/* treble or bass */

void change_selections_by_up_button() {
    int i;
    int bar_no_b;
    int bar_no_t;

    restore_score_selections();

    /* check no of bar at selection in bass */
    bar_no_b = 0;
    for (i=0; i<=selection_from; i++)
        if (score_b[i].category == SCORE_BAR)
            bar_no_b++;

    /* set no of bar into selection in treble */
    bar_no_t = 0;
    for (i=0; i<no_of_elements_t; i++) {
        if (bar_no_b <= bar_no_t) {
            selection_from = i;
            break;
        }
        if (score_t[i].category == SCORE_BAR)
            bar_no_t++;
    }
    /* reached end of loop means bass score has more bars than treble score */
    if (no_of_elements_t <= i) selection_from = no_of_elements_t - 1;

    if (no_of_elements_t <= selection_from) selection_from = no_of_elements_t - 1;

    t_or_b = TREBLE;
    selection_to = selection_from;

    /* scroll */
    if (selection_position_from_t() - position_offset < 0) {
        position_offset = selection_position_from_t() - 100;

        if (position_offset < 0) position_offset = 0;
        show_all();

    } else {
        paint_score_selections();
    }
}


void change_selections_by_down_button() {
    int i;
    int bar_no_t;
    int bar_no_b;

    restore_score_selections();

    /* check no of bar at selection in treble */
    bar_no_t = 0;
    for (i=0; i<=selection_from; i++)
        if (score_t[i].category == SCORE_BAR)
            bar_no_t++;

    /* set no of bar into selection in treble */
    bar_no_b = 0;
    for (i=0; i<no_of_elements_b; i++) {
        if (bar_no_t <= bar_no_b) {
            selection_from = i;
            break;
        }
        if (score_b[i].category == SCORE_BAR)
            bar_no_b++;
    }
    /* reached end of for loop means treble score has more bars than bass score */
    if (no_of_elements_b <= i) selection_from = no_of_elements_b - 1;

    if (no_of_elements_b <= selection_from) selection_from = no_of_elements_b - 1;

    t_or_b = BASS;
    selection_to = selection_from;

    if (selection_position_from_b() - position_offset < 0) {
        position_offset = selection_position_from_b() - 100;

        if (position_offset < 0) position_offset = 0;
        show_all();

    } else {
        paint_score_selections();
    }
}


/* Vertical Blank IRQ is used as playing timer */

void far f_VBLINT_Handler() {

    int vol;

    sys_interrupt_reset_hook(SYS_INT_VBLANK, NULL);

    /* envelope */
    if (--vcount_1 <= 0) {
        vol = (int)sound_get_volume(CHANNEL1);
        if (vol != 0) sound_set_volume(CHANNEL1, vol-0x11);
        vcount_1 = 10;
    }
    if (--vcount_2 <= 0) {
        vol = (int)sound_get_volume(CHANNEL2);
        if (vol != 0) sound_set_volume(CHANNEL2, vol-0x11);
        vcount_2 = 10;
    }
    if (--vcount_3 <= 0) {
        vol = (int)sound_get_volume(CHANNEL3);
        if (vol != 0) sound_set_volume(CHANNEL3, vol-0x11);
        vcount_3 = 3;
    }

    /* count tichs */
    if (tick_switch != 0) {
        total_tick += (long)(8*tempo/75);  /* 480*tempo/60/75 */
        play_note_t(0);
        play_note_b(0);
        if ((no_of_elements_t <= play_selection_t) && (no_of_elements_b <= play_selection_b)) {
            tick_switch = 0;

            /* trigger update play selection */
            play_selection_t = 0;
            play_selection_b = 0;
        }
    }

    /* message position */
    if (--message_pos_timer <= 0) {
        if (message_pos_min < message_pos) {
            message_pos--;
            show_msg = 1;
        }
        message_pos_timer = 1;
    }

    sys_interrupt_set_hook(SYS_INT_VBLANK, &intVector, NULL);

}


/* main program */

void main(int argc, char *argv[]) {

    int i, j;
    unsigned int char_no;
    char  initial_press = 0;    /* auto repeat */
    int   repeat_times;  /* auto repeat */

    init_char_forms();
    init_elements_menu();
    init_score_t();
    init_score_b();
    adjust_positions();
    set_c_major_and_a_minor();

    /* check if wonderswan color */
    if (wwc_get_hardarch() == HARDARCH_WS ) {
        text_screen_init();
        text_put_string(1, 1, "Solfege runs only on WonderSwan Color, sorry");
        while(1) {
            if ((key_press_check() & KEY_START) != 0)
                break;
            if ((key_press_check() & KEY_A) != 0)
                break;
        }
        return;
    }

    wwc_set_color_mode(COLOR_MODE_16PACKED);
    wwc_clear_font();
    display_control(DCM_SCR1);
    screen_set_scroll(SCREEN1, 0, 0);

    wwc_palette_set_color(0x0, 0x0, 0x0000);  /* black */
    wwc_palette_set_color(0x0, 0x1, 0x0fff);  /* white background */
    wwc_palette_set_color(0x0, 0x2, 0x01ff);  /* edit selection */
    wwc_palette_set_color(0x0, 0x3, 0x0bbb);  /* pentagramma */
    wwc_palette_set_color(0x0, 0x4, 0x03f9);  /* play selection */
    wwc_palette_set_color(0x0, 0x5, 0x0444);  /* menu frame */
    wwc_palette_set_color(0x0, 0x6, 0x02f2);  /* title */

    /* title screen */
    clear_screen();
    show_form(78, 35, solfege_form, 0x00);
    show_form(146, 45, quaver_noml_form, 0x06);
    show_string(61, 75, "press start button", 0x00);

    char_no = 0;
    for (j=0; j<18; j++)
        for (i=0; i<28; i++)
            screen_fill_char(SCREEN1, i, j, 1, 1, char_no++);

    wwc_font_set_colordata(0, 28*18, buffer);

    /* sound init */
    sound_init();
    sound_set_output(SSO_SPK | SSO_EXT | SSO_D29);
    sound_set_channel(SSC_CH1 | SSC_CH2 | SSC_CH3);
    sound_set_wave(CHANNEL1, wave_form);
    sound_set_wave(CHANNEL2, wave_form);
    sound_set_wave(CHANNEL3, wave_form);

    play_simple_sound(1830, 12);
    play_simple_sound_b(1911, 12);

    /* vbl interrupt set */
    intVector.callback = (void(near *)())FP_OFF(f_VBLINT_Handler);
    intVector.cs = _asm_inline("mov ax, cs");
    intVector.ds = _asm_inline("mov ax, ds");
    sys_interrupt_set_hook(SYS_INT_VBLANK, &intVector, NULL);

    /* wait start button pressed */
    while(1) {
#ifdef SHOT_ENABLE
       if ((key_press_check() & KEY_Y1) != 0) do_shot();
#endif
       if ((key_press_check() & KEY_START) != 0)
           break;
       if ((key_press_check() & KEY_A) != 0)
           break;
    }
    play_ok_button();

    /* draw initial score */
    show_all();
    set_message("welcome to solfege");

    /* wait key release */
    while((key_press_check() & KEY_A) != 0)
        {}
    while((key_press_check() & KEY_START) != 0)
        {}
    delay();

    while(1) {
        if((key_press_check() & KEY_A) != 0)
        {
            play_ok_button();
            set_message("");    /* clear message if any button pressed */
            show_message();
            if (t_or_b == TREBLE)
                handle_main_menu(score_t[selection_from].position + 20 - position_offset, 5);
            else
                handle_main_menu(score_b[selection_from].position + 20 - position_offset, 30);
        }
        if((key_press_check() & KEY_B) != 0)
        {
            set_message("");    /* clear message if any button pressed */
        }
        if((key_press_check() & KEY_X2) != 0)
        {
            set_message("");    /* clear message if any button pressed */
            if (t_or_b == TREBLE) change_selections_by_right_button_t();
            else change_selections_by_right_button_b();
            wwc_font_set_colordata(0, 28*18, buffer);
            invalidate = 0;

            /* key repeat */
            if (initial_press == 0) {
                repeat_times = 5000;
                initial_press = 1;
            } else {
                repeat_times = 500;
            }
            for (i=0; i<repeat_times; i++) {
                if ((key_press_check() & KEY_X2) == 0) break;
            }
        }
        if((key_press_check() & KEY_X4) != 0)
        {
            set_message("");    /* clear message if any button pressed */
            if (t_or_b == TREBLE) change_selections_by_left_button_t();
            else change_selections_by_left_button_b();
            wwc_font_set_colordata(0, 28*18, buffer);
            invalidate = 0;

            /* key repeat */
            if (initial_press == 0) {
                repeat_times = 5000;
                initial_press = 1;
            } else {
                repeat_times = 500;
            }
            for (i=0; i<repeat_times; i++) {
                if ((key_press_check() & KEY_X4) == 0) break;
            }
        }
        if((key_press_check() & KEY_X1) != 0)
        {
            set_message("");    /* clear message if any button pressed */
            if (t_or_b == BASS) change_selections_by_up_button();
        }
        if((key_press_check() & KEY_X3) != 0)
        {
            set_message("");    /* clear message if any button pressed */
            if (t_or_b == TREBLE) change_selections_by_down_button();
        }
        if((key_press_check() & KEY_Y2) != 0)
        {
            set_message("");    /* clear message if any button pressed */
            change_selections_by_R_button();

            /* key repeat */
            if (initial_press == 0) {
                repeat_times = 5000;
                initial_press = 1;
            } else {
                repeat_times = 500;
            }
            for (i=0; i<repeat_times; i++) {
                if ((key_press_check() & KEY_Y2) == 0) break;
            }
        }
        if((key_press_check() & KEY_Y4) != 0)
        {
            set_message("");    /* clear message if any button pressed */
            change_selections_by_L_button();

            /* key repeat */
            if (initial_press == 0) {
                repeat_times = 5000;
                initial_press = 1;
            } else {
                repeat_times = 500;
            }
            for (i=0; i<repeat_times; i++) {
                if ((key_press_check() & KEY_Y4) == 0) break;
            }
        }
        if((key_press_check() & KEY_START) != 0) {
            sound_set_volume(CHANNEL1, 0);
            sound_set_volume(CHANNEL2, 0);
            break;
        }

        /* cancel auto repeat */
        if (((key_press_check() & KEY_X2) == 0)
        && ((key_press_check() & KEY_X4) == 0)
        && ((key_press_check() & KEY_Y2) == 0)
        && ((key_press_check() & KEY_Y4) == 0))
            initial_press = 0;

        update_play_selection();
        if (show_msg != 0) show_message();
        if (invalidate != 0) {
            wwc_font_set_colordata(0, 28*18, buffer);
            invalidate = 0;
        }

        /* screen shot */
#ifdef SHOT_ENABLE
        if ((key_press_check() & KEY_Y1) != 0) do_shot();
#endif
    }
}

